unit Askme;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, Mask, ToolEdit, CurrEdit, Buttons, RXLookup,
  DB, DBTables, VCLUtils, RXSpin, QUICKRPT, qrextra, AppUtils, registry,
  URepTypes, vvMultiSelectCombo, ComCtrls;

type
  TParamAskMeInputTyp = (prmString, prmDate, prmMoney, prmDouble, prmInteger,
                         prmDBLookup, prmDBMultiSelect, prmDateTime, prmPassword);

  TParamAskMeValTyp = (valEmpty, valString, valDate, valMoney, valDouble, valInteger, valDateTime);

  TDBLookupAskMe = class
    DatabaseName: TFileName;
    sSQL : string;
    LookupField : string;
    LookupDisplay : string;
    LookupDisplayIndex : integer;
    Value : string;
    DisplayValue : string;
    isEmpty : Boolean;
    constructor Create;
  end;

  TParamAskMe = class
  private
    FChanged : Boolean;
    FValue : Variant;
    procedure FSetValue(val: Variant);
  public
    parName : string;
    inpTyp : TParamAskMeInputTyp;   //   
    valTyp: TParamAskMeValTyp; //   

    Width : Integer;
    slabel : string;

    DBLookupAskMe : TDBLookupAskMe;
    IdParam : integer;
    IdTypDefVal : integer;
    DefSQL : string;

    property Value: Variant read FValue write FSetValue;
    function ValueAsString: string;
    constructor Create(lbl:string; inpTp:TParamAskMeInputTyp; name:string);
    destructor Destroy; override;
  end;


  TParamAskMeList = class
  private
    FParamAskMeList : TList;
    function FGetCount: integer;
    function FGetItems(iIndex: integer) : TParamAskMe;
  public
    property Count: Integer read FGetCount;
    procedure Add(ParamAskMe: TParamAskMe);
    property Items[iIndex: integer] : TParamAskMe read FGetItems; default;
    function ParamByName(ParName: String) : TParamAskMe;

    constructor Create;
    constructor CreateParamList(IdCaller: integer; ItemType: TURepTreeItemType=ttForm);
    destructor Destroy; override;
  end;


  TQueryListItemType = (ttSQL, ttRemoteData);

  TQueryListItem = class
  public
    id : integer;
    name, description : string;
    isMaster: Boolean;
    QueryShortName : string;
    QueryType: TQueryListItemType;
    //ttSQL
    sqltext: string;
    //ttRemoteData
    IdRemtServerType: integer;
    ServerName: String;
    ServerGUID: String;
    ComputerName: String;
    IdRemtProtocol, Port: integer;
    function FindDataSet(aForm: TForm): TDataSet;
  end;

  TQueryList = class
  private
    FQueryList : TList;
    function FGetCount: integer;
    function FGetItems(iIndex: integer) : TQueryListItem;
  public
    property Count: Integer read FGetCount;
    procedure Add(QueryListItem: TQueryListItem);
    property Items[iIndex: integer] : TQueryListItem read FGetItems; default;
    constructor Create;
    destructor Destroy; override;
    function ItemByShortName(ShortName: String) : TQueryListItem;
  end;



  TfrmAskMe = class(TForm)
    BitBtnOK: TBitBtn;
    BitBtnCancel: TBitBtn;
    procedure BitBtnOKClick(Sender: TObject);
    procedure CompExit(Sender: TObject);
    procedure CompEnter(Sender: TObject);
    procedure CompChange(Sender: TObject);
    procedure RxSpinButtonTopClick(Sender: TObject);
    procedure RxSpinButtonBottomClick(Sender: TObject);
    procedure SelAllSpeedButtonClick(Sender: TObject);
  private
    aParamAskList : TParamAskMeList;
  public
    constructor Create(AOwner: TComponent; aParAskList : TParamAskMeList); reintroduce;
  end;

  function AskMeParams(Caption: string; aParAskList: TParamAskMeList) : Boolean;
  function cmpParList(parList1, parList2 : TParamAskMeList) : Boolean;
  function DataSetBindParamAskMe(aDataSet: TDataSet; aParAskList: TParamAskMeList) : Boolean;
  function getParamIdentName(aParamAskMe : TParamAskMe): string; // for save-load def value

var
  frmAskMe: TfrmAskMe;
  glDatabaseName : string;

implementation

{$R *.DFM}
{$R AskMe.RES}

uses fURepFunc1, URepConstants;

procedure ReadParamValue(aParamAskMe: TParamAskMe; frmAsk: TfrmAskMe); forward;
procedure WriteParamValue(aParamAskMe: TParamAskMe;aComponent: TComponent);forward;

const spbtnPrefix = 'sbtn__';
      labelPrefix = 'lbl';
      queryPrefix = 'query_';
      datasourcePrefix ='datasource_';
      cnstURepParSec = 'UREP_PARAMS_VALUE';
      DatePrefix = 'DatePart_';
      TimePrefix = 'TimePart_';

//------------------------------------------------------------------
// TParamAskMe
//------------------------------------------------------------------
constructor TDBLookupAskMe.Create;
begin
  DatabaseName := glDatabaseName;
  sSQL := '';
  LookupField := '';
  LookupDisplay := '';
  LookupDisplayIndex :=0;
  isEmpty := False;
end;


constructor TParamAskMe.Create(lbl:string; inpTp:TParamAskMeInputTyp; name:string);
begin
  {inherited Create;}
  parName := name;
  inpTyp := inpTp;
  slabel := lbl;
  Width := 0;
  FChanged := False;
  case inpTyp of
  prmString:   begin FValue := ''; valTyp := valString; end;
  prmDate:     begin FValue := Date; valTyp := valDate; end;
  prmMoney:    begin FValue := VarAsType(0.0, varDouble); valTyp := valMoney; end;
  prmDouble:   begin FValue := VarAsType(0.0, varDouble); valTyp := valDouble; end;
  prmInteger:  begin FValue := 0; valTyp := valInteger; end;
  prmDateTime: begin FValue := Date; valTyp := valDateTime; end;
  prmPassword: begin FValue := ''; valTyp := valString; end;
  end;
end;

destructor TParamAskMe.Destroy;
begin
  if DBLookupAskMe<>nil then
    DBLookupAskMe.Destroy;
  inherited Destroy;
end;

procedure TParamAskMe.FSetValue(val: Variant);
begin
  FValue := val;
end;

function TParamAskMe.ValueAsString: string;
begin
  case valTyp of
    valString :
      result := FValue ;
    valDate :
      result := DateToStr(FValue);
    valMoney :
      result := FormatFloat('#,#0.00',FValue);
    valDouble :
      result := FloatToStr(FValue);
    valInteger:
      result := IntToStr(FValue) ;
    valDateTime :
      result := DateTimeToStr(FValue);
  end;
end;

constructor TParamAskMeList.Create;
begin
  FParamAskMeList := TList.Create;
end;

constructor TParamAskMeList.CreateParamList(IdCaller: integer; ItemType: TURepTreeItemType=ttForm);
begin
  FParamAskMeList := TList.Create;
  URepInternalCreateParamList(IdCaller, ItemType, FParamAskMeList);
end;

destructor TParamAskMeList.Destroy;
var i: integer;
begin
  for i:=0 to FParamAskMeList.Count-1 do
    TParamAskMe(FParamAskMeList.Items[i]).Free;
  FParamAskMeList.Free;
  inherited Destroy;
end;

procedure TParamAskMeList.Add(ParamAskMe: TParamAskMe);
begin
  FParamAskMeList.Add(ParamAskMe);
end;

function TParamAskMeList.FGetCount: integer;
begin
  result := FParamAskMeList.Count;
end;

function TParamAskMeList.FGetItems(iIndex: integer) : TParamAskMe;
begin
  result := TParamAskMe(FParamAskMeList.Items[iIndex]);
end;

type EParamAskMeNotFound = class (Exception);

function TParamAskMeList.ParamByName(ParName: String) : TParamAskMe;
var
  i: integer;
  aParamAskMe: TParamAskMe;
begin
  //result := nil;
  for i:=0 to Count-1 do
  begin
    aParamAskMe := Items[i];
    if aParamAskMe.parName=ParName then
    begin
      result := aParamAskMe;
      Exit;
    end;
  end;
  raise EParamAskMeNotFound.Create(SInvalidParamAsk + '"'+ ParName + '"');
end;

//------------------------------------------------------------------
// TQueryList
//------------------------------------------------------------------

function TQueryListItem.FindDataSet(aForm: TForm): TDataSet;
begin
  result := TDataSet(aForm.FindComponent(QueryShortName));
end;


constructor TQueryList.Create;
begin
  FQueryList := TList.Create;
end;

destructor TQueryList.Destroy;
var i: integer;
begin
  for i:= 0 to FQueryList.Count-1 do
    TQueryListItem(FQueryList.Items[i]).Free;
  FQueryList.Free;
  inherited Destroy;
end;

procedure TQueryList.Add(QueryListItem: TQueryListItem);
begin
  FQueryList.Add(QueryListItem);
end;

function TQueryList.FGetCount: integer;
begin
  result := FQueryList.Count;
end;

function TQueryList.FGetItems(iIndex: integer) : TQueryListItem;
begin
  result := TQueryListItem(FQueryList.Items[iIndex]);
end;

function TQueryList.ItemByShortName(ShortName: String) : TQueryListItem;
var
  i: integer;
  QueryListItem: TQueryListItem;
begin
  for i:=0 to Count-1 do
  begin
    QueryListItem := Items[i];
    if QueryListItem.QueryShortName=ShortName then
    begin
      result := QueryListItem;
      Exit;
    end;
  end;
  raise Exception.Create(SInvalidQueryListItem + '"'+ ShortName + '"');
end;

//------------------------------------------------------------------
// frmAskMe common functions
//------------------------------------------------------------------
// Default begin value for dbLookupCombo
procedure dbLookupComboSetDefVal(dbLookupCombo: TRxDBLookupCombo; aParamAskMe: TParamAskMe);
begin
  if aParamAskMe.IdTypDefVal<>0 then
    if aParamAskMe.DBLookupAskMe.Value<>'' then
      try
      dbLookupCombo.LookupSource.DataSet.Locate(dbLookupCombo.LookupField, aParamAskMe.DBLookupAskMe.Value, []);
      except
      end;
  dbLookupCombo.Value :=
    dbLookupCombo.LookupSource.DataSet.FieldByName(dbLookupCombo.LookupField).AsString;
end;


function DLookupParam(SQLstring:String; aParAskList: TParamAskMeList):Variant;
var
  Query : TQuery;
begin
   Query := TQuery.Create(nil);
   try
    with Query do
    begin
     DatabaseName := glDatabaseName;
     SQL.Add(SQLstring);
     DataSetBindParamAskMe( Query, aParAskList );
     Open;
     result:=Fields[0].AsVariant;
     Close;
    end;
   finally
    Query.Free;
   end;
end;

// defSQL      
function isDefSQLParams(curParam: TParamAskMe; aParAskList: TParamAskMeList): Boolean;
begin
  result := False;
  if curParam.IdTypDefVal=2 then // def SQL
    begin
    result := True;
    end;
end;

//   
procedure SetParamDefValue(aParamAskMe: TParamAskMe; aParAskList: TParamAskMeList);
var
  reginiFile : TRegistryIniFile;
begin
  reginiFile := TRegistryIniFile.Create(GetDefaultIniRegKey);
  // Default begin value = last values
  if aParamAskMe.IdTypDefVal=1 then
    begin
    case aParamAskMe.inpTyp of
      prmString:
        aParamAskMe.Value := reginiFile.ReadString(cnstURepParSec, getParamIdentName(aParamAskMe), aParamAskMe.Value);
      prmDate:
        aParamAskMe.Value := reginiFile.ReadDate(cnstURepParSec, getParamIdentName(aParamAskMe), aParamAskMe.Value);
      prmMoney, prmDouble:
        aParamAskMe.Value := reginiFile.ReadFloat(cnstURepParSec, getParamIdentName(aParamAskMe), aParamAskMe.Value);
      prmInteger :
        aParamAskMe.Value := reginiFile.ReadInteger(cnstURepParSec, getParamIdentName(aParamAskMe), aParamAskMe.Value);
      prmDBLookup:
        aParamAskMe.DBLookupAskMe.Value := reginiFile.ReadString(cnstURepParSec, getParamIdentName(aParamAskMe), aParamAskMe.DBLookupAskMe.Value);
      prmDateTime:
        aParamAskMe.Value := reginiFile.ReadDateTime(cnstURepParSec, getParamIdentName(aParamAskMe), aParamAskMe.Value);
      end;
    end;
  // Default begin value = SQL
  if aParamAskMe.IdTypDefVal=2 then
    begin
    if aParamAskMe.inpTyp=prmDBLookup then
      aParamAskMe.DBLookupAskMe.Value := DLookupParam(aParamAskMe.DefSQL, aParAskList)
    else
      aParamAskMe.Value := DLookupParam(aParamAskMe.DefSQL, aParAskList);
    end;
  reginiFile.Free;
end;

// Insert Items in MultiSelectCombo
procedure FillMultiSelectCombo(vvMultiSelectCombo: TvvMultiSelectCombo;
                               aParamAskMe: TParamAskMe; aQuery: TQuery);
var
  fldLookupField, fldLookupDisplay: TField;
begin
  fldLookupField := aQuery.FieldByName(aParamAskMe.DBLookupAskMe.LookupField);
  fldLookupDisplay := aQuery.FieldByName(aParamAskMe.DBLookupAskMe.LookupDisplay);
  vvMultiSelectCombo.Items.BeginUpdate;
  vvMultiSelectCombo.Values.BeginUpdate;
  vvMultiSelectCombo.Items.Clear;
  vvMultiSelectCombo.Values.Clear;
  aQuery.First;
  while not aQuery.EOF do
  begin
    vvMultiSelectCombo.Items.Add(fldLookupDisplay.AsString);
    vvMultiSelectCombo.Values.Add(fldLookupField.AsString);
    aQuery.Next;
  end;
  vvMultiSelectCombo.Items.EndUpdate;
  vvMultiSelectCombo.Values.EndUpdate;
  if not aParamAskMe.DBLookupAskMe.isEmpty then
    vvMultiSelectCombo.CheckAll;
end;

// check param changed
function isDBLookupParamChanged(aQuery: TQuery; aParamAskList: TParamAskMeList): Boolean;
var
  i: integer;
  aParamAskMe : TParamAskMe;
begin
  result := False;
  if aQuery=nil then Exit;
  for i:=0 to aQuery.Params.Count-1 do
  begin
    try
      aParamAskMe := aParamAskList.ParamByName(aQuery.Params[i].Name);
      if aParamAskMe.FChanged then
      begin
        result := True;
        Exit;
      end;
    except
    end;
  end;
end;

// reopen query with params
function ReQueryWithParams(aQuery: TQuery; aParamAskList: TParamAskMeList): Boolean;
var
  curParName: string;
  curParamAskMe : TParamAskMe;
begin
  curParName := Copy(aQuery.Name, Length(queryPrefix)+1, Length(aQuery.Name));
  curParamAskMe := aParamAskList.ParamByName(curParName);
  with aQuery do
    begin
    Close;
    SQL.Text := curParamAskMe.DBLookupAskMe.sSQL;
    result := DataSetBindParamAskMe(aQuery, aParamAskList);
    Open;
    FetchAll;
    First;
    end;
end;

// defSQL      () 
function isDefSQLParamChanged(curParam: TParamAskMe; aParAskList: TParamAskMeList): Boolean;
begin
  result := True;
end;

//------------------------------------------------------------------
// TfrmAskMe
//------------------------------------------------------------------
var btmpBtnSelAll: TBitmap;

constructor TfrmAskMe.Create(AOwner: TComponent; aParAskList : TParamAskMeList);
var
  i, iCmpWidth, maxCmpWidth, iMaxLabelLen, iTabOrder : integer;
  aParamAskMe : TParamAskMe;
  aLabel : TLabel;
  aEdit : TEdit;
  aDateEdit: TDateEdit;
  currEdit : TCurrencyEdit;
  aRxSpinButton : TRxSpinButton;
  aSpeedButton : TSpeedButton;

  aDataSource: TDataSource;
  aQuery: TQuery;
  dbLookupCombo: TRxDBLookupCombo;
  vvMultiSelectCombo : TvvMultiSelectCombo;
  aDateTimePicker: TDateTimePicker;
  bgetParam : Boolean;
begin
  inherited Create(AOwner);
  aParamAskList := aParAskList;
  maxCmpWidth := 0;

  // calc iMaxLabelLen
  iMaxLabelLen := 0;
  for i:=0 to aParAskList.Count-1 do
  begin
    aParamAskMe := aParAskList.Items[i];
    iCmpWidth := Canvas.TextWidth(aParamAskMe.slabel);
    if iCmpWidth > iMaxLabelLen then
      iMaxLabelLen := iCmpWidth;
  end;
  if iMaxLabelLen<100 then iMaxLabelLen:=100;
  iMaxLabelLen := iMaxLabelLen + 10;

  iTabOrder := 0;
  for i:=0 to aParAskList.Count-1 do
  begin
    aParamAskMe := aParAskList.Items[i];
    if aParamAskMe.Width=0 then
      iCmpWidth := 200
    else
      iCmpWidth := aParamAskMe.Width;

    SetParamDefValue(aParamAskMe, aParAskList); //  

    aLabel := TLabel.Create(Self);
    aLabel.Parent := Self;
    aLabel.Name := labelPrefix + aParamAskMe.parName;
    aLabel.Top := (30 * i)+10;
    aLabel.Left := 5;
    aLabel.Width := iMaxLabelLen;
    aLabel.Alignment := taRightJustify;
    aLabel.Caption := aParamAskMe.slabel;


    case aParamAskMe.inpTyp of
    prmString, prmPassword :
      begin
      aEdit := TEdit.Create(Self);
      with aEdit do
        begin
        Parent := Self;
        Name := aParamAskMe.parName;
        Tag := i;
        Top := (30 * i)+10;
        Left := iMaxLabelLen + 10;
        Width := iCmpWidth;
        TabOrder := iTabOrder;
        Text := aParamAskMe.Value;
        if aParamAskMe.inpTyp=prmPassword then
          PasswordChar := '*';
        OnExit := CompExit;
        if isDefSQLParams(aParamAskMe, aParAskList) then
          OnEnter := CompEnter;
        end;
      end;
    prmDate :
      begin
      if aParamAskMe.Width=0 then
        iCmpWidth := 150;
      aDateEdit := TDateEdit.Create(Self);
      with aDateEdit do
        begin
        Parent := Self;
        Name := aParamAskMe.parName;
        Tag := i;
        Top := (30 * i)+10;
        Left := iMaxLabelLen + 10;
        Width := iCmpWidth;
        TabOrder := iTabOrder;
        Date := aParamAskMe.Value;
        OnExit := CompExit;
        if isDefSQLParams(aParamAskMe, aParAskList) then
          OnEnter := CompEnter;
        end;
      aRxSpinButton := TRxSpinButton.Create(Self);
      with aRxSpinButton do
        begin
        Parent := Self;
        FocusControl := aDateEdit;
        Name := spbtnPrefix + aParamAskMe.parName;
        Tag := -1;
        Top := (30 * i)+10;
        Height := aDateEdit.Height-1;
        Width := Height;
        Left := iMaxLabelLen + 10 + aDateEdit.Width;
        OnTopClick := RxSpinButtonTopClick;
        OnBottomClick := RxSpinButtonBottomClick;
        Hint := AskmeHintSpinDate;
        iCmpWidth := iCmpWidth + Width;
        end;
      end;
    prmMoney :
      begin
      currEdit := TCurrencyEdit.Create(Self);
      with currEdit do
        begin
        Parent := Self;
        Name := aParamAskMe.parName;
        Tag := i;
        Top := (30 * i)+10;
        Left := iMaxLabelLen + 10;
        Width := iCmpWidth;
        TabOrder := iTabOrder;
        DisplayFormat := ',0.00;-,0.00';
        DecimalPlaces := 2;
        Value := aParamAskMe.Value;
        OnExit := CompExit;
        if isDefSQLParams(aParamAskMe, aParAskList) then
          OnEnter := CompEnter;
        end;
      end;
    prmDouble :
      begin
      currEdit := TCurrencyEdit.Create(Self);
      with currEdit do
        begin
        Parent := Self;
        Name := aParamAskMe.parName;
        Tag := i;
        Top := (30 * i)+10;
        Left := iMaxLabelLen + 10;
        Width := iCmpWidth;
        TabOrder := iTabOrder;
        DisplayFormat := ',0.######;-,0.######';
        DecimalPlaces := 6;
        Value := aParamAskMe.Value;
        OnExit := CompExit;
        if isDefSQLParams(aParamAskMe, aParAskList) then
          OnEnter := CompEnter;
        end;
      end;
    prmInteger :
      begin
      currEdit := TCurrencyEdit.Create(Self);
      with currEdit do
        begin
        Parent := Self;
        Name := aParamAskMe.parName;
        Tag := i;
        Top := (30 * i)+10;
        Left := iMaxLabelLen + 10;
        Width := iCmpWidth;
        TabOrder := iTabOrder;
        DisplayFormat := ',0;-,0';
        DecimalPlaces := 0;
        Value := aParamAskMe.Value;
        OnExit := CompExit;
        if isDefSQLParams(aParamAskMe, aParAskList) then
          OnEnter := CompEnter;
        end;
      end;
    prmDBLookup :
      begin  // IsDbLookup
      if aParamAskMe.DBLookupAskMe<>nil then
        begin

        aQuery := TQuery.Create(Self);
        with aQuery do
          begin
          Name := queryPrefix+aParamAskMe.parName;
          DatabaseName:= aParamAskMe.DBLookupAskMe.DatabaseName;
          Tag := -1;
          end;
        bgetParam := ReQueryWithParams(aQuery, aParAskList);

        aDataSource := TDataSource.Create(Self);
        with aDataSource do
          begin
          Name := datasourcePrefix+aParamAskMe.parName;
          DataSet := aQuery;
          Tag := -1;
          end;

        dbLookupCombo := TRxDBLookupCombo.Create(Self);
        with dbLookupCombo do
          begin
          Parent := Self;
          Name := aParamAskMe.parName;
          Tag := i;
          Top := (30 * i)+10;
          Left := iMaxLabelLen + 10;
          Width := iCmpWidth;
          TabOrder := iTabOrder;
          LookupSource := aDataSource;
          LookupField := aParamAskMe.DBLookupAskMe.LookupField;
          LookupDisplay := aParamAskMe.DBLookupAskMe.LookupDisplay;
          If aParamAskMe.DBLookupAskMe.LookupDisplayIndex>=0 Then
            LookupDisplayIndex := aParamAskMe.DBLookupAskMe.LookupDisplayIndex
          Else
            DisplayAllFields := True;
          if not aParamAskMe.DBLookupAskMe.isEmpty then
            begin
            EscapeClear := False;
            dbLookupComboSetDefVal(dbLookupCombo, aParamAskMe);
            end
          else
            begin
            DisplayEmpty := '<>';
            EscapeClear := True;
            end;
          OnExit := CompExit;   //8.02.1999
          OnChange := CompChange;
          if bgetParam = True then
            OnEnter := CompEnter;
          end;
        ReadParamValue(aParamAskMe, Self);
        end;
      end;

    prmDBMultiSelect :
      begin  // IsDbLookup
      if aParamAskMe.DBLookupAskMe<>nil then
       begin
        aQuery := TQuery.Create(Self);
        with aQuery do
         begin
          Name := 'query_'+aParamAskMe.parName;
          DatabaseName:= aParamAskMe.DBLookupAskMe.DatabaseName;
          Tag := -1;
         end;
        bgetParam := ReQueryWithParams(aQuery, aParAskList);

        vvMultiSelectCombo := TvvMultiSelectCombo.Create(Self);
        with vvMultiSelectCombo do
         begin
          Parent := Self;
          Name := aParamAskMe.parName;
          Tag := i;
          GlyphKind := gkDropDown;
          Top := (30 * i)+10;
          Left := iMaxLabelLen + 10;
          Width := iCmpWidth;
          TabOrder := iTabOrder;

          FillMultiSelectCombo(vvMultiSelectCombo, aParamAskMe, aQuery);

          OnExit := CompExit;   //8.02.1999
          OnChange := CompChange;
          if bgetParam = True then  OnEnter := CompEnter;
         end;
        ReadParamValue(aParamAskMe, Self);

        aSpeedButton := TSpeedButton.Create(Self);
        with aSpeedButton do
         begin
          Parent := Self;
          Height := vvMultiSelectCombo.Height-2;
          Width := Height;
          Name := spbtnPrefix + aParamAskMe.parName;
          Tag := -1;
          Top := (30 * i)+10+1;
          Left := iMaxLabelLen + 10 + vvMultiSelectCombo.Width;
          OnClick := SelAllSpeedButtonClick;
          Hint := AskmeHintSelectAll;
          iCmpWidth := iCmpWidth + Width;
         end;

         if btmpBtnSelAll=nil then
          begin
           btmpBtnSelAll := TBitmap.Create;
           btmpBtnSelAll.LoadFromResourceName(HInstance, 'BTNSELECTALL');
          end;
         aSpeedButton.Glyph.Assign(btmpBtnSelAll);

        end;
      end;

    prmDateTime :
      begin
      if aParamAskMe.Width=0 then
        iCmpWidth := 120;
      aDateEdit := TDateEdit.Create(Self);
      with aDateEdit do
        begin
        Parent := Self;
        Name := DatePrefix + aParamAskMe.parName;
        Tag := i;
        Top := (30 * i)+10;
        Left := iMaxLabelLen + 10;
        Width := iCmpWidth;
        TabOrder := iTabOrder;
        Date := Int(aParamAskMe.Value);
        OnExit := CompExit;
        //if isDefSQLParams(aParamAskMe, aParAskList) then
        //  OnEnter := CompEnter;
        end;

      aRxSpinButton := TRxSpinButton.Create(Self);
      with aRxSpinButton do
        begin
        Parent := Self;
        FocusControl := aDateEdit;
        Name := spbtnPrefix + DatePrefix + aParamAskMe.parName;
        Tag := -1;
        Top := (30 * i)+10;
        Height := aDateEdit.Height-1;
        Width := Height;
        Left := iMaxLabelLen + 10 + aDateEdit.Width;
        OnTopClick := RxSpinButtonTopClick;
        OnBottomClick := RxSpinButtonBottomClick;
        Hint := AskmeHintSpinDate;
        end;

      Inc(iTabOrder);
      aDateTimePicker := TDateTimePicker.Create(Self);
      with aDateTimePicker do
        begin
        Parent := Self;
        Name := TimePrefix + aParamAskMe.parName;
        Tag := i;
        Top := (30 * i)+10;
        Left := aRxSpinButton.Left + aRxSpinButton.Width + 10;
        Width := aDateEdit.Width - aDateEdit.Height;
        TabOrder := iTabOrder;
        Kind := dtkTime;
        Time := Frac(aParamAskMe.Value);
        OnExit := CompExit;
        //if isDefSQLParams(aParamAskMe, aParAskList) then
        //  OnEnter := CompEnter;
        end;

      iCmpWidth := iCmpWidth + aRxSpinButton.Width + 10 + aDateTimePicker.Width;
      end;
    end;
    Inc(iTabOrder);
    if maxCmpWidth < iCmpWidth then
      maxCmpWidth := iCmpWidth;
  end;

  Self.ClientHeight := (aParAskList.Count*30) + BitBtnOK.Height + 20;
  Self.Width := iMaxLabelLen + 30 + maxCmpWidth;

  BitBtnOK.Top := (aParAskList.Count*30) + 10;
  BitBtnCancel.Top := (aParAskList.Count*30) + 10;

  BitBtnOK.Left := (Self.Width div 2) - 10 - BitBtnOK.Width;
  BitBtnCancel.Left := (Self.Width div 2) + 10;
end;


procedure ReadParamValue(aParamAskMe: TParamAskMe; frmAsk: TfrmAskMe);
var
  aComponent: TComponent;
  aField : TField;
  //sFldNm : string;
  rxDBLookupCombo : TRxDBLookupCombo;
  vvMultiSelectCombo : TvvMultiSelectCombo;
  iVal : integer;
  dValue, tmValue, dtmValue: TDateTime;
begin
  aComponent := nil;
  if aParamAskMe.inpTyp<>prmDateTime Then
    begin
    aComponent := frmAsk.FindComponent(aParamAskMe.parName);
    if (aComponent=nil) then
        raise Exception.Create('   '+aParamAskMe.parName);
    end;

  case aParamAskMe.inpTyp of
  prmString, prmPassword :
    begin
    aParamAskMe.FChanged := not (aParamAskMe.Value = (aComponent as TEdit).Text);
    aParamAskMe.Value := (aComponent as TEdit).Text;
    end;
  prmDate :
    begin
    aParamAskMe.FChanged := not (aParamAskMe.Value = (aComponent as TDateEdit).Date);
    aParamAskMe.Value := (aComponent as TDateEdit).Date;
    end;
  prmMoney, prmDouble :
    begin
    aParamAskMe.FChanged := not (aParamAskMe.Value = (aComponent as TCurrencyEdit).Value);
    aParamAskMe.Value := (aComponent as TCurrencyEdit).Value;
    end;
  prmInteger:
    begin
    iVal := Round((aComponent as TCurrencyEdit).Value);
    aParamAskMe.FChanged := not (aParamAskMe.Value = iVal);
    aParamAskMe.Value := iVal;
    end;
  prmDBLookup:
    begin
    rxDBLookupCombo := aComponent as TRxDBLookupCombo;
    aField := rxDBLookupCombo.LookupSource.DataSet.FieldByName(rxDBLookupCombo.LookupField);
    case aField.DataType of  // set val typ
      ftString :   aParamAskMe.valTyp := valString;
      ftSmallint,
      ftInteger,
      ftAutoInc :  aParamAskMe.valTyp := valInteger;
      ftFloat,
      ftCurrency,
      ftBCD :      aParamAskMe.valTyp := valDouble;
      ftDate ,
      ftDateTime : aParamAskMe.valTyp := valDate;
      end;
    aParamAskMe.FChanged := (VarType(aParamAskMe.Value)=varEmpty) or
                            (not (aParamAskMe.Value = aField.Value));
    aParamAskMe.Value := aField.Value;  // m.b. not string
    aParamAskMe.DBLookupAskMe.Value := rxDBLookupCombo.Value;  // string
    aParamAskMe.DBLookupAskMe.DisplayValue := rxDBLookupCombo.DisplayValue;
    end;
  prmDBMultiSelect:
    begin
    vvMultiSelectCombo := aComponent as TvvMultiSelectCombo;
    aParamAskMe.valTyp := valString;
    aParamAskMe.FChanged := (VarType(aParamAskMe.Value)=varEmpty) or
                            (not (aParamAskMe.Value = vvMultiSelectCombo.Value));
    aParamAskMe.Value := vvMultiSelectCombo.Value;
    aParamAskMe.DBLookupAskMe.Value := vvMultiSelectCombo.Value;
    aParamAskMe.DBLookupAskMe.DisplayValue:= vvMultiSelectCombo.Text;
    end;
  prmDateTime :
    begin
    aComponent := frmAsk.FindComponent(DatePrefix + aParamAskMe.parName);
    dValue := (aComponent as TDateEdit).Date;
    aComponent := frmAsk.FindComponent(TimePrefix + aParamAskMe.parName);
    tmValue := (aComponent as TDateTimePicker).Time;
    dtmValue := Int(dValue) + Frac(tmValue);
    aParamAskMe.FChanged := not (aParamAskMe.Value = dtmValue);
    aParamAskMe.Value := dtmValue
    end;
  end;
{$ifdef DEMO_VER}
  if (aParamAskMe.inpTyp=prmDate) and
     (not isAcceptDemoData(aParamAskMe.Value)) then
       (aComponent as TWinControl).SetFocus;
{$endif}
end;

procedure WriteParamValue(aParamAskMe: TParamAskMe;aComponent: TComponent);
//var
  //aField : TField;
  //sFldNm : string;
//  rxDBLookupCombo : TRxDBLookupCombo;
begin
{if aParamAskMe.FIsDbLookup then
  begin
//  rxDBLookupCombo := aComponent as TRxDBLookupCombo;
{  sFldNm := rxDBLookupCombo.LookupField;
  aField := rxDBLookupCombo.LookupSource.DataSet.FieldByName(sFldNm);
  case aField.DataType of
    ftString :
        begin
        aParamAskMe.Typ := prmString;
        aParamAskMe.FChanged := not (aParamAskMe.sValue = aField.AsString);
        if (aParamAskMe.DBLookupAskMe.isEmpty) and
           (rxDBLookupCombo.Value='') then
          aParamAskMe.sValue := ''
        else
          aParamAskMe.sValue := aField.AsString;
        end;
    ftSmallint, ftInteger, ftAutoInc :
        begin
        aParamAskMe.Typ := prmInteger;
        aParamAskMe.FChanged := not (aParamAskMe.iValue = aField.AsInteger);
        aParamAskMe.iValue := aField.AsInteger;
        end;
    ftFloat, ftCurrency, ftBCD :
        begin
        aParamAskMe.Typ := prmDouble;
        aParamAskMe.FChanged := not (aParamAskMe.fValue = aField.AsFloat);
        aParamAskMe.fValue := aField.AsFloat;
        end;
    ftDate , ftDateTime :
        begin
        aParamAskMe.Typ := prmDate;
        aParamAskMe.FChanged := not (aParamAskMe.dValue = aField.AsDateTime);
        aParamAskMe.dValue := aField.AsDateTime;
        end;
    end;
  aParamAskMe.DBLookupAskMe.Value:=
                     (aComponent as TRxDBLookupCombo).Value;
  aParamAskMe.DBLookupAskMe.DisplayValue:=
                     (aComponent as TRxDBLookupCombo).DisplayValue;
  end}
    case aParamAskMe.inpTyp of
    prmString, prmPassword :
      begin
      (aComponent as TEdit).Text := aParamAskMe.Value ;
      end;
    prmDate :
      begin
      (aComponent as TDateEdit).Date := aParamAskMe.Value ;
      end;
    prmMoney, prmDouble :
      begin
      (aComponent as TCurrencyEdit).Value := aParamAskMe.Value;
      end;
    prmInteger:
      begin
      (aComponent as TCurrencyEdit).Value := aParamAskMe.Value;
      end;
    end;
end;


function getParamIdentName(aParamAskMe : TParamAskMe): string;
begin
  result := IntToStr(aParamAskMe.IdParam) +
            aParamAskMe.parName +
            IntToStr(Integer(aParamAskMe.inpTyp))
end;

procedure TfrmAskMe.BitBtnOKClick(Sender: TObject);
var
  i : integer;
  aParamAskMe : TParamAskMe;
  reginiFile : TRegistryIniFile;
begin
  reginiFile := TRegistryIniFile.Create(GetDefaultIniRegKey);
  try
    for i:=0 to aParamAskList.Count-1 do
      begin
      aParamAskMe := aParamAskList.Items[i];

      ReadParamValue(aParamAskMe, Self);
      // save values
      if aParamAskMe.IdTypDefVal=1 then
        begin
          case aParamAskMe.inpTyp of
            prmString:
              reginiFile.WriteString(cnstURepParSec,
                getParamIdentName(aParamAskMe), aParamAskMe.Value);
            prmDate:
              reginiFile.WriteDate(cnstURepParSec,
                getParamIdentName(aParamAskMe), aParamAskMe.Value);
            prmMoney, prmDouble:
              reginiFile.WriteFloat(cnstURepParSec,
                getParamIdentName(aParamAskMe), aParamAskMe.Value);
            prmInteger :
              reginiFile.WriteInteger(cnstURepParSec,
                getParamIdentName(aParamAskMe), aParamAskMe.Value);
            prmDBLookup, prmDBMultiSelect:
              reginiFile.WriteString(cnstURepParSec,
                getParamIdentName(aParamAskMe), aParamAskMe.DBLookupAskMe.Value);
          end;
        end;
      end;
  finally
    reginiFile.Free;
    end;
end;


function AskMeParams(Caption:string; aParAskList: TParamAskMeList) : Boolean;
begin
  AskMeParams := False;
  frmAskMe := TfrmAskMe.Create(Application, aParAskList);
  frmAskMe.Caption := Caption;
  if frmAskMe.ShowModal=mrOK then
    AskMeParams := True;
  frmAskMe.Release;
end;


function DataSetBindParamAskMe(aDataSet: TDataSet; aParAskList: TParamAskMeList) : Boolean;
const
  MACRO_ATTRIBUTE='%%';
var
  i : integer;
  bFind : Boolean;
  aParamAskMe : TParamAskMe;
  aParams: TParams;
  qQuery: TQuery;

  str,mcr, SQLstr :string;
  function IsFoundMacros(out mcr:string):boolean;
  var position:Integer;
  begin
    if str='' then str:=qQuery.SQL.Text;
    mcr:='';
    position:=pos(MACRO_ATTRIBUTE,str);
    result:=(position>0);
    if result then
    begin
      str:=copy(str,position+Length(MACRO_ATTRIBUTE),Length(str));
      position:=pos(MACRO_ATTRIBUTE,str)-1;
      if position>0 then
      begin
        mcr:=copy(str,1,position);
        str:=copy(str,position+Length(MACRO_ATTRIBUTE),Length(str));
      end;
      if (trim(mcr)='') or (trim(mcr)<>mcr) then
        raise Exception.Create('Error of syntax of macro');
    end;
  end;

begin
  result := True;
  qQuery := aDataSet as TQuery;

  // find macros
  SQLstr := qQuery.SQL.Text;
  while IsFoundMacros(mcr) do
  begin
    try
      aParamAskMe := aParAskList.ParamByName(mcr);
      case aParamAskMe.valTyp of
        valString, valInteger:
        SQLstr := StringReplace(SQLstr,
                                MACRO_ATTRIBUTE+mcr+MACRO_ATTRIBUTE,
                                aParamAskMe.Value,
                                [rfReplaceAll, rfIgnoreCase]);
      end;
    except // param not found
    end;
  end;
  qQuery.SQL.Text := SQLstr;

  //
  qQuery.Prepare;
  aParams := qQuery.Params;
  for i:=0 to aParams.Count-1 do
  begin
    bFind:=False;
    try
      aParamAskMe := aParAskList.ParamByName(aParams[i].Name);
      case aParamAskMe.valTyp of
        valString :
          aParams[i].AsString:=aParamAskMe.Value;
        valDate :
          aParams[i].AsDate:=aParamAskMe.Value;
        valMoney :
          aParams[i].AsFloat:=aParamAskMe.Value;
        valDouble :
          aParams[i].AsFloat:=aParamAskMe.Value;
        valInteger:
          aParams[i].AsInteger:=aParamAskMe.Value;
        valDateTime :
          aParams[i].AsDateTime:=aParamAskMe.Value;
      end;
      bFind:=True;
    except // param not found
    end;

    if not bFind then result := False;
  end;
end;


procedure TfrmAskMe.CompExit(Sender: TObject);
begin
  ReadParamValue(aParamAskList.Items[(Sender as TComponent).Tag], Self);
end;



procedure TfrmAskMe.CompEnter(Sender: TObject);
var
  aParamAskMe : TParamAskMe;
  aQuery : TQuery;
  dbLookupCombo : TRxDBLookupCombo;
begin
  aParamAskMe := aParamAskList.ParamByName((Sender as TComponent).Name);

  if aParamAskMe.inpTyp=prmDBLookup then
    begin  // dbLookup
    dbLookupCombo := Sender as TRxDBLookupCombo;
    aQuery := TQuery(dbLookupCombo.LookupSource.DataSet);

    if isDBLookupParamChanged(aQuery, aParamAskList) then
      begin
      ReQueryWithParams(aQuery, aParamAskList);
      if not aParamAskMe.DBLookupAskMe.isEmpty then
        begin
        dbLookupCombo.EscapeClear := False;
        dbLookupComboSetDefVal(dbLookupCombo, aParamAskMe); // beg val
        end
      else
        begin
        dbLookupCombo.EscapeClear := True;
        dbLookupCombo.DisplayEmpty := '<>';
        end;
      end;
    end
  else if aParamAskMe.inpTyp in [prmString, prmDate, prmMoney, prmDouble, prmInteger] then
    begin   // simle component
    SetParamDefValue(aParamAskMe, aParamAskList);
    WriteParamValue(aParamAskMe, Sender as TComponent);
    end;
end;

procedure TfrmAskMe.CompChange(Sender: TObject);
var
  i : integer;
  aComponent, compDefSQL : TComponent;
  aParamAskMe : TParamAskMe;
  LkUpCombo : TRxDBLookupCombo;
  vvMultiSelectCombo: TvvMultiSelectCombo;
  aQuery : TQuery;
  bNextComps: Boolean;
begin
  aComponent := Sender as TComponent;
  bNextComps := False;

  for i:=0 to aParamAskList.Count-1 do
    begin
    aParamAskMe := aParamAskList.Items[i];

    if bNextComps {and (aParamAskMe.inpTyp in [prmDBLookup,prmDBMultiSelect])} then //ReOpen
    begin
      aQuery := FindComponent(queryPrefix+aParamAskMe.parName) as TQuery;

      //      
      if isDBLookupParamChanged(aQuery, aParamAskList) then
      begin
        ReQueryWithParams(aQuery, aParamAskList);
        if (aParamAskMe.inpTyp=prmDBLookup) then //ReOpen DBLookup
        begin
          LkUpCombo := FindComponent(aParamAskMe.parName) as TRxDBLookupCombo;
          if not aParamAskMe.DBLookupAskMe.isEmpty then
            dbLookupComboSetDefVal(LkUpCombo, aParamAskMe);
        end;
        if (aParamAskMe.inpTyp=prmDBMultiSelect) then //ReOpen DBMultiSelect
        begin
          vvMultiSelectCombo := FindComponent(aParamAskMe.parName) as TvvMultiSelectCombo;
          FillMultiSelectCombo(vvMultiSelectCombo, aParamAskMe, aQuery);
        end;
      end;

      //       
      if isDefSQLParams(aParamAskMe,  aParamAskList) Then
      begin
        if isDefSQLParamChanged(aParamAskMe, aParamAskList) then
        begin
          if (aParamAskMe.inpTyp=prmDBLookup) then
          begin
            SetParamDefValue(aParamAskMe, aParamAskList);
            LkUpCombo := FindComponent(aParamAskMe.parName) as TRxDBLookupCombo;
              if not aParamAskMe.DBLookupAskMe.isEmpty then
                dbLookupComboSetDefVal(LkUpCombo, aParamAskMe);
          end
          else if (aParamAskMe.inpTyp=prmDBLookup) then
          begin
            //   
          end
          else
          begin
            compDefSQL := FindComponent(aParamAskMe.parName);
            SetParamDefValue(aParamAskMe, aParamAskList);
            WriteParamValue(aParamAskMe, compDefSQL);
          end;
        end;
      end;

    end;

    if aParamAskMe.parName=aComponent.Name then
      begin
      ReadParamValue(aParamAskMe, Self); //!!!!
      bNextComps := True;
      end;
    end;
end;

procedure TfrmAskMe.RxSpinButtonTopClick(Sender: TObject);
var
  aDateEdit: TDateEdit;
  sNm : string;
begin
  sNm := TComponent(Sender).Name;
  sNm := Copy(sNm, Length(spbtnPrefix)+1, Length(sNm));
  aDateEdit := TDateEdit(FindComponent(sNm));
  aDateEdit.Date := aDateEdit.Date + 1;
  CompChange(aDateEdit);
end;

procedure TfrmAskMe.RxSpinButtonBottomClick(Sender: TObject);
var
  aDateEdit: TDateEdit;
  sNm : string;
begin
  sNm := TComponent(Sender).Name;
  sNm := Copy(sNm, Length(spbtnPrefix)+1, Length(sNm));
  aDateEdit := TDateEdit(FindComponent(sNm));
  aDateEdit.Date := aDateEdit.Date - 1;
  CompChange(aDateEdit);
end;


procedure TfrmAskMe.SelAllSpeedButtonClick(Sender: TObject);
var
  vvMultiSelectCombo: TvvMultiSelectCombo;
  sNm : string;
begin
  sNm := TComponent(Sender).Name;
  sNm := Copy(sNm, Length(spbtnPrefix)+1, Length(sNm));
  vvMultiSelectCombo:= TvvMultiSelectCombo(FindComponent(sNm));
  vvMultiSelectCombo.CheckAll;
  CompChange(vvMultiSelectCombo);
end;


function cmpParList(parList1, parList2 : TParamAskMeList) : Boolean;
var
 i : integer;
 parAskMe1, parAskMe2 : TParamAskMe;
begin
  result := False;
  if parList1.Count <> parList2.Count then Exit;
  for i :=0 to (parList1.Count-1) do
    begin
    parAskMe1 := parList1.Items[i];
    parAskMe2 := parList2.Items[i];
    if (parAskMe1.parName <> parAskMe1.parName) or
       (parAskMe1.inpTyp <> parAskMe1.inpTyp) then Exit;

    result := (parAskMe1.Value = parAskMe2.Value);
    if result=False then Exit;
    end;
end;

initialization
   glDatabaseName:='dbMain';
finalization

end.
