unit Consol;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, StdCtrls, Grids, DBGrids, Buttons,
  RXSplit, ComCtrls, Menus, ConstEx, ADODB, mwHighlighter,
  mwCustomEdit, Db, DBCtrls, vvVBSSyn, vvSQLSyn;

type
  TfmConsol = class(TForm)
    pcPages: TPageControl;
    tsQuery: TTabSheet;
    tsResult: TTabSheet;
    tsMEMO: TTabSheet;
    pnMEMO: TPanel;
    MEMO: TMemo;
    pnQuery: TPanel;
    Panel1: TPanel;
    Panel2: TPanel;
    DBGrid1: TDBGrid;
    DBNavigator1: TDBNavigator;
    Panel3: TPanel;
    btGO: TSpeedButton;
    lbStatus: TLabel;
    btToExcel: TSpeedButton;
    lbRowCount: TLabel;
    MainMenu1: TMainMenu;
    Info1: TMenuItem;
    ADODataSet1: TADODataSet;
    ADOConnection1: TADOConnection;
    QueryBox: TmwCustomEdit;
    DataSource: TDataSource;
    SQLSyntax: TvvSqlSyn;
    VBSsyntax: TvvVbsSyn;
    procedure btGOClick(Sender: TObject);
    procedure QueryBoxChange(Sender: TObject);
    procedure btToExcelClick(Sender: TObject);
    procedure pcPagesChange(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure Info1Click(Sender: TObject);
    procedure DBNavigator1BeforeAction(Sender: TObject;
      Button: TNavigateBtn);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormActivate(Sender: TObject);
    procedure ADODataSet1AfterOpen(DataSet: TDataSet);
    procedure ADODataSet1AfterRefresh(DataSet: TDataSet);
    procedure ADODataSet1AfterPost(DataSet: TDataSet);
    procedure ADOConnection1AfterConnect(Sender: TObject);
  private
    function GetConnectionString:WideString;
    function GetUser:WideString;
    function GetPassword:WideString;
    procedure CorrectGridCols;
  end;

procedure ShowConsol(const SQL:string='');

var DefaultSQL:string;  //     

implementation

uses QueryEx, ExcelEx, Common, AppInfo,
     RecInfo, DBenv;

{$R *.DFM}

procedure ShowConsol(const SQL:string='');
begin
  with TfmConsol.Create(Forms.Application.MainForm) do
  begin
    try
      with ADOConnection1 do
      begin
        ConnectionString:=GetConnectionString;
        LoginPrompt:=not IsAdmin;
        Open;
      end;
      if SQL<>'' then QueryBox.Text:=SQL
        else if DefaultSQL<>'' then QueryBox.Text:=DefaultSQL;
      ShowModal;
    finally
      Release;
    end;
  end;
end;

function TfmConsol.GetConnectionString: WideString;
begin
  result:=GetADOConnString(GetUser,self.GetPassword);
end;


procedure TfmConsol.btGOClick(Sender: TObject);
var i,stat:integer; err:string;
begin
  with ADODataset1 do
  begin
    if Active then Close;
    CommandText:=QueryBox.Text;
    try
      Open;  Last;  First;
      stat:=2;
      for i:=0 to Fields.Count-1 do
        if not Fields[i].ReadOnly then
        begin
          stat:=1;
          break;
        end;
      tsResult.TabVisible:=true;
      tsResult.Show;
    except on E:Exception do
     begin
      if Active then Close;
      tsResult.TabVisible:=false;
      tsQuery.Show;
      if pos('not return a result set', LowerCase(E.Message))>0 then stat:=3
      else
        begin
          stat:=4;
          err:=E.Message;
        end;
     end;
    end;
  end;

  case stat of
   1:begin
       lbStatus.Caption:='Live';
       lbStatus.Font.Color:=clGreen;
       DBGrid1.Color:=clWindow;
       DBGrid1.ReadOnly:=false;
     end;
   2:begin
       lbStatus.Caption:='No Live';
       lbStatus.Font.Color:=clYellow;
       DBGrid1.Color:=clSilver;
       DBGrid1.ReadOnly:=true;
     end;
   3:begin lbStatus.Caption:='Procedure'; lbStatus.Font.Color:=clBlue; end;
   4:begin lbStatus.Caption:='Error'; lbStatus.Font.Color:=clRed; end;
  end;
  lbStatus.Hint:=err;
  if stat in [1,2] then lbRowCount.Caption:=IntToStr(ADODataSet1.RecordCount)+' records';
  if stat in [1,2] then btToExcel.Show;
  QueryBox.SelectAll;
end;

procedure TfmConsol.QueryBoxChange(Sender: TObject);
begin
  lbStatus.Caption:='';
  lbRowCount.Caption:='';
  btToExcel.Hide;
end;

procedure TfmConsol.btToExcelClick(Sender: TObject);
begin
  FormToExcel;
end;

procedure TfmConsol.pcPagesChange(Sender: TObject);
begin
  if pcPages.ActivePage=tsQuery then
  begin
    QueryBox.SetFocus;
    QueryBox.SelectAll;
  end;
  if pcPages.ActivePage=tsMEMO then MEMO.SetFocus;
end;

procedure TfmConsol.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if Key=VK_ESCAPE then Close;
  if (ssCtrl in Shift) and ((Key=Ord('g')) or (Key=Ord('G')))
    then btGOClick(Sender);
end;

procedure TfmConsol.Info1Click(Sender: TObject);
begin
  ShowRecorgInfo;
end;

procedure TfmConsol.DBNavigator1BeforeAction(Sender: TObject;
  Button: TNavigateBtn);
begin
  case Button of
    nbRefresh: begin
                 ReQueryOne(ADODataSet1);
                 ADODataSet1AfterRefresh(ADODataSet1);
                 Abort;
               end;
    nbInsert:  begin
                 ADODataSet1.Append;
                 Abort;
               end;
  end;
end;

procedure TfmConsol.CorrectGridCols;
var i:integer;
begin
    for i:=0 to DBGrid1.Columns.Count-1 do
     if DBGrid1.Columns[i].Width>150
      then DBGrid1.Columns[i].Width:=150;
end;

procedure TfmConsol.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  if not IsAdmin then ADOConnection1.Connected:=false;
  Action:=caFree;
end;

procedure TfmConsol.FormActivate(Sender: TObject);
begin
  pcPages.ActivePage:=tsQuery;
  QueryBox.SetFocus;
  QueryBox.SelectAll;
end;

function TfmConsol.GetPassword: WideString;
begin
  if IsAdmin
    then result:=DBenv.GetPassword
    else result:='';
end;

function TfmConsol.GetUser: WideString;
begin
  if IsAdmin then result:=Login else result:='sa';
end;

procedure TfmConsol.ADODataSet1AfterOpen(DataSet: TDataSet);
begin
  CorrectGridCols;
end;

procedure TfmConsol.ADODataSet1AfterRefresh(DataSet: TDataSet);
begin
  CorrectGridCols;
end;

procedure TfmConsol.ADODataSet1AfterPost(DataSet: TDataSet);
begin
  CorrectGridCols;
end;

procedure TfmConsol.ADOConnection1AfterConnect(Sender: TObject);
const err='It is impossible';
begin
  try ADOConnection1.Execute('if Lower(user_name())<>'#39'dbo'#39' raiserror('#39+err+#39',16,-1)');
  except on e:Exception do
    if pos(err,e.message)>0 then
      raise Exception.Create('    ');
  end
end;

end.

