unit CmdPrms;

interface

uses classes, ComServ, sysutils, Common, Forms;

const HARD_KEYS_1='[/s:<server>] [/db:<database>] [/l:<login> [/p:<password>]]';
      HARD_KEYS_2='[-s:<server>] [-db:<database>] [-l:<login> [-p:<password>]]';
      HARD_KEYS_3='[s:<server>] [db:<database>] [l:<login> [p:<password>]]';
      HARD_KEYS_4='[/h]';
      HARD_KEYS_5='[-h]';
      HARD_KEYS_6='[h]';
      HARD_KEYS_7='[/?]';
      HARD_KEYS_8='[-?]';
      HARD_KEYS_9='[?]';
      CMDPARAM_KEY_ENABLE='DB640652807911D39BB3004095A0A2B2';

var HelpText:string;

function GetCmdParam(const Key:string):string;
function IsCmdKeyEnable(const Key:string):boolean;
function IsCmdValueExist(const Key:string):boolean;
procedure ShowCmdParamHelp(const str:string='');

implementation

uses AppInfo;

var CommandParams:TStringList;

procedure ShowCmdParamHelp;
begin
  if HelpText=''
     then HelpText:='  :'#10#13
                   +'          "'+GetProgName+' '+HARD_KEYS_1+'"'#10#13
                   +'       "'+GetProgName+' '+HARD_KEYS_2+'"'#10#13
                   +'       "'+GetProgName+' '+HARD_KEYS_3+'"'#10#13
                   +'       "'+GetProgName+' '+HARD_KEYS_4+'"'#10#13
                   +'       "'+GetProgName+' '+HARD_KEYS_5+'"'#10#13
                   +'       "'+GetProgName+' '+HARD_KEYS_6+'"'#10#13
                   +'       "'+GetProgName+' '+HARD_KEYS_7+'"'#10#13
                   +'       "'+GetProgName+' '+HARD_KEYS_8+'"'#10#13
                   +'       "'+GetProgName+' '+HARD_KEYS_9;
  WarningMsg(HelpText);
end;

function IsValuePresent(const OneParam:string):boolean;
begin
  result:=(pos(':',OneParam)>1) and (pos(':',OneParam)<>Length(OneParam));
end;

function GetPartKey(const OneParam:string):string;
var ps:integer;
begin
  ps:=pos(':',OneParam);
  if ps>0 then result:=copy(OneParam,1,ps-1)
          else result:=OneParam;
  if result[1] in ['/','-'] then result:=copy(result,2,length(result)-1)
end;

function GetPartValue(const OneParam:string):string;
var ps:integer;
begin
  ps:=pos(':',OneParam);
  result:=copy(OneParam,ps+1,Length(OneParam)-ps);
end;

procedure SaveOneParam(const OneParam:string);
var Key, Value:string;
begin
   Key:=UpperCase(GetPartKey(OneParam));
   if IsValuePresent(OneParam)
     then Value:=GetPartValue(OneParam)
     else Value:=CMDPARAM_KEY_ENABLE;
   if (Key<>'?') and (Key<>'H') then
     begin
       if CommandParams.Values[Key]=''
        then CommandParams.Values[Key]:=Value
        else WarningMsg('   ');
     end
    else
     begin
       ShowCmdParamHelp;
       //RaiseProjectCloseException;
     end;
end;

procedure SetCommandParams;
var i:integer;
begin
  try
    CommandParams:=TStringList.Create;
    if ComServer.StartMode<>smStandalone then Exit;
    for i:=1 to ParamCount do  SaveOneParam(ParamStr(i));
  except on e:Exception do ErrorMsg(e.Message);
  end;
end;

function GetCmdParam(const Key:string):string;
begin
  result:=CommandParams.Values[UpperCase(Key)];
end;

function IsCmdKeyEnable(const Key:string):boolean;
begin
  result:=(GetCmdParam(Key)<>'')
end;

function IsCmdValueExist(const Key:string):boolean;
begin
  result:=IsCmdKeyEnable(Key) and (GetCmdParam(Key)<>CMDPARAM_KEY_ENABLE);
end;


initialization
   SetCommandParams;

finalization
   CommandParams.Free;

end.
