unit ExcelEx;

interface

uses classes, SysUtils, Forms, db;

procedure FormToExcel(Form:TForm=nil);
procedure DataSetToExcel(ds:TDataSet; var xl:OleVariant);Overload;
procedure DataSetToExcel(ds:TDataSet); Overload;

implementation

uses Windows, clipbrd, Common, ComObj, RxShell,
     controls, dbgrids, Graphics;

const
  xlMaximized = $FFFFEFD7;
  xlCenter = $FFFFEFF4;
  xlEdgeBottom = $00000009;
  xlEdgeLeft = $00000007;
  xlEdgeRight = $0000000A;
  xlEdgeTop = $00000008;
  xlContinuous = $00000001;
  xlThin = $00000002;
  xlInsideHorizontal = $0000000C;
  xlInsideVertical = $0000000B;

function CreateNewInstanceExcel:OleVariant;
var i:integer;
begin
    try result:=CreateOLEObject('Excel.Application');
    except
      for i:=1 to 30 do //, 
      begin
        Application.ProcessMessages;
        sleep(30);
      end;
      try result:=CreateOLEObject('Excel.Application');
      except
        //      SaveException(E.ClassName, E.Message);
        raise Exception.Create(' ,  Excel'#39'   '#10#13
                              +'     .'#10#13
                              +'    ,  ,'#10#13
                              +',  ,   Excel');
      end;
    end;
    result.Visible:=true;
    result.WorkBooks.Add;
    result.ActiveWindow.WindowState:=xlMaximized;
    result.WindowState:=xlMaximized;
end;

procedure DataSetToExcel(ds:TDataSet);
var xl:OleVariant;
begin
  DataSetToExcel(ds, xl);
end;

procedure FormToExcel(Form:TForm=nil);
var k:integer; qs:TList; xl:OleVariant;
begin
  if Form=nil then Form:=Screen.ActiveForm;
  qs:=TList.Create;
  try
    if Screen.ActiveControl is TDbGrid then
      qs.Add(TDbGrid(Screen.ActiveControl).DataSource.DataSet);
    if (qs.Count=0) then
      for k:=0 to Form.ComponentCount-1 do
       if (Form.Components[k] is TDataSet) then
        if (qs.Count=0) or ((qs.Count>0) and (TDataSet(Form.Components[k])<>TDataSet(qs[0]))) then
         if TDataSet(Form.Components[k]).Active then
          if TDataSet(Form.Components[k]).RecordCount>0 then
           qs.Add(TDataSet(Form.Components[k]));
    if qs.Count>0 then
      begin
        for k:=0 to qs.Count-1 do
         try DataSetToExcel(TDataSet(qs[k]),xl)
         except on e:Exception do
           raise Exception.Create('  "'+TDataSet(qs[k]).Name+'"'#10#13'NativeError: '+e.Message);
         end;
         xl.Sheets[TDataSet(qs[0]).Name].Select
      end
    else
      begin
        WarningMsg('    !');
      end;
  finally
    qs.Free;
  end;
end;


function FieldNameIsEng(const Field:TField):boolean;
var i:integer;
label EXIT_FUN;
begin
  result:=((trim(Field.DisplayLabel)='')
     or (Field.DisplayLabel=Field.FieldName));
  if not result then
  begin
    for i:=1 to Length(Field.DisplayLabel) do
      if Field.DisplayLabel[i] in [''..'',''..'']
        then goto EXIT_FUN;
    result:=true; //  
  end;
EXIT_FUN:
end;

function IsRussianFieldPresent(const ds:TDataSet):boolean;
var i:integer;
begin
  result:=false;
  for i:=0 to ds.FieldCount-1 do
  begin
    if not FieldNameIsEng(ds.Fields[i]) then
      begin
        result:=true;
        break;
      end;
  end;
end;

function IsSystem(const Field:TField):boolean;
begin
  if IsRussianFieldPresent(Field.DataSet)
    then result:=FieldNameIsEng(Field)
    else result:=(pos('id',LowerCase(Field.FieldName))=1);
end;

function GetFieldName(const Field:TField):string;
begin
  if IsSystem(Field)
      or (Field.DisplayLabel=Field.FieldName)
      or (Trim(Field.DisplayLabel)='') then result:=Field.FieldName
  else result:=Field.DisplayLabel;
end;

function IsNumeric(const str:string):boolean;
var i:integer;
begin
  if trim(str)='' then
  begin
    result:=false;
    exit;
  end;
  result:=true;
  for i:=1 to Length(str) do
  begin
    if not (str[i] in ['0'..'9',#32,#9,'.','e','E','+','-',',']) then
    begin
      result:=false;
      break;
    end;
  end;
end;

function GetFieldValue(const Field:TField):OleVariant;
begin
  if Field.Value=NULL then result:=''
  else if (Field.DataType in [ftString, ftWideString, ftGuid])
        and IsNumeric(Field.AsString) then result:='="'+Field.AsString+'"'
  else result:=Field.Value;
end;

function chcol(const index:integer):string;
const PERIOD=26;
begin
  if index<=PERIOD then result:=chr(ord('a')+index-1)
  else result:=chr(ord('a')+(index div PERIOD)-1)
              +chr(ord('a')+(index mod PERIOD)-1);
{  case index of
     1..26: result:=chr(ord('a')+index-1);
    27..52: result:='a'+chr(ord('a')+index-27);
    53..78: result:='b'+chr(ord('a')+index-53);
  end;}
end;

function Addr(const row,col:integer):string;
begin
  result:=chcol(col)+IntToStr(row);
end;

procedure FormatExcelSheet(xl:OleVariant; const RowCount,ColCount:integer);
var i:integer;
   procedure SetBorder(const arg:integer);
   begin
     xl.Range['a1', addr(RowCount,ColCount)].Borders[arg].LineStyle:=xlContinuous;
   end;
begin
    xl.Rows[1].Font.Bold:=true;
    xl.Columns.AutoFit;
    for i:=1 to ColCount do
      if xl.Columns[i].ColumnWidth>40 then xl.Columns[i].ColumnWidth:=40;
    xl.Rows[RowCount+2].Font.Bold:=true;
    xl.Rows[RowCount+1].RowHeight:=xl.Rows[RowCount+1].RowHeight div 2;
    xl.Rows[1].RowHeight:=xl.Rows[1].RowHeight*2;
    xl.Rows[1].HorizontalAlignment:=xlCenter;
    xl.Rows[1].VerticalAlignment:=xlCenter;
    SetBorder(xlEdgeLeft);
    SetBorder(xlEdgeTop);
    SetBorder(xlEdgeBottom);
    SetBorder(xlEdgeRight);
    if ColCount>1 then SetBorder(xlInsideVertical);
    if RowCount>1 then SetBorder(xlInsideHorizontal);
    xl.ActiveWindow.DisplayGridlines:=False;
end;

procedure FormatSystemCells(xl:OleVariant; const FirstSystemCol, RowCount,ColCount:integer);
begin
  xl.Range[addr(1,FirstSystemCol),addr(RowCount,ColCount)].Font.Color:=$00FFCECE{clAqua};
end;

procedure DataSetToExcel(ds:TDataSet; var xl:OleVariant);
var   i, NumFirstSystem, ColCount, RowCount:integer;
      p:TDataSetNotifyEvent;
      v, fnum, ffil:Variant;
      bmark:string;
      IsFirst, IsSystemPresent:boolean;
begin
    IsFirst:=VarType(xl)=varEmpty;
    if IsFirst then xl:=CreateNewInstanceExcel;
    xl.WorkSheets.Add;
    xl.ActiveSheet.Name:=ds.Name;
    if IsFirst then
    begin
      xl.DisplayAlerts:=False;
      i:=xl.WorkSheets.Count;
      if i>1 then for i:=i downto 1 do
       if xl.WorkSheets[i].Name<>ds.Name then
        xl.WorkSheets[i].Delete;
      xl.DisplayAlerts:=True;
    end;
    p:=ds.AfterScroll;
    ds.AfterScroll:=nil;
    try
      v:=VarArrayCreate([1, ds.RecordCount+1, 1, ds.FieldCount], varVariant);
      fnum:=VarArrayCreate([1, ds.FieldCount], varInteger);
      ffil:=VarArrayCreate([1, ds.FieldCount], varBoolean);
      ColCount:=0;
      for i:=0 to ds.FieldCount-1 do
        if not IsSystem(ds.Fields[i]) then
        begin
          Inc(ColCount);
          v[1,ColCount]:=GetFieldName(ds.Fields[i]);
          fnum[ColCount]:=i;
          ffil[ColCount]:=false;
        end;
      IsSystemPresent:=(ColCount<ds.FieldCount);
      if IsSystemPresent
        then NumFirstSystem:=ColCount+1
        else NumFirstSystem:=0;
      for i:=0 to ds.FieldCount-1 do
        if IsSystem(ds.Fields[i]) then
        begin
          Inc(ColCount);
          v[1,ColCount]:=GetFieldName(ds.Fields[i]);
          fnum[ColCount]:=i;
          ffil[ColCount]:=false;
        end;
      bmark:=ds.Bookmark;
      ds.DisableControls;
      try
        ds.First;
        RowCount:=1;
        while not ds.Eof do
        begin
          inc(RowCount);
          for i:=1 to ds.FieldCount do
          begin
            v[RowCount,i]:=GetFieldValue(ds.Fields[fnum[i]]);
            if (not ffil[i]) and (ds.Fields[fnum[i]].Value<>NULL)
               then ffil[i]:=true;
          end;
          ds.Next;
        end;
        ds.Bookmark:=bmark;
      finally
        ds.EnableControls;
      end;
    finally
      ds.AfterScroll:=p;
    end;
    if (ColCount<>ds.FieldCount) then
        raise Exception.Create('   "'+ds.Name+'"');
    RowCount:=ds.RecordCount+1;
    if (RowCount<>(ds.RecordCount+1)) then
       raise Exception.Create('  '#10#13'   '#10#13': "'+ds.Name+'"');
    xl.Range['a1', addr(RowCount,ColCount)].value:= v;
    if IsSystemPresent then
      begin
         FormatSystemCells(xl, NumFirstSystem, RowCount, ColCount);
         FormatExcelSheet(xl,RowCount,NumFirstSystem-1);
      end
    else FormatExcelSheet(xl,RowCount,ColCount);

    for i:=ColCount downto 1 do
    begin
      if ds.Fields[fnum[i]].DataType in [ftFloat,ftCurrency, ftBCD] then
      begin
        xl.Cells[RowCount+2,i].Value:='=sum('+addr(2,i)+':'+addr(RowCount,i)+')';
        try xl.Columns[i].NumberFormat:='# ### ### ##0.00';
        except try xl.Columns[i].NumberFormat:='# ### ### ##0,00';
               except WarningMsg('     Excel');
               end;
        end;
      end;
      if (not ffil[i]) then xl.Columns[i].Delete;
    end;
end;

{function FormToExcel(Form:TForm=nil; IsFast:boolean=false):boolean;
var k:integer; xl:variant; qs:TList;

  procedure ToExcel(ds:TDataSet);
  var i,j:integer; p:TDataSetNotifyEvent; bkmark:TBookmark;
      list:TStringList; s:AnsiString; sw:WideString;
      swHandle:THandle; swSize:Integer; psw:Pointer;
  begin
    if not xl.Visible then xl.Visible:=true;
    xl.WorkSheets.Add;
    xl.ActiveSheet.Name:=ds.Name;
    ds.DisableControls;
    p:=ds.AfterScroll;
    ds.AfterScroll:=nil;
    bkmark:=ds.GetBookmark;
    ds.First;
    if IsFast then
        begin
          list:=TStringList.Create;
          try
            s:='';
            for j:=0 to ds.FieldCount-1 do
             if (Copy(ds.Fields[j].FieldName,1,2)<>'Id')
                 and (Trim(ds.Fields[j].DisplayName)<>'')
               then s:=s+ds.Fields[j].DisplayName+#9;
            list.Add(s);
            while not ds.EOF do
            begin
              s:='';
              for j:=0 to ds.FieldCount-1 do
               if (Copy(ds.Fields[j].FieldName,1,2)<>'Id')
                 and (Trim(ds.Fields[j].DisplayName)<>'')
                then s:=s+ds.Fields[j].AsString+#9;
              list.Add(s);
              ds.Next;
            end;
            sw:=list.Text;
            s:=list.Text;
            list.Clear;
          finally
            list.Free;
          end;
          if Win32Platform=VER_PLATFORM_WIN32_NT then
            begin
              swSize:=(Length(sw)+1)*SizeOf(WideChar);
              ClipBoard.Open;
              try
                Clipboard.Clear;
                Forms.Application.ProcessMessages;
                Sleep(500);
                Forms.Application.ProcessMessages;
                swHandle:=GlobalAlloc(GMEM_MOVEABLE,swSize);
                try
                  psw:=GlobalLock(swHandle);
                  Move(sw[1],psw^,swSize);
                  Clipboard.SetAsHandle(CF_UNICODETEXT,swHandle);
                  GlobalUnlock(swHandle);
                finally
                  GlobalFree(swHandle);
                end;
              finally
                Clipboard.Close;
              end;
            end
           else
            begin
              Clipboard.SetTextBuf(PChar(s));
            end;
          Forms.Application.ProcessMessages;
          Sleep(1000);
          Forms.Application.ProcessMessages;
          try xl.ActiveSheet.Cells[1,1].Value:='ServiceCheck';
          except result:=false;
            ErrorMsg('  ');
            exit;
          end;
          xl.ActiveSheet.Paste;
        end
      else
        begin
          for j:=0 to ds.FieldCount-1 do
              xl.Cells[1,j+1].Value:=ds.Fields[j].DisplayName;
          i:=2;
          while not ds.EOF do
          begin
            for j:=0 to ds.FieldCount-1 do
                 xl.Cells[i,j+1].Value:=ds.Fields[j].Value;
            ds.Next;
            Inc(i);
          end;
        end;
        ds.GotoBookmark(bkmark);
        ds.AfterScroll:=p;
        ds.EnableControls;
        xl.Rows[1].Font.Bold:=true;
        xl.Columns.AutoFit;
        if ds=TDataSet(qs[0]) then
        begin
          xl.DisplayAlerts:=False;
          i:=xl.WorkSheets.Count;
          if i>1 then for i:=i downto 1 do
           if xl.WorkSheets[i].Name<>ds.Name then
            xl.WorkSheets[i].Delete;
          xl.DisplayAlerts:=True;
        end;
  end;

begin
  if Form=nil then Form:=Screen.ActiveForm;
  result:=true;
  qs:=nil;
  try xl:=CreateOLEObject('Excel.Application');
  except
    try xl:=CreateOLEObject('Excel.Application');
    except
      try xl:=GetActiveOLEObject('Excel.Application');
      except
        try
         FileExecute('Excel','','',esNormal);
         xl:=GetActiveOLEObject('Excel.Application');
        except
         ErrorMsg('Excel  !');
         Exit;
        end;
      end;
    end;
  end;
  try
    xl.WorkBooks.Add;
  except
    ErrorMsg('Excel  !');
    xl.Quit;
    Exit;
  end;
  xl.ActiveWindow.WindowState:=xlMaximized;
  xl.WindowState:=xlMaximized;
  try
    Screen.Cursor:=crHourGlass;
    qs:=TList.Create;
    if Screen.ActiveControl is TDbGrid then
      qs.Add(TDbGrid(Screen.ActiveControl).DataSource.DataSet);
    if (not IsFast) or (qs.Count=0) then
      for k:=0 to Form.ComponentCount-1 do
       if (Form.Components[k] is TDataSet) then
        if (qs.Count=0) or ((qs.Count>0) and (TDataSet(Form.Components[k])<>TDataSet(qs[0]))) then
         if TDataSet(Form.Components[k]).Active then
          if TDataSet(Form.Components[k]).RecordCount>0 then
           qs.Add(TDataSet(Form.Components[k]));
    for k:=0 to qs.Count-1 do
     try ToExcel(TDataSet(qs[k]))
     except
      ErrorMsg('  '+TDataSet(qs[k]).Name);
      result:=false;
     end;
    if qs.Count>0 then xl.Sheets[TDataSet(qs[0]).Name].Select
      else
        begin
          xl.ActiveWorkbook.Close(false);
          if xl.WorkBooks.Count=0 then xl.Quit;
          WarningMsg('    !');
        end;
  finally
    Screen.Cursor:=crDefault;
    qs.Free;
  end;
end;}


end.
