unit GridEx;

interface

uses Windows, Graphics, classes, Forms, Dialogs,
     controls, Grids, DB, dbtables, dbgrids,
     RxDBCtrl, SysUtils, VCLUtils;

procedure BoolRXDBGridCellClick(Column: TColumn);//overload;
procedure BoolRXDBGridKeyPress(RxGrid:TRxDBGrid; var Key: Char);//overload;
procedure BoolRXDBGridDrawColumnCell(const Rect: TRect; DataCol: Integer; Column: TColumn; State: TGridDrawState);//overload;
procedure BoolRXDBGridShowEditor(Field: TField; var AllowEdit: Boolean);
{
  BoolRXDBGridCellClick(Column);
  BoolRXDBGridDrawColumnCell(Rect,DataCol,Column,State);
  BoolRXDBGridKeyPress(TRxDBGrid(Sender), Key);
  BoolRXDBGridShowEditor(Field, AllowEdit);
}

function GetBoolImage(const BoolExpr:boolean):TBitmap;
procedure SetUnknowColor(Field:TField;aFont:TFont);
procedure ShowSummGridColumn(aDBGrid: TDBGrid);

// For Multi Select Delete
type TCheckDSDelete = function(DataSet: TDataSet) : Boolean;
function DBGridMultiSelectDelete(Grid: TDBGrid; fCheckDSDelete: TCheckDSDelete): Boolean;

// Ctrl-D = DoubleRecord
type TAfterDoubleRecordProc = procedure(DataSet: TDataSet);
procedure DoubleRecord(WinControl:TWinControl);
function GetAfterDoubleRecordProc : TAfterDoubleRecordProc;
procedure SetAfterDoubleRecordProc(pAfterDoubleRecordProc : TAfterDoubleRecordProc);


implementation

{$R BoolGrid.RES}

var
  GalochkaRect, BlanckRect, rxMemoBitmap, rxBlobBitmap: TBitmap;

  LastClicTime: TDateTime;
  LastClicFieldName: string;
  LastClicRecNo: integer;

  bInDBGridMultiSelectDelete : Boolean;

  FAfterDoubleRecordProc: TAfterDoubleRecordProc;

function GetAfterDoubleRecordProc : TAfterDoubleRecordProc;
begin
  result :=FAfterDoubleRecordProc;
end;

procedure SetAfterDoubleRecordProc(pAfterDoubleRecordProc : TAfterDoubleRecordProc);
begin
  FAfterDoubleRecordProc := pAfterDoubleRecordProc;
end;

function GetBoolImage(const BoolExpr:boolean):TBitmap;
var resr:string;
begin
  if BoolExpr then result:=GalochkaRect else result:=BlanckRect;
  if result=nil then
  begin
    if BoolExpr then resr:='BOOLGRDCHECKED' else resr:='BOOLGRDUNCHECK';
    result:=TBitmap.Create;
    result.LoadFromResourceName(HInstance,resr);
    if BoolExpr then GalochkaRect:=result else BlanckRect:=result;
  end;
end;

function GetMemoImage:TBitmap;
begin
  if rxMemoBitmap=nil then
  begin
    rxMemoBitmap:=TBitmap.Create;
    rxMemoBitmap.Handle:= LoadBitmap(HInstance, 'DBG_MEMO');
  end;
  result:=rxMemoBitmap;
end;

function GetBlobImage:TBitmap;
begin
  if rxBlobBitmap=nil then
  begin
    rxBlobBitmap := TBitmap.Create;
    rxBlobBitmap.Handle := LoadBitmap(HInstance, 'DBG_BLOB');
  end;
  result:=rxBlobBitmap;
end;

procedure BoolRXDBGridCellClick(Column: TColumn);
var IsNotModified:boolean; SavIndex: integer;
  function RxDBGrid:TRxDBGrid; begin result:=(Column.Grid as TRxDBGrid) end;
  function SelField:TField; begin result:=RxDBGrid.SelectedField end;
  function DataSet:TDataSet; begin result:=RxDBGrid.DataSource.DataSet end;
begin
  if Column<>nil then
    begin
    if (LastClicTime=0) or
       ((Time - LastClicTime)>EncodeTime(0, 0, 0, 500)) or
       (LastClicFieldName <> Column.FieldName) or
       (LastClicRecNo <> DataSet.RecNo) Then
      begin
      LastClicTime := Time;
      LastClicFieldName := Column.FieldName;
      LastClicRecNo := DataSet.RecNo;
      Exit;
      end;
    LastClicTime := Time;
    LastClicFieldName := Column.FieldName;
    LastClicRecNo := DataSet.RecNo;
    end;
  if not (SelField.DataType=ftBoolean) then Exit;
  IsNotModified:=not DataSet.Modified;
  if IsNotModified then DataSet.Edit;
  SelField.AsBoolean:=not SelField.AsBoolean;
  if IsNotModified and (DataSet.State<>dsInsert) then
    try
      DataSet.Post;
    finally
      SavIndex:=RxDBGrid.SelectedIndex;
      RxDBGrid.SelectedIndex:=-1;
      if RxDBGrid.SelectedIndex=SavIndex then
        begin
        RxDBGrid.SelectedIndex:= 0;
        if RxDBGrid.SelectedIndex=SavIndex then
          RxDBGrid.SelectedIndex:=RxDBGrid.Columns.Count-1;
        end;
      RxDBGrid.SelectedIndex:=SavIndex;
      RxDBGrid.Refresh;
      RxDBGrid.Repaint;
    end;
end;

procedure BoolRXDBGridKeyPress(RxGrid: TRxDBGrid; var Key: Char);
begin
  if (RxGrid.SelectedField.DataType=ftBoolean) and (Key=#32)
    then BoolRXDBGridCellClick(nil);
end;

procedure BoolRXDBGridDrawColumnCell(const Rect: TRect;
                      DataCol: Integer; Column: TColumn; State: TGridDrawState);
  function RxDBGrid:TRxDBGrid; begin result:=(Column.Grid as TRxDBGrid) end;
  function bmp:TBitmap; begin result:=GetBoolImage(TBooleanField(Column.Field).AsBoolean) end;
begin
  if Column.Field is TBooleanField then
    begin
      RxDBGrid.Canvas.FillRect(Rect);
      RxDBGrid.Canvas.Draw((Rect.Left+Rect.Right-bmp.Width) div 2,(Rect.Top+Rect.Bottom-bmp.Height) div 2,bmp);
    end
  else if Column.Field.DataType=ftMemo then
    begin
      RxDBGrid.Canvas.FillRect(Rect);
      DrawBitmapTransparent(RxDBGrid.Canvas,(Rect.Left+Rect.Right-GetMemoImage.Width) div 2,
                              (Rect.Top+Rect.Bottom-GetMemoImage.Height) div 2, GetMemoImage, clOlive);
    end
  else if Column.Field.DataType in [ftBlob, ftGraphic] then
    begin
      RxDBGrid.Canvas.FillRect(Rect);
      DrawBitmapTransparent(RxDBGrid.Canvas,(Rect.Left+Rect.Right-GetBlobImage.Width) div 2,
                             (Rect.Top+Rect.Bottom-GetBlobImage.Height) div 2, GetBlobImage, clOlive);
    end
  else
    begin
      RxDBGrid.DefaultDrawColumnCell(Rect,DataCol,Column,State);
    end;
end;

procedure BoolRXDBGridShowEditor(Field: TField; var AllowEdit: Boolean);
begin
   AllowEdit:=not (Field.DataType=ftBoolean);
end;

function DBGridMultiSelectDelete(Grid: TDBGrid; fCheckDSDelete: TCheckDSDelete): Boolean;
var
  cntDel, i : integer;
  bCheck : Boolean;
  aDataSet: TDataSet;
begin
  result := False;
  if bInDBGridMultiSelectDelete Then Exit;

  cntDel := Grid.SelectedRows.Count;
  if cntDel<=0 then Exit;

  aDataSet := Grid.DataSource.DataSet;
  aDataSet.DisableControls;
  try
    //1. Check
    for i:=0 to (Grid.SelectedRows.Count-1) do
    begin
      aDataSet.Bookmark := Grid.SelectedRows.Items[i];
      bCheck := fCheckDSDelete(aDataSet);
      if not bCheck then
      begin
        result := True;
        Exit;
      end;
    end;

    //2. Delete
    bInDBGridMultiSelectDelete := True;
    for i:=0 to (Grid.SelectedRows.Count-1) do
    begin
      aDataSet.Bookmark := Grid.SelectedRows.Items[i];
      try
        aDataSet.Delete;
      except
        on E:EAbort do ;
        else raise;
      end;
    end;
    result := True;
    Grid.SelectedRows.Clear;
  finally
    bInDBGridMultiSelectDelete := False;
    aDataSet.EnableControls;
  end;
end;

procedure ShowSummGridColumn(aDBGrid: TDBGrid);
var
  SelectedField: TField;
  aDataSet : TDataSet;
  curBookmark: TBookmarkStr;
  intSum : integer;
  fltSum : Double;
  isInt, isFlt: Boolean;
  strSum, sMSG1 : string;
begin
  aDataSet := aDBGrid.DataSource.DataSet;
  SelectedField := aDBGrid.SelectedField;
  sMSG1 := '';
  if SelectedField<>nil then
    begin
    intSum := 0; fltSum := 0;
    isInt := False; isFlt := False;
    case SelectedField.DataType of
      ftInteger, ftSmallint, ftAutoInc, ftLargeint:  isInt := True;
      ftFloat, ftCurrency, ftBCD: isFlt :=True;
      end;
    if isInt or isFlt then
      begin
      curBookmark := aDataSet.Bookmark;
      aDataSet.DisableControls;
      try
        aDataSet.First;
        while not aDataSet.EOF  do
          begin
          if  isInt then intSum := intSum + SelectedField.AsInteger;
          if  isFlt then fltSum := fltSum + SelectedField.AsFloat;
          aDataSet.Next;
          end;
      finally
        aDataSet.Bookmark := curBookmark;
        aDataSet.EnableControls;
        end;
      if  isInt then
        strSum := IntToStr(intSum);
      if  isFlt then
      begin
        if SelectedField.DataType=ftCurrency then
          strSum := FormatFloat('#,##0.00##', fltSum)
        else
          strSum := FormatFloat('#,##0.0000####', fltSum);
      end;

      sMSG1 := '  ' + SelectedField.DisplayLabel + ' [' +
                 SelectedField.FieldName+'] = ' + strSum + ' '#10#13;
      end;

    end
  else
    begin
    sMSG1 := '  !'#10#13;
    end;
  sMSG1 := sMSG1 + '  '+IntToStr(aDataSet.RecordCount)+' ';
  MessageDlg(sMSG1, mtInformation, [mbOK], 0);
end;

procedure SetUnknowColor(Field:TField;aFont:TFont);
begin
   if Field<>nil then
     if Field.AsString='< >' then
       aFont.Color:=clMoneyGreen;
  {clOlive, clTeam, clSilver, clCream, clMoneyGreen}
end;

procedure DoubleRecord(WinControl:TWinControl);
type
   PTPV=^TVP;
   TVP=record
         Name:string;
         Value:variant;
       end;
var ds:TDataSet;
    i:integer;
    rec:tlist;
    p:PTPV;
begin
  if not (WinControl is TDBGrid) then Exit;
  ds:=TDBGrid(Screen.ActiveControl).DataSource.DataSet;
  if ds.Modified then exit;
  rec:=TList.Create;
  try
    for i:=0 to ds.FieldCount-1 do
    begin
      New(p);
      if (ds.Fields[i].FieldKind=fkData)
         and (ds.Fields[i].DataType<>ftAutoInc)
         and (ds.Fields[i].FieldName<>'Id')
         and (ds.Fields[i].AsVariant<>NULL) then
      begin
        p^.Name:=ds.Fields[i].FieldName;
        p^.Value:=ds.Fields[i].AsVariant;
        rec.Add(p);
      end;
    end;
    ds.DisableControls;
    ds.Append;
    for i:=0 to rec.Count-1 do
      ds.FieldByName(PTPV(rec[i])^.Name).AsVariant:=PTPV(rec[i])^.Value;

    If Assigned(FAfterDoubleRecordProc) Then
      FAfterDoubleRecordProc(ds);
  finally
    for i:=rec.Count-1 downto 0 do dispose(PTPV(rec[i]));
    rec.Free;
    ds.EnableControls;
  end;
end;


initialization
finalization
   GalochkaRect.Free;
   BlanckRect.Free;
   rxMemoBitmap.Free;
   rxBlobBitmap.Free;
end.
