unit Impl_URepCollection;

interface

uses
  Windows, Classes, SysUtils, ComObj, ActiveX;

type
{$IFDEF VER130}  
  TIEnumVariantEnumerator = class(TContainedObject, IEnumVariant_D4)
{$ELSE}
  {$IFDEF VER120}
  TIEnumVariantEnumerator = class(TContainedObject, IEnumVariant)
  {$ENDIF}
{$ENDIF}
    private
      FController: IUnknown;
      fList: TInterfaceList;   // list of objects in collection
      fPos: Integer;           // current position in list
    protected
      function Next(celt: Longint; out elt; pceltFetched: PLongint): HResult; stdcall;
      function Skip(celt: Longint): HResult; stdcall;
      function Reset: HResult; stdcall;
      function Clone(out Enum: IEnumVariant): HResult; stdcall;
    public
      constructor Create(Controller: IUnknown; aList: TInterfaceList; aPos: Integer);
      destructor Destroy; override;
  end;

implementation

//===========================================================================
// TIEnumVariantEnumerator - ennumerator for IEnumVariant and InterfaceList
//===========================================================================
constructor TIEnumVariantEnumerator.Create(Controller: IUnknown;
                                        aList: TInterfaceList; aPos: Integer);
var
  I: Integer;
begin
  inherited Create(Controller);
  FController := Controller;
  fPos := aPos;
  fList := TInterfaceList.Create;
  for I := 0 to aList.Count - 1 do
    fList.Add(aList[I]);
end;

destructor TIEnumVariantEnumerator.Destroy;
begin
  fList.Free;
  inherited Destroy;
end;

function TIEnumVariantEnumerator.Clone(out Enum: IEnumVariant): HResult;
// Return a copy of this enumerator
begin
  try
    Enum := TIEnumVariantEnumerator.Create(FController, fList, fPos) as IEnumVariant;
    Result := S_OK;
  except
    Result := E_OUTOFMEMORY;
  end;
end;

function TIEnumVariantEnumerator.Next(celt: Integer; out elt; pceltFetched: PLongint):
HResult;
// Return the next celt members of collection in elt array.
// Return the actual number fetched in pceltFetched (unless its a nil pointer).
// Return S_OK if fetched celt members else return S_FALSE.
type
  TOleVariantArray = array[0..10000] of OleVariant;
  POleVariantArray = ^TOleVariantArray;
var
  I, N, Avail: Integer;
  Items: POleVariantArray;
begin
  // init caller's array
  Items := @elt;
  for I := 0 to celt - 1 do
    VariantInit(Items[I]);

  // how many items will we fetch?
  Avail := fList.Count - fPos;
  N := celt;
  if N > Avail then N := Avail;

  // copy interface ptrs into caller's array
  for I := 0 to N - 1 do
  begin
    Items[I] := fList[fPos] as IDispatch;
    Inc(fPos);
  end;

  // return how many Items we fetched
  if pceltFetched <> nil then
    pceltFetched^ := N;

  // tell if we returned the requested number of items
  if N = celt then
    Result := S_OK
  else
    Result := S_FALSE;
end;

function TIEnumVariantEnumerator.Reset: HResult;
// Move to start of collection.
begin
  fPos := 0;
  Result := S_OK;
end;

function TIEnumVariantEnumerator.Skip(celt: Integer): HResult;
// Skip over the next celt members of collection.
// Return S_OK if skipped that many, S_FALSE if not.
begin
  fPos := fPos + celt;
  if fPos <= fList.Count then
    Result := S_OK
  else
  begin
    fPos := fList.Count;
    Result := S_FALSE;
  end;
end;


end.
