unit Impl_URepFields;

interface

uses
  Classes, SysUtils, Db, Dbtables, ComObj, ActiveX, URepGen_TLB;

type
  TURepFields = class(TAutoIntfObject, URepFields)
  private
    FIntList: TInterfaceList;    // list of objects in collection
    function ConvertIndex(V: OleVariant): Integer;
  protected
    function Get__NewEnum: IUnknown; safecall;
    function Get_Item(Index: OleVariant): URepField; safecall;
    function Get_Count: Integer; safecall;
  public
    constructor Create(aDataSet: TDataSet);
    destructor Destroy; override;
  end;

  TURepField = class(TAutoIntfObject, URepField)
  private
    FField: TField;
  protected
    function Get_FieldName: WideString; safecall;
    function Get_DisplayLabel: WideString; safecall;
    function Get_BDEDataType: BDEFieldType; safecall;
    function Get_Value: OleVariant; safecall;
    function Get_ADODataType: Integer; safecall;
    function Get_Size: Integer; safecall;
    procedure SaveBLOBToFile(const FileName: WideString); safecall;
  public
    constructor Create(aField: TField);
  end;

implementation

uses Impl_URepObjectFactory, Impl_URepCollection, URepConstants;

//========================================================================
// TURepFields - collection of URepField
//========================================================================
constructor TURepFields.Create(aDataSet: TDataSet);
var
  i : integer;
  aURepField: TURepField;
begin
  inherited Create(GetURepTypeLib, URepFieldsDisp);
  FIntList := TInterfaceList.Create;

  for i:=0 to aDataSet.FieldCount-1 do
    begin
    aURepField := TURepField.Create(aDataSet.Fields[i]);
    FIntList.Add(aURepField as IUnknown);
    end;
end;

destructor TURepFields.Destroy;
begin
  FIntList.Free;
  inherited Destroy;
end;

function TURepFields.ConvertIndex(V: OleVariant): Integer;
var
  i : integer;
  bFind : Boolean;
begin
  Result := 0;
  case VarType(V) of
    varSmallInt, varInteger, varSingle, varDouble, varCurrency:
      begin
      Result := V - 1;
      if (Result<0) or (Result>FIntList.Count-1) then
        raise Exception.Create(SInvalidIndexVal);
      end;  
    varOleStr:
      begin
        bFind := False;
        for i:=0 to FIntList.Count-1 do
          if ((FIntList.Items[i] as URepField).FieldName)=V then
          begin
            bFind := True;
            Result := i;
            Break;
          end;
        if not bFind then
         raise Exception.Create(SInvalidIndexVal);
      end;
    varUnknown:
      Result := FIntList.IndexOf(IUnknown(V));
    varDispatch:
      Result := FIntList.IndexOf(IDispatch(V) as IUnknown);
    else
      raise Exception.Create(SInvalidIndexTyp);
  end;
end;

function TURepFields.Get__NewEnum: IUnknown;
begin
  result := TIEnumVariantEnumerator.Create(Self, FIntList, 0) as IUnknown;
end;

function TURepFields.Get_Item(Index: OleVariant): URepField;
begin
  Result := FIntList.Items[ConvertIndex(Index)] as URepField;
end;

function TURepFields.Get_Count: Integer;
begin
  result :=  FIntList.Count;
end;

//========================================================================
// TURepField
//========================================================================
constructor TURepField.Create(aField: TField);
begin
  inherited Create(GetURepTypeLib, URepFieldDisp);
  FField := aField;
end;

function TURepField.Get_FieldName: WideString;
begin
  result := FField.FieldName;
end;

function TURepField.Get_DisplayLabel: WideString;
begin
  result := FField.DisplayLabel;
end;

function TURepField.Get_BDEDataType: BDEFieldType;
begin
  result := Ord(FField.DataType);
end;

function TURepField.Get_Value: OleVariant;
begin
  result := FField.Value;
end;

// DataTypeEnum constants
//type
//  DataTypeEnum = TOleEnum;
const
  adEmpty = $00000000;
  adTinyInt = $00000010;
  adSmallInt = $00000002;
  adInteger = $00000003;
  adBigInt = $00000014;
  adUnsignedTinyInt = $00000011;
  adUnsignedSmallInt = $00000012;
  adUnsignedInt = $00000013;
  adUnsignedBigInt = $00000015;
  adSingle = $00000004;
  adDouble = $00000005;
  adCurrency = $00000006;
  adDecimal = $0000000E;
  adNumeric = $00000083;
  adBoolean = $0000000B;
  adError = $0000000A;
  adUserDefined = $00000084;
  adVariant = $0000000C;
  adIDispatch = $00000009;
  adIUnknown = $0000000D;
  adGUID = $00000048;
  adDate = $00000007;
  adDBDate = $00000085;
  adDBTime = $00000086;
  adDBTimeStamp = $00000087;
  adBSTR = $00000008;
  adChar = $00000081;
  adVarChar = $000000C8;
  adLongVarChar = $000000C9;
  adWChar = $00000082;
  adVarWChar = $000000CA;
  adLongVarWChar = $000000CB;
  adBinary = $00000080;
  adVarBinary = $000000CC;
  adLongVarBinary = $000000CD;
  adChapter = $00000088;
  adFileTime = $00000040;
  adDBFileTime = $00000089;
  adPropVariant = $0000008A;
  adVarNumeric = $0000008B;

function TURepField.Get_ADODataType: Integer;
begin
 case FField.DataType of
  db.ftUnknown   : result := adEmpty;
  db.ftString    : result := adChar; //adBSTR;
  db.ftSmallint  : result := adSmallInt;
  db.ftInteger   : result := adInteger;
  db.ftWord      : result := adEmpty;
  db.ftBoolean   : result := adBoolean;
  db.ftFloat     : result := adDouble;
  db.ftCurrency  : result := adCurrency;
  db.ftBCD       : result := adEmpty;
  db.ftDate      : result := adDBDate;
  db.ftTime      : result := adDBTime;
  db.ftDateTime  : result := adDate;
  db.ftBytes     : result := adEmpty;
  db.ftVarBytes  : result := adEmpty;
  db.ftAutoInc   : result := adInteger;
  db.ftBlob      : result := adEmpty;
  db.ftMemo      : result := adEmpty;
  db.ftGraphic   : result := adEmpty;
  db.ftFmtMemo   : result := adEmpty;
  db.ftParadoxOle  : result := adEmpty;
  db.ftDBaseOle    : result := adEmpty;
  db.ftTypedBinary : result := adEmpty;
  db.ftCursor      : result := adEmpty;
  db.ftFixedChar   : result := adEmpty;
  db.ftWideString  : result := adWChar;
  db.ftLargeint    : result := adBigInt;
  db.ftADT         : result := adEmpty;
  db.ftArray       : result := adEmpty;
  db.ftReference   : result := adEmpty;
  db.ftDataSet     : result := adEmpty;
  else result := adEmpty;
  end;
end;

function TURepField.Get_Size: Integer;
begin
  result := FField.Size;
end;

procedure TURepField.SaveBLOBToFile(const FileName: WideString);
begin
  if FField.IsBlob then
    (FField as TBlobField).SaveToFile(FileName)
  else
    raise Exception.Create(SInvalidDataType);  
end;

end.
