unit Impl_URepFunction;

interface

uses
  Windows, ComObj, sysutils, classes, Forms, ActiveX, DateUtil,
  db, dbtables, MAPI, URepGen_TLB;

type
  TURepFunction = class(TAutoIntfObject, URepFunction)
  private
    FQuery : TQuery;

    function NumToWordRus(fSumm: Double; bIsMoney: WordBool; iFracAccuracy: Integer;
                          IntNames: OleVariant; FracNames: OleVariant): WideString; safecall;
    function Pretty(const InputStr: WideString): WideString; safecall;
    function OemToAnsi(const InputStr: WideString): WideString; safecall;
    function AnsiToOem(const InputStr: WideString): WideString; safecall;
    function OpenLocalTable(const TblName: WideString; TblType: URepExportType): URepQuery; safecall;
    function StrToDateFmt(const InStr: WideString; const Format: WideString;
                          dtType: URepDateTimeType): TDateTime; safecall;
    function DelExtraSpace(const InStr: WideString): WideString; safecall;
    function StrToDoubleFmt(const InStr: WideString; const DecimalSep: WideString;
                            const ThousandSep: WideString): Double; safecall;
    procedure SendToMailSlot(Servers: OleVariant; const Slot: WideString;
                             const Message: WideString; Asynchronously: WordBool; Notise: WordBool); safecall;
    function  GetComputerName: WideString; safecall;
    procedure SetStatusText(const Text: WideString; Color: OLE_COLOR; isBold: WordBool;
                            isItalic: WordBool; isUnderline: WordBool); safecall;
    function  InputBox(const Caption: WideString; const Prompt: WideString;
                       const Default: WideString): WideString; safecall;
    function  MessageBox(const Prompt: WideString; Buttons: Integer; const Title: WideString): Integer; safecall;
    function  LoadURL(const URL: WideString; const OutFileName: WideString): OleVariant; safecall;
    procedure NWSendMessage(const User: WideString; const Message: WideString); safecall;
    function OpenQuery(const SQLtext: WideString; Params: OleVariant): URepQuery; safecall;
    function  GetLogin: WideString; safecall;
    function  CreateObject(const ObjectID: WideString; const ComputerName: WideString): IDispatch; safecall;
    procedure ExecQuery(const SQLtext: WideString; Params: OleVariant); safecall;
    procedure SendEmail(const Subject: WideString; const Message: WideString; EMailTo: OleVariant;
                        Attachment: OleVariant; Options: Integer); safecall;
  public
    destructor Destroy; override;
  end;

implementation

uses
  URepConstants, Impl_URepQuery, Impl_URepQryPrm, UserInfo,
  WordNumber, StrUtils, QueryEx, unThrdSender, mdMailSlot,
  SBctrl, Graphics, WinInet, AskMe, fURepFunc1; {, NWsend}

//==============================================================
// TURepFunction
//==============================================================
destructor TURepFunction.Destroy;
begin
  if assigned(FQuery) then
    FQuery.Free;
  inherited Destroy;
end;


function TURepFunction.NumToWordRus(fSumm: Double; bIsMoney: WordBool; iFracAccuracy: Integer;
                          IntNames: OleVariant; FracNames: OleVariant): WideString;
var
  iVarType, i : integer;
  strgsIntNames : TStringList;
  strgsFracNames : TStringList;
begin
  result := '';

  iVarType := VarType(IntNames);
  if  (iVarType=varOleStr) or
      (iVarType=varVariant) then
    begin
    strgsIntNames := TStringList.Create;
    strgsIntNames.Add(IntNames);
    end
  else if VarIsArray(IntNames) and
          (VarArrayDimCount(IntNames)=1) then
    begin
    strgsIntNames := TStringList.Create;
    strgsIntNames.Sorted := False;
    for i := VarArrayLowBound(IntNames,1) to VarArrayHighBound(IntNames,1) do
      begin
      strgsIntNames.Add(IntNames[i]);
      end;
    end
  else if (iVarType=varEmpty) or (iVarType=varNull) or
          ((iVarType=varError) and (TVarData(IntNames).VError = $80020004)) then
    begin
    strgsIntNames :=nil;
    end
  else
    begin
    raise Exception.Create('  IntNames');
    Exit;
    end;

  iVarType := VarType(FracNames);
  if  (iVarType=varOleStr) or
      (iVarType=varVariant) then
    begin
    strgsFracNames := TStringList.Create;
    strgsFracNames.Add(FracNames);
    end
  else if VarIsArray(FracNames) and
          (VarArrayDimCount(FracNames)=1) then
    begin
    strgsFracNames := TStringList.Create;
    strgsFracNames.Sorted := False;
    for i := VarArrayLowBound(FracNames,1) to VarArrayHighBound(FracNames,1) do
      strgsFracNames.Add(FracNames[i]);
    end
  else if (iVarType=varEmpty) or (iVarType=varNull) or
          ((iVarType=varError) and (TVarData(FracNames).VError = $80020004)) then
    begin
    strgsFracNames :=nil;
    end
  else
    begin
    raise Exception.Create('  FracNames');
    if assigned(strgsIntNames) then strgsIntNames.Free;
    Exit;
    end;


  result := FullWord_Number(fSumm, bIsMoney, iFracAccuracy,
                            strgsIntNames, strgsFracNames);
  if assigned(strgsIntNames) then strgsIntNames.Free;
  if assigned(strgsFracNames) then strgsFracNames.Free;
end;

function TURepFunction.Pretty(const InputStr: WideString): WideString;
begin
  result := AnsiUpperCase(Copy(InputStr,1,1))+AnsiLowerCase(Copy(InputStr,2,Length(InputStr)-1));
end;

function TURepFunction.OemToAnsi(const InputStr: WideString): WideString;
begin
  result := OemToAnsiStr(InputStr);
end;

function TURepFunction.AnsiToOem(const InputStr: WideString): WideString;
begin
  result := StrToOem(InputStr);
end;

function TURepFunction.OpenLocalTable(const TblName: WideString; TblType: URepExportType): URepQuery;
var
  LclTbl: TTable;
begin
  result := nil;
  LclTbl := TTable.Create(Forms.Application);
  case TblType of
    exptDBase   : LclTbl.TableType := ttDBase;
    exptParadox : LclTbl.TableType := ttParadox;
    exptFoxPro  : LclTbl.TableType := ttFoxPro;
    exptASCII   : LclTbl.TableType := ttASCII;
  else
    raise Exception.Create(SInvalidTableType);
  end;
  LclTbl.TableName := TblName;

  try
    LclTbl.Open;
  except
    on Err : EXception do
      raise Exception.Create(Err.ClassName+': '+Err.Message);
  end;
  result := TURepSingleQuery.Create(LclTbl);
end;

function TURepFunction.StrToDateFmt(const InStr: WideString; const Format: WideString;
                          dtType: URepDateTimeType): TDateTime;
var
  iD, iM, iY, iH, iMin, iSec, iRealLen: integer;
  sRestIn, sRestFmt, smmm : string;
  n1, n2, nTag, i : integer;
const
  dtTags : array[1..10] of string   =('dd','mmm','mm','yyyy','yy','hh','mi','ss', 'd', 'm');
  cntTags = 10;
  engShortMonth : array[1..12] of string =
    ('jan','feb','mar','apr','may','jun','jul', 'aug', 'sep', 'oct','nov','dec');
  rusShortMonth : array[1..12] of string =
    ('','','','','','','', '', '', '','','');

  procedure decode(s, f: string; nTag, nPos: integer;
                   var iD, iM, iY, iH, iMin, iSec, iRealLen : integer);
  var
    nLen, nPosDelim, iii : integer;
    iCenturyYear, iPivotYear : integer;
    sDelimiter : string;
  begin
    nLen := length(dtTags[nTag]);
    iRealLen := nLen;
    case nTag of
      1: iD  := StrToInt(Copy(s, nPos, nLen));
      2: // mmm - month short string format
         begin
         smmm := UpperCase(Copy(s, nPos, nLen));
         for iii:=1 to 12 do
           begin
           if UpperCase(engShortMonth[iii])=smmm then
             begin
             iM := iii;
             Break;
             end;
           end;
         if iM=0 then
           for iii:=1 to 12 do
             begin
             if UpperCase(rusShortMonth[iii])=smmm then
               begin
               iM := iii;
               Break;
               end;
             end;
         end;
      3: iM  := StrToInt(Copy(s, nPos, nLen));
      4: iY  := StrToInt(Copy(s, nPos, nLen));
      5: begin
           iCenturyYear := CurrentYear div 100 * 100;
           iY  := iCenturyYear + StrToInt(Copy(s, nPos, nLen));
           If TwoDigitYearCenturyWindow>0 then
           begin
             iPivotYear := CurrentYear - TwoDigitYearCenturyWindow;
             If iY<iPivotYear Then Inc(iY, 100);
           end;
         end;
      6: iH  := StrToInt(Copy(s, nPos, nLen));
      7: iMin:= StrToInt(Copy(s, nPos, nLen));
      8: iSec:= StrToInt(Copy(s, nPos, nLen));
      9: // d + delimiter
         begin
         sDelimiter := Copy(f, nPos+nLen, 1);
         nPosDelim := Pos(sDelimiter, Copy(s, nPos, nLen+2));
         if (nPosDelim>0) and (nPosDelim<=3) then
           begin
           iD  := StrToInt(Copy(s, nPos, nPosDelim-1));
           iRealLen := nPosDelim-1;
           end;
         end;
      10: // m + delimiter
         begin
         sDelimiter := Copy(f, nPos+nLen, 1);
         nPosDelim := Pos(sDelimiter, Copy(s, nPos, nLen+2));
         if (nPosDelim>0) and (nPosDelim<=3) then
           begin
           iM  := StrToInt(Copy(s, nPos, nPosDelim-1));
           iRealLen := nPosDelim-1;
           end;
         end;
    end;
  end;
begin
  result := 0;
  iD:=0; iM:=0; iY:=0; iH:=0; iMin:=0; iSec:=0;

  sRestIn := InStr;
  sRestFmt := Format;
  while length(sRestIn)>0 do
    begin
    //    Tag
    n2 := 0; nTag := 0;
    for i:=1 to cntTags do
      begin
      n1 := Pos(dtTags[i], sRestFmt);
      if (n1>0) and ( ((n2<>0)and(n1<n2)) or (n2=0) ) then
        begin
        n2 := n1;
        nTag := i;
        end;
      end;
    if nTag=0 then break;

    //  Tag
    decode(sRestIn, sRestFmt, nTag, n2,  iD, iM, iY, iH, iMin, iSec, iRealLen);
    sRestIn := Copy(sRestIn, n2+iRealLen, length(sRestIn));
    sRestFmt := Copy(sRestFmt, n2+length(dtTags[nTag]), length(sRestFmt));
    end;

  case dtType of
   dtDateOnly:
     begin
     if not((iY=0) and (iM=0) and (iD=0)) then
       result := EncodeDate(iY, iM, iD);
     end;
   dtTimeOnly:
     begin
     if not((iH=0) and (iMin=0) and (iSec=0))  then
       result := EncodeTime(iH, iMin, iSec, 0);
     end;
   dtDateAndTime:
     begin
     if not((iY=0) and (iM=0) and (iD=0) and (iH=0) and (iMin=0) and (iSec=0)) then
       result := EncodeDate(iY, iM, iD) + EncodeTime(iH, iMin, iSec, 0);
     end;
  end;
end;


function TURepFunction.DelExtraSpace(const InStr: WideString): WideString;
begin
  result := DelSpace1(InStr);
end;

function TURepFunction.StrToDoubleFmt(const InStr: WideString; const DecimalSep: WideString;
                            const ThousandSep: WideString): Double;
var
 sss, sThousandSep: String;
begin
  sss := InStr;
  sThousandSep := ThousandSep;
  If Length(sThousandSep)>=1 then
    sss := DelChars(sss, sThousandSep[1]);
  If Length(DecimalSep)>=1 then
    sss := ReplaceStr(sss, DecimalSep, SysUtils.DecimalSeparator);
  Result := StrToFloat(sss);
end;

procedure TURepFunction.SendToMailSlot(Servers: OleVariant; const Slot: WideString;
                             const Message: WideString; Asynchronously: WordBool; Notise: WordBool); safecall;
var
  iVarType, i, lBytes : integer;
  strgsServers : TStringList;
  sMessage: string;
begin
  sMessage := Message;

  iVarType := VarType(Servers);
  if  (iVarType=varOleStr) or
      (iVarType=varVariant) then
    begin
    strgsServers := TStringList.Create;
    strgsServers.Add(Servers);
    end
  else if VarIsArray(Servers) and
          (VarArrayDimCount(Servers)=1) then
    begin
    strgsServers := TStringList.Create;
    for i := VarArrayLowBound(Servers,1) to VarArrayHighBound(Servers,1) do
      strgsServers.Add(Servers[i]);
    end
  else
    begin
    raise Exception.Create('  Servers');
    if assigned(strgsServers) then strgsServers.Free;
    Exit;
    end;

  try
    if Asynchronously then
    begin
      TThreadSender.CreateEx(strgsServers, Slot, sMessage);
    end
    else
    begin
      while strgsServers.Count<>0 do
      begin
       mdMailSlot.SendToMailSlot(strgsServers[0], Slot, sMessage);
       strgsServers.Delete(0);
       Sleep(0);
      end;
    end;
  finally
    if assigned(strgsServers) then strgsServers.Free;
  end;
end;

function  TURepFunction.GetComputerName: WideString;
var
  sz : Cardinal; str: string;
begin
   sz:=MAX_COMPUTERNAME_LENGTH + 1;
   SetLength(str, sz);
   Windows.GetComputerName(PChar(str), sz);
   SetLength(str, sz);
   result:=str;
end;

procedure TURepFunction.SetStatusText(const Text: WideString; Color: OLE_COLOR; isBold: WordBool;
                            isItalic: WordBool; isUnderline: WordBool);
var
  SBctrl: TSBctrl;
  Style:TFontStyles;
begin
  SBctrl := GetURepScrollBarControl;
  If SBctrl=nil Then Exit;

  SBctrl.Text:= Text;
  SBctrl.Color:= Color;
  Style:=[];
  If isBold Then Style := Style + [fsBold];
  If isItalic Then Style := Style + [fsItalic];
  If isUnderline Then Style := Style + [fsUnderline];
  SBctrl.Style:= Style;
  SBctrl.Repaint;
end;

function TURepFunction.InputBox(const Caption: WideString; const Prompt: WideString;
                       const Default: WideString): WideString;
begin
  result := InputBox(Caption, Prompt, Default);
end;

function TURepFunction.MessageBox(const Prompt: WideString; Buttons: Integer; const Title: WideString): Integer;
var
  sTitle: String;
begin
  sTitle := Title;
  If sTitle='' Then sTitle := Application.Title;
  result := Application.MessageBox(PChar(String(Prompt)), PChar(sTitle), Buttons);
end;


procedure WinInetErrorException;
const
  Size:DWord = 3000;
var
  Error: DWord;
  S: string;
begin
  Error := GetLastError;
  SetLength(S, Size);
  InternetGetLastResponseInfo(Error, PChar(S), Size);
  raise Exception.Create(Format('WinInet Error Number: %d. Hex: %x #13#10 %s', [Error, Error, S]));
end;

function TURepFunction.LoadURL(const URL: WideString; const OutFileName: WideString): OleVariant;
const
  BufSize = 1024;
var
  Agent, outString: string;
  FInetRoot, FInetURL: HINTERNET;
  AccessType: Integer;
  bSucsses, bWriteFile: Boolean;
  Buffer: Pointer;
  NumRead: Cardinal;
  memStream: TMemoryStream;
begin
  Agent := Forms.Application.ExeName;
  AccessType := INTERNET_OPEN_TYPE_PRECONFIG; //INTERNET_OPEN_TYPE_PROXY
  bWriteFile := Length(Trim(OutFileName))>0;

  FInetRoot := InternetOpen(PChar(Agent), AccessType, nil, nil, 0);
  //if InternetAttemptConnect(0) <> ERROR_SUCCESS then SysUtils.Abort;
  If not Assigned(FInetRoot) then WinInetErrorException
  Else
    try
    FInetURL := InternetOpenUrl(FInetRoot, PChar(String(URL)), nil, 0, INTERNET_FLAG_RAW_DATA, 0);
    If not assigned(FInetURL) then WinInetErrorException
    else
      try
        memStream := TMemoryStream.Create;
        GetMem(Buffer, BufSize);
        try
          repeat
            bSucsses := InternetReadFile(FInetURL, Buffer, Bufsize, NumRead);
            If Not bSucsses then WinInetErrorException;
            if NumRead > 0 then
              memStream.Write(Buffer^, NumRead);
          until NumRead < BufSize;

          SetString(outString, PChar(memStream.Memory), memStream.Size);
          If bWriteFile then memStream.SaveToFile(OutFileName);
        finally
          FreeMem(Buffer, BufSize);
          memStream.Free;
        end;
      finally
        InternetCloseHandle(FInetURL);
      end;
    finally
      InternetCloseHandle(FInetRoot);
    end;
  result := outString;
end;

procedure TURepFunction.NWSendMessage(const User: WideString; const Message: WideString);
begin
  //NovellSendMessage(User, Message);
end;

function TURepFunction.OpenQuery(const SQLtext: WideString; Params: OleVariant): URepQuery;
var
  singlQuery : TQuery;
begin
  singlQuery := TQuery.Create(Forms.Application);
  singlQuery.DatabaseName:=glDatabaseName;
  singlQuery.SQL.Text := SQLtext;
  singlQuery.Prepare;

  BindQueryOleParams(singlQuery, Params);

  try
    singlQuery.Open;
  except
    on Err : EXception do
      raise Exception.Create(Err.ClassName+': '+Err.Message);
  end;

  result := TURepSingleQuery.Create(singlQuery);
end;

function TURepFunction.GetLogin: WideString;
begin
  result:= GetLoginName;
end;

function  TURepFunction.CreateObject(const ObjectID: WideString; const ComputerName: WideString): IDispatch;
var
  objGUID: TGUID;
  bSuccs: Boolean;
  IUnkn: IUnknown;
begin
  bSuccs := False;
  try
    objGUID := StringToGUID(ObjectID);
    bSuccs := True;
  except
  end;

  if not bSuccs then
    objGUID := ProgIDToClassID(ObjectID);
  IUnkn := CreateRemoteComObject(ComputerName, objGUID);
  result:= IUnkn as IDispatch;
end;


procedure TURepFunction.ExecQuery(const SQLtext: WideString; Params: OleVariant);
begin
  if not assigned(FQuery) then
  begin
    FQuery := TQuery.Create(Forms.Application.MainForm);
    FQuery.DatabaseName:=glDatabaseName;
  end;
  FQuery.SQL.Text := SQLtext;
  FQuery.Prepare;

  BindQueryOleParams(FQuery, Params);

  try
    FQuery.ExecSQL;
  except
    on Err : EXception do
      raise Exception.Create(Err.ClassName+': '+Err.Message);
  end;
end;

procedure MapiErrorMessage(MapiReslt: Cardinal);
begin
  if MapiReslt=SUCCESS_SUCCESS then Exit;
  raise Exception.Create('MAPI ERROR: '+IntToStr(MapiReslt));
end;

procedure TURepFunction.SendEmail(const Subject: WideString; const Message: WideString; EMailTo: OleVariant;
                        Attachment: OleVariant; Options: Integer);
var
  iVarTypeTo, iVarTypeAttach: integer;
  sEMailTo: String;
  sAttachment: String;
  bAttachment: Boolean;

  hSession: LHANDLE;
  flFlagsLogon, flFlagsSendMail: FLAGS;
  aMapiMessage: TMapiMessage;
  aMapiFileDesc: TMapiFileDesc;
  aMapiTORecipDesc: TMapiRecipDesc;
  MapiReslt: Cardinal;
begin
  case Options of
    1:   begin flFlagsLogon:=MAPI_LOGON_UI; flFlagsSendMail:=0; end;
    2:   begin flFlagsLogon:=MAPI_LOGON_UI; flFlagsSendMail:=MAPI_DIALOG; end;
    else begin flFlagsLogon:=0; flFlagsSendMail:=0; end;
  end;

  iVarTypeTo := VarType(EMailTo);
  if iVarTypeTo=varOleStr then
  begin
    sEMailTo := EMailTo;
  end
  else
    raise Exception.Create('EMailTo: -'+SInvalidDataType);

  bAttachment := False;
  iVarTypeAttach := VarType(Attachment);
  if not((iVarTypeAttach=varEmpty) or (iVarTypeAttach=varNull) or
         ((iVarTypeAttach=varError) and (TVarData(Attachment).VError = $80020004)) ) then
  begin
    if iVarTypeAttach=varOleStr then
    begin
      sAttachment := Attachment;
      bAttachment := True;
      with aMapiFileDesc do
      begin
        ulReserved := 0;
        flFlags := 0;
        nPosition := 0;
        lpszPathName := PChar(sAttachment);
        lpszFileName := PChar(ExtractFileName(sAttachment)) ;
        lpFileType := nil;
      end;
    end
    else
      raise Exception.Create('Attachment: -'+SInvalidDataType);
  end;

  //MapiLogon
  MapiReslt := MapiLogon(Forms.Application.Handle, '', '', flFlagsLogon, 0, @hSession);
  MapiErrorMessage(MapiReslt);
  //MapiSendMail
  with aMapiToRecipDesc do
  begin
    ulReserved := 0;
    ulRecipClass := MAPI_TO;
    lpszName := PChar(sEMailTo);
    lpszAddress := PChar('SMTP:'+ sEMailTo);
    ulEIDSize := 0;
    lpEntryID := nil;
  end;
  with aMapiMessage do
  begin
    ulReserved :=0 ;          //Reserved for future use (M.B. 0)
    lpszSubject := PChar(String(Subject)); //Message Subject
    lpszNoteText:= PChar(String(Message));    //Message Text
    lpszMessageType := '';       //Message Class
    lpszDateReceived :='';      //in YYYY/MM/DD HH:MM format
    lpszConversationID := '';    //conversation thread ID
    flFlags :=0;               //unread,return receipt
    lpOriginator := nil;   //Originator descriptor //PMapiRecipDesc;
    nRecipCount := 1;        //Number of recipients
    lpRecips := @aMapiToRecipDesc ;     //Recipient descriptors //PMapiRecipDesc
    if bAttachment then
    begin
      nFileCount := 1;         //# of file attachments
      lpFiles := @aMapiFileDesc;       //Attachment descriptors  //PMapiFileDesc
    end
      else
    begin
      nFileCount := 0;         //# of file attachments
      lpFiles := nil;       //Attachment descriptors  //PMapiFileDesc
    end;
  end;
  MapiReslt := MapiSendMail(hSession, Forms.Application.Handle, aMapiMessage, flFlagsSendMail, 0);
  MapiErrorMessage(MapiReslt);
  //MapiLogOff
  MapiReslt := MapiLogOff(hSession, Forms.Application.Handle, 0, 0);
  MapiErrorMessage(MapiReslt);
end;

end.
