unit Impl_URepImports;

interface

uses
  Windows, Classes, SysUtils, ComObj, ActiveX, AxCtrls, URepGen_TLB,
  Db, DbTables, Forms;

type
  TURepDataImports = class(TAutoIntfObject, URepDataImports)
  private
    FIntList: TInterfaceList;    // list of objects in collection
    function ConvertIndex(V: OleVariant): Integer;
  protected
    function Get__NewEnum: IUnknown; safecall;
    function Get_Item(Index: OleVariant): URepDataImport; safecall;
    function Get_Count: Integer; safecall;
  public
    constructor Create;
    destructor Destroy; override;
  end;

  TURepDataImport = class(TAutoIntfObject, URepDataImport)
  private
    FId: Integer;
    FName: WideString;
    FNumm: Integer;
    FDescription: WideString;
    FFileTemplate: WideString;
    FScrEngName: WideString;
    FSQLtext: WideString;
    FScriptText: WideString;
    FFileName: WideString;

    FQuery : TQuery;
    FURepFunction: URepFunction;

    FURepMain : URepMain;
  protected
    function Get_Id: Integer; safecall;
    function Get_Name: WideString; safecall;
    function Get_Numm: Integer; safecall;
    function Get_Description: WideString; safecall;
    function Get_FileTemplate: WideString; safecall;
    function Get_ScrEngName: WideString; safecall;
    function Get_SQLtext: WideString; safecall;
    function Get_ScriptText: WideString; safecall;
    function Get_FileName: WideString; safecall;
    procedure Set_FileName(const Value: WideString); safecall;
    function ExecSQL(Params: OleVariant; const SQLtext: WideString): Integer; safecall;
    function Get_URepFunction: URepFunction; safecall;
    function Get_URepMain: URepMain; safecall;
  public
    constructor Create(id: integer);
    destructor Destroy; override;
  end;

implementation

uses Impl_URepObjectFactory, Impl_URepCollection, Impl_URepQuery,
     Impl_URepQryPrm, Impl_URepFunction,  Impl_URepMain, URepConstants,
     dmdURep, QueryEx, AskMe;

//========================================================================
// TURepDataImports - collection of URepDataImport
//========================================================================
constructor TURepDataImports.Create;
begin
  inherited Create(GetURepTypeLib, URepDataImportsDisp);
  FIntList := TInterfaceList.Create;
  with dmURep do
  begin
    qImports.Close;
    qImports.Open;
    qImports.First;
    while not qImports.EOF do
      begin
        FIntList.Add( TURepDataImport.Create(qImportsId.Value) as URepDataImport);
        qImports.Next;
      end;
  end;
end;

destructor TURepDataImports.Destroy;
begin
  FIntList.Free;
  inherited Destroy;
end;

function TURepDataImports.ConvertIndex(V: OleVariant): Integer;
var
  i : integer;
  bFind : Boolean;
begin
  Result := 0;
  case VarType(V) of
    varSmallInt, varInteger, varSingle, varDouble, varCurrency:
      begin
      Result := V - 1;
      if (Result<0) or (Result>FIntList.Count-1) then
        raise Exception.Create(SInvalidIndexVal);
      end;
    varOleStr:
      begin
        bFind := False;
        for i:=0 to FIntList.Count-1 do
          if ((FIntList.Items[i] as URepDataImport).Name)=V then
          begin
            bFind := True;
            Result := i;
            Break;
          end;
        if not bFind then
         raise Exception.Create(SInvalidIndexVal);
      end;
    varUnknown:
      Result := FIntList.IndexOf(IUnknown(V));
    varDispatch:
      Result := FIntList.IndexOf(IDispatch(V) as IUnknown);
    else
      raise Exception.Create(SInvalidIndexTyp);
  end;
end;

function TURepDataImports.Get__NewEnum: IUnknown;
begin
  result := TIEnumVariantEnumerator.Create(Self, FIntList, 0) as IUnknown;
end;

function TURepDataImports.Get_Item(Index: OleVariant): URepDataImport;
begin
  Result := FIntList.Items[ConvertIndex(Index)] as URepDataImport;
end;

function TURepDataImports.Get_Count: Integer;
begin
  result :=  FIntList.Count;
end;

//========================================================================
// TURepDataImport - object for Data Import
//========================================================================
constructor TURepDataImport.Create(id: integer);
begin
  //WarningMsg('TURepDataImport.Create');
  inherited Create(GetURepTypeLib, URepDataImportDisp);
  with dmURep do
  begin
    qImportsBody.Close;
    qImportsBody.ParamByName('Id').asInteger := id;
    qImportsBody.Open;

    FId := qImportsBodyId.Value;
    FName := qImportsBodyName.Value;
    FDescription := qImportsBodyDescription.Value;
    FNumm := qImportsBodyNumm.Value;
    FFileTemplate := qImportsBodyFileTemplate.Value;
    FSQLtext := qImportsBodySQLtext.Value;
    FScriptText := qImportsBodyScriptText.Value;
    FScrEngName := qImportsBodyScrEngName.Value;
  end;
end;

destructor TURepDataImport.Destroy;
begin
  //WarningMsg('TURepDataImport.Destroy');
  if assigned(FQuery) then
    FQuery.Free;

  FURepFunction := nil;
  FURepMain := nil;

  inherited Destroy;
end;


function TURepDataImport.Get_Id: Integer;
begin
  result := FId;
end;

function TURepDataImport.Get_Name: WideString;
begin
  result := FName;
end;

function TURepDataImport.Get_Numm: Integer;
begin
  result := FNumm;
end;

function TURepDataImport.Get_Description: WideString;
begin
  result := FDescription;
end;

function TURepDataImport.Get_FileTemplate: WideString;
begin
  result := FFileTemplate;
end;

function TURepDataImport.Get_ScrEngName: WideString;
begin
  result := FScrEngName;
end;

function TURepDataImport.Get_SQLtext: WideString;
begin
  result := FSQLtext;
end;

function TURepDataImport.Get_ScriptText: WideString;
begin
  result := FScriptText;
end;


function TURepDataImport.ExecSQL(Params: OleVariant; const SQLtext: WideString): Integer;
var
  ExecSQLtext: WideString;
begin
  result := -1;
  if not assigned(FQuery) then
  begin
    FQuery := TQuery.Create(Forms.Application.MainForm);
    FQuery.DatabaseName:=glDatabaseName;
  end;

  if SQLtext<>'' then
    ExecSQLtext := SQLtext
  else
    ExecSQLtext := FSQLtext;

  if FQuery.SQL.Text<>ExecSQLtext then
    FQuery.SQL.Text := ExecSQLtext;
  FQuery.Prepare;

  BindQueryOleParams(FQuery, Params);

  try
    FQuery.ExecSQL;
  except
    on Err : EXception do
      raise Exception.Create(Err.ClassName+': '+Err.Message);
  end;
  result := 0;
end;

function TURepDataImport.Get_FileName: WideString;
begin
  result := FFileName;
end;

procedure TURepDataImport.Set_FileName(const Value: WideString);
begin
  FFileName := Value;
end;

function TURepDataImport.Get_URepFunction: URepFunction;
begin
  if FURepFunction=nil then
    FURepFunction := TURepFunction.Create(GetURepTypeLib, URepFunctionDisp);
  result := FURepFunction;
end;

function TURepDataImport.Get_URepMain: URepMain;
begin
  if FURepMain=nil then
    FURepMain := TURepMain.Create(GetURepTypeLib, URepMainDisp);
  result := FURepMain;
end;


{
function TURepDataImport.OpenQuery(const SQLtext: WideString; Params: OleVariant): URepQuery;
var
  singlQuery : TQuery;
  i, iVarType: integer;
  parName: String;
  varValue: OleVariant;
begin
  singlQuery := TQuery.Create(Forms.Application);
  singlQuery.DatabaseName:=glDatabaseName;
  singlQuery.SQL.Text := SQLtext;
  singlQuery.Prepare;

  iVarType := VarType(Params);
  if not((iVarType=varEmpty) or (iVarType=varNull) or
         ((iVarType=varError) and
          (TVarData(Params).VError = $80020004)) ) then
  begin
    for i:=0 to singlQuery.ParamCount-1 do
    begin
      parName := singlQuery.Params[i].Name;
      if not Params.Exists(parName) then
      begin
        raise Exception.Create('Parametr with name <'+parName+'> NOT FOUND');
      end;
      varValue := Params.Item[parName];
      singlQuery.Params[i].Value := varValue;
    end;
  end;
  try
    singlQuery.Open;
  except
    on Err : EXception do
      raise Exception.Create(Err.ClassName+': '+Err.Message);
  end;

  result := TURepSingleQuery.Create(singlQuery);
end;

procedure TURepDataImport.BindBlobParamToFile(const ParamName: WideString; const FileName: WideString);
var
  iIndex: integer;
begin
  iIndex := FBlobParamToFile.IndexOfName(ParamName);
  if iIndex>=0 then
    FBlobParamToFile.Values[ParamName] := FileName
  else
    FBlobParamToFile.Add(ParamName+'='+FileName);
end;


function  TURepDataImport.GetReport(ReportId: OleVariant): UReport; safecall;
var
  repId, VarTypeReportId: integer;
  repName: string;
  bFind: Boolean;
  aTUReport: TUReport;
  frmAskMe : TfrmAskMe;
begin
  result := nil;

  if not dmURep.qForm.Active then
    RequeryOne(dmURep.qForm);
  VarTypeReportId := VarType(ReportId);
  case VarTypeReportId of
    varSmallInt, varInteger, varSingle, varDouble, varCurrency:
      begin
      repId := ReportId;
      bFind := dmURep.qForm.Locate('Id', repId, []);
      end;
    varOleStr:
      begin
      repName := ReportId;
      bFind := dmURep.qForm.Locate('Name', repName, []);
      end;
    else
      raise Exception.Create(SInvalidIndexTyp);
  end;

  if bFind then
    begin
    // create UReport and fill
    with dmURep do
      begin
      aTUReport := TUReport.Create(GetURepTypeLib, UReportDisp);
      aTUReport.FId := qFormId.Value;
      aTUReport.FNumm := qFormNumm.Value;
      aTUReport.FName := qFormName.Value;
      aTUReport.FDescription := qFormDescription.Value;
      aTUReport.FDestination := qFormIdDestination.Value;
      aTUReport.FTypExec := qFormIdTypExec.Value;
      end;

    // create and init param List (TList)
    aTUReport.FparamList := TParamAskMeList.CreateParamList(aTUReport.FId);
    frmAskMe := TfrmAskMe.Create(Forms.Application, aTUReport.FparamList);
    try
      frmAskMe.BitBtnOKClick(nil);
    finally
      frmAskMe.Release;
      end;

    FUReport := aTUReport as UReport;
    result := FUReport;
    end
  else
    raise Exception.Create(SInvalidIndexVal);
end;
}


end.
