unit Impl_URepMain;

interface

uses
  Windows, ComObj, sysutils, classes, Forms, ActiveX,
  URepGen_TLB, AskMe;

type
  TURepMain = class(TAutoIntfObject, URepMain)
  private
    FUReport: UReport;
    FURepFunction: URepFunction;
  protected
    function GetReport(ReportId: OleVariant): UReport; safecall;
    function GetDataImports: URepDataImports; safecall;
    function Get_URepFunction: URepFunction; safecall;
    function Create_URepQueryParams: URepQueryParams; safecall;
  public
    destructor Destroy; override;
  end;

  TUReport = class(TAutoIntfObject, UReport)
  private
  protected
    function Get_Id: Integer; safecall;
    function Get_Numm: Integer; safecall;
    function Get_Name: WideString; safecall;
    function Get_Destination: URepDestination; safecall;
    function Get_TypExec: URepTypExec; safecall;
    function Get_Description: WideString; safecall;
    function Get_Params: URepParams; safecall;
    procedure RunReport; safecall;
    procedure RunReportExt(Destination: URepDestination; TypExec: URepTypExec;
                           const FileName: WideString); safecall;
    function AskParams: WordBool; safecall;
    function Get_GetQuerys: URepQuerys; safecall;
  public
    FId: Integer;
    FNumm: Integer;
    FName: WideString;
    FDestination: URepDestination;
    FTypExec: URepTypExec;
    FDescription: WideString;
    FParams : URepParams;

    FparamList: TParamAskMeList;  // list of TParamAskMe
    FHiddenForm: TForm;

    destructor Destroy; override;
  end;


implementation

uses
  Impl_URepObjectFactory, Impl_URepQuerys, Impl_URepParams,
  Impl_URepImports, Impl_URepFunction, Impl_URepQryPrm, URepConstants,
  dmdURep, fURepFunc0, fURepFunc1, URepTypes, QueryEx;


//==============================================================
// TURepMain - report generator main object
//==============================================================
function TURepMain.GetReport(ReportId: OleVariant): UReport;
var
  repId, VarTypeReportId: integer;
  repName: string;
  bFind: Boolean;
  aTUReport: TUReport;
  frmAskMe : TfrmAskMe;
begin
  result := nil;

  if not dmURep.qForm.Active then
    RequeryOne(dmURep.qForm);
  VarTypeReportId := VarType(ReportId);
  case VarTypeReportId of
    varSmallInt, varInteger, varSingle, varDouble, varCurrency:
      begin
      repId := ReportId;
      bFind := dmURep.qForm.Locate('Id', repId, []);
      end;
    varOleStr:
      begin
      repName := ReportId;
      bFind := dmURep.qForm.Locate('Name', repName, []);
      end;
    else
      raise Exception.Create(SInvalidIndexTyp);
  end;

  if bFind then
    begin
    // create UReport and fill
    with dmURep do
      begin
      aTUReport := TUReport.Create(GetURepTypeLib, UReportDisp);
      aTUReport.FId := qFormId.Value;
      aTUReport.FNumm := qFormNumm.Value;
      aTUReport.FName := qFormName.Value;
      aTUReport.FDescription := qFormDescription.Value;
      aTUReport.FDestination := qFormIdDestination.Value;
      aTUReport.FTypExec := qFormIdTypExec.Value;
      end;

    // create and init param List (TList)
    aTUReport.FparamList := TParamAskMeList.CreateParamList(aTUReport.FId);
    frmAskMe := TfrmAskMe.Create(Forms.Application, aTUReport.FparamList);
    try
      frmAskMe.BitBtnOKClick(nil);
    finally
      frmAskMe.Release;
      end;

    FUReport := aTUReport as UReport;
    result := FUReport;
    end
  else
    raise Exception.Create(SInvalidIndexVal);
end;

function TURepMain.GetDataImports: URepDataImports;
begin
  result := TURepDataImports.Create;
end;

function TURepMain.Get_URepFunction: URepFunction;
begin
  if FURepFunction=nil then
    FURepFunction := TURepFunction.Create(GetURepTypeLib, URepFunctionDisp);
  result := FURepFunction;
end;


function TURepMain.Create_URepQueryParams: URepQueryParams;
begin
  result := TURepQueryParams.Create as URepQueryParams;
end;


destructor TURepMain.Destroy;
begin
  //WarningMsg('TURepMain.Destroy');
  FUReport := nil;
  FURepFunction := nil;
  inherited Destroy;
end;

//==============================================================
// TUReport
//==============================================================
destructor TUReport.Destroy;
begin
  FParams := nil;
  if assigned(FHiddenForm) then FHiddenForm.Release;
  if assigned(FparamList) then FparamList.Free;
  inherited Destroy;
end;

function TUReport.Get_Id: Integer;
begin
  result := FId;
end;

function TUReport.Get_Numm: Integer;
begin
  result := FNumm;
end;

function TUReport.Get_Name: WideString;
begin
  result := FName;
end;

function TUReport.Get_Destination: URepDestination;
begin
  result := FDestination;
end;

function TUReport.Get_TypExec: URepTypExec;
begin
  result := FTypExec;
end;

function TUReport.Get_Description: WideString;
begin
  result := FDescription;
end;

function TUReport.Get_Params: URepParams;
begin
  if FParams=nil then
    FParams := TURepParams.Create(FparamList);
  result := FParams;
end;

procedure TUReport.RunReport;
begin
  RunReportExt(FDestination, FTypExec, '');
end;

procedure TUReport.RunReportExt(Destination: URepDestination; TypExec: URepTypExec;
                           const FileName: WideString);
begin
  Application.Restore;
  Application.BringToFront;
  URepExtFormAction(FId, FName, FparamList, factExecDef, FileName,
                    Destination, TypExec, False);
end;

function TUReport.AskParams: WordBool;
begin
 Application.Restore;
 Application.BringToFront;
 Result := AskMeParams(FName, FparamList);
end;

function TUReport.Get_GetQuerys: URepQuerys;
var
  QueryList : TQueryList;
begin
  QueryList :=  URepCreateQueryList(FId, ttForm);
  if not assigned(FHiddenForm) then
    FHiddenForm := TForm.Create(Application);
  URepCreateDataSets(FHiddenForm, FparamList, QueryList, False );
  URepLogAction(2, FId, FName);
  Result := TURepQuerys.Create(QueryList, FHiddenForm);
  QueryList.Free;
end;



end.
