unit Impl_URepObjectFactory;

interface

uses SysUtils, ComObj, ActiveX, Windows, Dialogs, URepGen_TLB;

type
  TURepAutoObjectFactory = class(TAutoObjectFactory)
  public
    procedure UpdateRegistry(Register: Boolean); override;
    destructor Destroy; override;
  end;

function GetURepTypeLib: ITypeLib;
function LoadURepTypeLib : Boolean;

implementation

const
  cnstUrepGenTlb = 'URepGen.tlb';

var
  URepTypeLib : ITypeLib;
  URepGenTlbFileName: WideString;

function GetURepTypeLib: ITypeLib;
begin
  if URepTypeLib=nil then
     ShowMessage('   '+cnstUrepGenTlb+' !');
  result := URepTypeLib;
end;


function LoadURepTypeLib : Boolean;
var
  Buffer: array[0..1023] of Char;
  sSystemDir, sUrepGenTlbFN: string;
begin
  If URepTypeLib=nil Then
  begin
    result := False;
    //1. Windows/System
    SetString(sSystemDir, Buffer, GetSystemDirectory(Buffer, SizeOf(Buffer)));
    sUrepGenTlbFN := sSystemDir+'\'+cnstUrepGenTlb;
    if FileExists(sUrepGenTlbFN) then
    begin
      URepGenTlbFileName := sUrepGenTlbFN;
      OleCheck(LoadTypeLib(PWideChar(URepGenTlbFileName), URepTypeLib));
      result := True;
    end
    else //2. AppPath
    begin
      sUrepGenTlbFN := ExtractFilePath(ParamStr(0))+cnstUrepGenTlb;
      if FileExists(sUrepGenTlbFN) then
      begin
        URepGenTlbFileName := sUrepGenTlbFN;
        OleCheck(LoadTypeLib(PWideChar(URepGenTlbFileName), URepTypeLib));
        result := True;
      end;
    end;
  end
  else
   result := True;
end;

procedure RegisterTypeLibrary(TypeLib: ITypeLib; const ModuleName: WideString);
var
  HelpPath: WideString;
begin
  HelpPath := ExtractFilePath(ModuleName);
  OleCheck(RegisterTypeLib(TypeLib, PWideChar(ModuleName), PWideChar(HelpPath)));
end;

{procedure UnregisterTypeLibrary(TypeLib: ITypeLib);
type
  TUnregisterProc = function(const GUID: TGUID; VerMajor, VerMinor: Word;
    LCID: TLCID; SysKind: TSysKind): HResult stdcall;
var
  Handle: THandle;
  UnregisterProc: TUnregisterProc;
  LibAttr: PTLibAttr;
begin
  Handle := GetModuleHandle('OLEAUT32.DLL');
  if Handle <> 0 then
  begin
    @UnregisterProc := GetProcAddress(Handle, 'UnRegisterTypeLib');
    if @UnregisterProc <> nil then
    begin
      OleCheck(TypeLib.GetLibAttr(LibAttr));
      with LibAttr^ do
        UnregisterProc(guid, wMajorVerNum, wMinorVerNum, lcid, syskind);
      TypeLib.ReleaseTLibAttr(LibAttr);
    end;
  end;
end;
}
procedure TURepAutoObjectFactory.UpdateRegistry(Register: Boolean);
begin
  inherited UpdateRegistry(Register);
  if Register then
  begin
    if LoadURepTypeLib then
      RegisterTypeLibrary(URepTypeLib, URepGenTlbFileName)
  end
  else
    OleCheck(UnregisterTypeLib(LIBID_URepGen, URepGenMajorVersion, URepGenMinorVersion, 0, SYS_WIN32));
end;

destructor TURepAutoObjectFactory.Destroy;
begin
  URepTypeLib := nil; // Free interface on destroy object factory
  inherited Destroy;
end;

end.
