unit Impl_URepParams;

interface

uses
  Windows, Classes, SysUtils, ComObj, ActiveX, URepGen_TLB, AskMe;

type
  TURepParams = class(TAutoIntfObject, URepParams)
  private
    FIntList: TInterfaceList;    // list of objects in collection
    function ConvertIndex(V: OleVariant): Integer;
  protected
    function Get__NewEnum: IUnknown; safecall;
    function Get_Item(Index: OleVariant): URepParam; safecall;
    function Get_Count: Integer; safecall;
  public
    constructor Create(paramList: TParamAskMeList);
    destructor Destroy; override;
  end;


  TURepParam = class(TAutoIntfObject, URepParam)
  private
    FNumm : Integer;
    FparamAskMe: TParamAskMe;
  protected
    function Get_Id: Integer; safecall;
    function Get_Numm: Integer; safecall;
    function Get_Name: WideString; safecall;
    function Get_ParLabel: WideString; safecall;
    function Get_ParType: URepParamType; safecall;
    function Get_Value: OleVariant; safecall;
    procedure Set_Value(Value: OleVariant); safecall;
  public
    constructor Create(paramAskMe: TParamAskMe);
    destructor Destroy; override;
  end;


implementation

uses Impl_URepObjectFactory, Impl_URepCollection, URepConstants;

//==============================================================
// TURepParams - collection of URepParam
//==============================================================
constructor TURepParams.Create(paramList: TParamAskMeList);
var
  i : integer;
  paramAskMe : TParamAskMe;
  aTURepParam : TURepParam;
begin
  inherited Create(GetURepTypeLib, URepParamsDisp);
  FIntList := TInterfaceList.Create;
  //create TURepParams and add to intf list
  for i:=0 to paramList.Count-1 do
    begin
    paramAskMe := TParamAskMe(paramList.Items[i]);
    aTURepParam := TURepParam.Create(paramAskMe);
    aTURepParam.FNumm := i + 1;
    FIntList.Add(aTURepParam as IUnknown);
    end;
end;

destructor TURepParams.Destroy;
begin
  //sTmp := 'TURepParams.Destroy';
  FIntList.Free;
  inherited Destroy;
end;

function TURepParams.ConvertIndex(V: OleVariant): Integer;
var
  i : integer;
  bFind : Boolean;
begin
  Result := 0;
  case VarType(V) of
    varSmallInt, varInteger, varSingle, varDouble, varCurrency:
      Result := V - 1;
    varOleStr:
      begin
        bFind := False;
        for i:=0 to FIntList.Count-1 do
          if ((FIntList.Items[i] as URepParam).Name)=V then
          begin
            bFind := True;
            Result := i;
            Break;
          end;
        if not bFind then
         raise Exception.Create(SInvalidIndexVal);
      end;
    varUnknown:
      Result := FIntList.IndexOf(IUnknown(V));
    varDispatch:
      Result := FIntList.IndexOf(IDispatch(V) as IUnknown);
    else
      raise Exception.Create(SInvalidIndexTyp);
  end;
end;

function TURepParams.Get__NewEnum: IUnknown;
begin
  result := TIEnumVariantEnumerator.Create(Self, FIntList, 0) as IUnknown;
end;

function TURepParams.Get_Item(Index: OleVariant): URepParam;
begin
  Result := FIntList.Items[ConvertIndex(Index)] as URepParam;
end;

function TURepParams.Get_Count: Integer;
begin
 result :=  FIntList.Count;
end;

//==============================================================
// TURepParam
//==============================================================
constructor TURepParam.Create(paramAskMe: TParamAskMe);
begin
  FparamAskMe := paramAskMe;
  inherited Create(GetURepTypeLib, URepParamDisp);
end;

destructor TURepParam.Destroy;
begin
  //sTmp := 'TURepParam.Destroy';
  inherited Destroy;
end;

function TURepParam.Get_Id: Integer;
begin
  result := FparamAskMe.IdParam;
end;

function TURepParam.Get_Numm: Integer;
begin
 result := FNumm;
end;

function TURepParam.Get_Name: WideString;
begin
  result := FparamAskMe.parName;
end;

function TURepParam.Get_ParLabel: WideString;
begin
  result := FparamAskMe.slabel;
end;

function TURepParam.Get_ParType: URepParamType;
begin
  result := Ord(FparamAskMe.inpTyp);
end;

function TURepParam.Get_Value: OleVariant;
begin
  result := FparamAskMe.Value;
end;

procedure TURepParam.Set_Value(Value: OleVariant);
begin
  FparamAskMe.Value := Value;
end;


end.
