unit Impl_URepQryPrm;

interface

uses
  Windows, Classes, SysUtils, ComObj, ActiveX, AxCtrls, URepGen_TLB,
  Db, DbTables, Forms;

type
  TURepQueryParams = class(TAutoIntfObject, URepQueryParams)
  private
    FIntList: TInterfaceList;      // list of objects (URepQueryParam) in collection
    FNamesQueryParam: TStringList; // list of URepQueryParam Name's

    function ConvertIndex(V: OleVariant): Integer;
  protected
    function Get__NewEnum: IUnknown; safecall;
    function Get_Item(Index: OleVariant): URepQueryParam; safecall;
    function Get_Count: Integer; safecall;

    function  Add(const Name: WideString; Value: OleVariant; DataType: BDEFieldType;
                            const FileName: WideString): URepQueryParam; safecall;
    procedure Clear; safecall;
  public
    constructor Create;
    destructor Destroy; override;
  end;

  TURepQueryParam = class(TAutoIntfObject, URepQueryParam)
  private
    FName : WideString;
    FDataType : BDEFieldType;
    FValue : OleVariant;
    FFileName : WideString;
  protected
    function  Get_DataType: BDEFieldType; safecall;
    procedure Set_DataType(Value: BDEFieldType); safecall;
    function  Get_Value: OleVariant; safecall;
    procedure Set_Value(Value: OleVariant); safecall;
    function  Get_Name: WideString; safecall;
    function  Get_FileName: WideString; safecall;
    procedure Set_FileName(const Value: WideString); safecall;
  public
    constructor Create(Name: WideString; Value: OleVariant; DataType: BDEFieldType; FileName: WideString);
  end;

procedure BindQueryOleParams(aQuery: TQuery; Params: OleVariant);

implementation

uses Impl_URepObjectFactory, Impl_URepCollection, URepConstants, ActivScp;

//========================================================================
// TURepQueryParams - collection of URepQueryParam
//========================================================================
constructor TURepQueryParams.Create;
begin
  inherited Create(GetURepTypeLib, URepQueryParamsDisp);
  FIntList := TInterfaceList.Create;
  FNamesQueryParam := TStringList.Create;
  FNamesQueryParam.Sorted := False;
end;

destructor TURepQueryParams.Destroy;
begin
  FIntList.Free;
  FNamesQueryParam.Free;
  inherited Destroy;
end;

function TURepQueryParams.ConvertIndex(V: OleVariant): Integer;
begin
  case VarType(V) of
    varSmallInt, varInteger, varSingle, varDouble, varCurrency:
      begin
      Result := V - 1;
      if (Result<0) or (Result>FIntList.Count-1) then
        raise Exception.Create(SInvalidIndexVal);
      end;
    varOleStr:
      begin
        Result := FNamesQueryParam.IndexOf(V);
        if (Result<0) then
         raise Exception.Create(SInvalidIndexVal);
      end;
    varUnknown:
      Result := FIntList.IndexOf(IUnknown(V));
    varDispatch:
      Result := FIntList.IndexOf(IDispatch(V) as IUnknown);
    else
      raise Exception.Create(SInvalidIndexTyp);
  end;
end;

function TURepQueryParams.Get__NewEnum: IUnknown;
begin
  result := TIEnumVariantEnumerator.Create(Self, FIntList, 0) as IUnknown;
end;

function TURepQueryParams.Get_Item(Index: OleVariant): URepQueryParam;
begin
  Result := FIntList.Items[ConvertIndex(Index)] as URepQueryParam;
end;

function TURepQueryParams.Get_Count: Integer;
begin
  result :=  FIntList.Count;
end;

function  TURepQueryParams.Add(const Name: WideString; Value: OleVariant; DataType: BDEFieldType;
                            const FileName: WideString): URepQueryParam;
var
  iIndex: integer;
begin
  iIndex := FNamesQueryParam.IndexOf(Name);
  if iIndex<0 then
  begin
    FNamesQueryParam.Add(Name);
    result :=  TURepQueryParam.Create(Name, Value, DataType, FileName);
    FIntList.Add(result);
  end
  else
    raise Exception.Create(SInvalidQueryParamName+Name);
end;

procedure TURepQueryParams.Clear;
begin
  FIntList.Clear;
  FNamesQueryParam.Clear;
end;


//========================================================================
// TURepQueryParam
//========================================================================
constructor TURepQueryParam.Create(Name: WideString; Value: OleVariant;
                                   DataType: BDEFieldType; FileName: WideString);
begin
  inherited Create(GetURepTypeLib, URepQueryParam);
  FName := Name;
  FValue := Value;
  FDataType := DataType;
  FFileName := FileName;
end;

function  TURepQueryParam.Get_DataType: BDEFieldType;
begin
  result := FDataType;
end;

procedure TURepQueryParam.Set_DataType(Value: BDEFieldType);
begin
  FDataType := Value;
end;

function  TURepQueryParam.Get_Value: OleVariant;
begin
  result := FValue;
end;

procedure TURepQueryParam.Set_Value(Value: OleVariant);
begin
  FValue := Value;
end;

function  TURepQueryParam.Get_Name: WideString;
begin
  result := FName;
end;

function  TURepQueryParam.Get_FileName: WideString; safecall;
begin
  result := FFileName;
end;

procedure TURepQueryParam.Set_FileName(const Value: WideString); safecall;
begin
  FFileName := Value;
end;


//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
procedure BindDictionaryParams(aQuery: TQuery; const i_IDictionary: IDictionary);
var
  i, iVarType: integer;
  Param: TParam;
  parName: WideString;
  varValue: OleVariant;
begin
  for i:=0 to aQuery.ParamCount-1 do
  begin
    Param := aQuery.Params[i];
    parName := Param.Name;
    varValue := parName;
    if i_IDictionary.Exists(varValue)=False then
    begin
      raise Exception.Create('Parametr with name <'+parName+'> NOT FOUND');
    end;
    varValue := i_IDictionary.Get_Item(varValue);
    iVarType := VarType(varValue);
    if not((iVarType=varEmpty) or (iVarType=varNull) or
           ((iVarType=varError) and (TVarData(varValue).VError = $80020004)) ) then
    begin
      Param.DataType := ftUnknown;
      Param.Value := varValue
    end
    else
    begin
      Param.DataType := ftString;
      Param.Clear;
      Param.Bound := True;
    end;
  end;
end;

procedure BindURepQueryParams(aQuery: TQuery; const i_URepQueryParams: URepQueryParams);
var
  i, iVarType: integer;
  param: TParam;
  parName: String;
  i_URepQueryParam: URepQueryParam;
  varValue: OleVariant;
begin
  for i:=0 to aQuery.ParamCount-1 do
  begin
    param := aQuery.Params[i];
    parName := Param.Name;
    i_URepQueryParam := i_URepQueryParams.Item[parName];

    if i_URepQueryParam.Get_FileName='' then
    begin
      varValue := i_URepQueryParam.Get_Value;
      iVarType := VarType(varValue);
      if not((iVarType=varEmpty) or (iVarType=varNull) or
             ((iVarType=varError) and (TVarData(varValue).VError = $80020004)) ) then
      begin
        //Param.DataType := ftUnknown;
        param.DataType := TFieldType(i_URepQueryParam.Get_DataType);
        Param.Value := varValue;
      end
      else
      begin
        Param.DataType := ftString;
        Param.Clear;
        Param.Bound := True;
      end;
    end
    else
    begin
      Param.DataType := ftUnknown;
      Param.LoadFromFile(i_URepQueryParam.Get_FileName, ftBlob);
    end;

  end;
end;


procedure BindQueryOleParams(aQuery: TQuery; Params: OleVariant);
var
  iVarType: integer;
  Params_IDispatch: IDispatch;
  Params_IDictionary: IDictionary;
  Params_URepQueryParams: URepQueryParams;
begin
  iVarType := VarType(Params);
  if not((iVarType=varEmpty) or (iVarType=varNull) or
         ((iVarType=varError) and (TVarData(Params).VError = $80020004)) ) then
  begin
    if iVarType=varDispatch then
    begin
      Params_IDispatch := Params;
      if Supports(Params_IDispatch, IID_IDictionary, Params_IDictionary) then
        BindDictionaryParams(aQuery, Params_IDictionary)
      else if Supports(Params_IDispatch, IID_URepQueryParams, Params_URepQueryParams) then
        BindURepQueryParams(aQuery, Params_URepQueryParams)
      else
        raise Exception.Create(SInvalidInterfaceType);
    end
    else
      raise Exception.Create(SInvalidDataType);
  end;
end;





end.
