unit Impl_URepQuery;

interface

uses
  Windows, Dialogs, Classes, SysUtils, Forms, Db, Dbtables,
  ComObj, ActiveX, URepGen_TLB, BdeUtils;

type
  TURepSimpleDataSet = class(TAutoIntfObject, URepQuery)
  private
  protected
    function InternalExportTo(const FileName: WideString; TableType: URepExportType; AsciiDelimited: WordBool): WordBool; virtual;

    function Get_Numm: Integer; safecall;
    function Get_ShortName: WideString; safecall;
    function Get_Name: WideString; safecall;
    function Get_RecordCount: Integer; safecall;
    function Get_RecNo: Integer; safecall;
    procedure Set_RecNo(Value: Integer); safecall;
    function Get_CurrRecord: OleVariant; safecall;
    function Get_Values: OleVariant; safecall;
    function Get_BOF: WordBool; safecall;
    function Get_EOF: WordBool; safecall;
    procedure First; safecall;
    procedure Next; safecall;
    procedure Last; safecall;
    procedure Prior; safecall;
    function MoveBy(Distance: Integer): Integer; safecall;
    function Get_FieldCount: Integer; safecall;
    function Get_Fields: URepFields; safecall;
    procedure Close; safecall;
    function Get_DisplayLabels: OleVariant; safecall;
    function Get_FieldNames: OleVariant; safecall;
    function ExportTo(const FileName: WideString; TableType: URepExportType; AsciiDelimited: WordBool): WordBool; safecall;
    function Get_LabeledValues: OleVariant; safecall;
  public
    FNumm: integer;
    FFullName: string;
    FDataSet: TDataSet;

    constructor Create(aDataSet: TDataSet);
  end;

  TURepQuery = class(TURepSimpleDataSet)
  private
    FBDEDataSet: TBDEDataSet;
  protected
    function InternalExportTo(const FileName: WideString; TableType: URepExportType; AsciiDelimited: WordBool): WordBool; override;
  public
    constructor Create(aBDEDataSet: TBDEDataSet);
  end;

  TURepSingleDataSet = class(TURepSimpleDataSet)
  public
    destructor Destroy; override;
  end;

  TURepSingleQuery = class(TURepQuery)
  public
    destructor Destroy; override;
  end;

implementation

uses Impl_URepObjectFactory, Impl_URepFields, URepConstants;

//========================================================================
// TURepSimpleDataSet
//========================================================================
constructor TURepSimpleDataSet.Create(aDataSet: TDataSet);
begin
  inherited Create(GetURepTypeLib, URepQueryDisp);
  FDataSet := aDataSet;
end;

function TURepSimpleDataSet.Get_Numm: Integer;
begin
  result := FNumm;
end;

function TURepSimpleDataSet.Get_ShortName: WideString;
begin
  result := FDataSet.Name;
end;

function TURepSimpleDataSet.Get_Name: WideString;
begin
  result := FFullName;
end;

function TURepSimpleDataSet.Get_RecordCount: Integer;
begin
  result := FDataSet.RecordCount;
end;

function TURepSimpleDataSet.Get_RecNo: Integer;
begin
  result := FDataSet.RecNo;
end;

procedure TURepSimpleDataSet.Set_RecNo(Value: Integer);
begin
  FDataSet.RecNo := Value;
end;

function TURepSimpleDataSet.Get_CurrRecord: OleVariant;
var
  arCurrRecord : OleVariant;
  i: integer;
begin
  result := unassigned;
  arCurrRecord := VarArrayCreate([1, FDataSet.FieldCount], varVariant);
  for i:=1 to FDataSet.FieldCount do
    arCurrRecord[i] := FDataSet.Fields[i-1].Value;
  result := arCurrRecord;
end;

function TURepSimpleDataSet.Get_Values: OleVariant;
var
  arValues : OleVariant;
  strBkmrk : string;
  i: integer;
begin
  result := unassigned;
  strBkmrk := FDataSet.Bookmark;
  try
    arValues := VarArrayCreate([1, FDataSet.RecordCount], varVariant);
    FDataSet.First; i:= 1;
    while not FDataSet.EOF do
    begin
      arValues[i] := Get_CurrRecord;
      FDataSet.Next; Inc(i);
    end;
  finally
    FDataSet.Bookmark := strBkmrk;
  end;
  result := arValues;
end;

function TURepSimpleDataSet.Get_BOF: WordBool;
begin
  result := FDataSet.BOF;
end;

function TURepSimpleDataSet.Get_EOF: WordBool;
begin
  result := FDataSet.EOF;
end;

procedure TURepSimpleDataSet.First;
begin
  FDataSet.First;
end;

procedure TURepSimpleDataSet.Next;
begin
  FDataSet.Next;
end;

procedure TURepSimpleDataSet.Last;
begin
  FDataSet.Last;
end;

procedure TURepSimpleDataSet.Prior;
begin
  FDataSet.Prior;
end;

function TURepSimpleDataSet.MoveBy(Distance: Integer): Integer;
begin
  result := FDataSet.MoveBy(Distance);
end;

function TURepSimpleDataSet.Get_FieldCount: Integer;
begin
  result := FDataSet.FieldCount;
end;

function TURepSimpleDataSet.Get_Fields: URepFields;
begin
  result := TURepFields.Create(FDataSet);
end;

function TURepSimpleDataSet.Get_DisplayLabels: OleVariant;
var
  arLabels : OleVariant;
  i: integer;
begin
  result := unassigned;
  arLabels := VarArrayCreate([1, FDataSet.FieldCount], varVariant);
  for i:=1 to FDataSet.FieldCount do
    arLabels[i] := FDataSet.Fields[i-1].DisplayLabel;
  result := arLabels;
end;

function TURepSimpleDataSet.Get_FieldNames: OleVariant;
var
  arNames : OleVariant;
  i: integer;
begin
  result := unassigned;
  arNames := VarArrayCreate([1, FDataSet.FieldCount], varVariant);
  for i:=1 to FDataSet.FieldCount do
    arNames[i] := FDataSet.Fields[i-1].FieldName;
  result := arNames;
end;

procedure TURepSimpleDataSet.Close;
begin
  FDataSet.Close;
end;

function TURepSimpleDataSet.Get_LabeledValues: OleVariant;
var
  arLabeledValues, arCurrRecord : OleVariant;
  strBkmrk : string;
  i, j: integer;
begin
  result := unassigned;
  strBkmrk := FDataSet.Bookmark;
  try
    arLabeledValues := VarArrayCreate([1, FDataSet.RecordCount+1], varVariant);
    arLabeledValues[1] := Get_DisplayLabels;

    FDataSet.First; i:= 2;
    while not FDataSet.EOF do
    begin
      arCurrRecord := VarArrayCreate([1, FDataSet.FieldCount], varVariant);
      for j:=1 to FDataSet.FieldCount do
        arCurrRecord[j] := FDataSet.Fields[j-1].asString;

      arLabeledValues[i] := arCurrRecord;
      FDataSet.Next; Inc(i);
    end;
  finally
    FDataSet.Bookmark := strBkmrk;
  end;
  result := arLabeledValues;
end;

function ConvTableType(TableType: URepExportType) : TTableType;
begin
  result := ttDefault;
  case TableType of
    exptDBase   : result := ttDBase;
    exptParadox : result := ttParadox;
    exptFoxPro  : result := ttFoxPro;
    exptASCII   : result := ttASCII;
  end;
end;

function TURepSimpleDataSet.ExportTo(const FileName: WideString; TableType: URepExportType; AsciiDelimited: WordBool) : WordBool;
begin
  result := InternalExportTo(FileName, TableType, AsciiDelimited);
end;

function TURepSimpleDataSet.InternalExportTo(const FileName: WideString; TableType: URepExportType; AsciiDelimited: WordBool) : WordBool;
var
  //DestTbl: TTable;
  CellFrom, CellTo : string;
  xlApplication, xlWorkbooks, xlWorkbook, xlSheet : OleVariant;
begin
  result := False;
  if TableType=exptExcel then
  begin
    xlApplication := CreateOLEObject('Excel.Application');
    if VarType(xlApplication) <> VarDispatch then
      begin
      MessageDlg('Excel  !',mtError,[mbAbort],0);
      Exit;
      end;
    try
      xlWorkbooks := xlApplication.WorkBooks;
      xlWorkbook := xlWorkbooks.Add;
      xlSheet := xlWorkbook.WorkSheets.Add;
      xlSheet.Name := Get_ShortName;

      CellFrom := 'A1';
      CellTo   := Char(Word('A')+Get_FieldCount-1)+
                  IntToStr(Get_RecordCount+1);
      xlSheet.Range[CellFrom, CellTo].FormulaArray  := Get_LabeledValues;
      xlSheet.Cells[1,1].Select;
      xlSheet.Rows[1].Font.Bold:=true;
      xlSheet.Columns.AutoFit;
      try
        xlWorkbook.SaveAs(FileName,xlWorkbookNormal, EmptyParam, EmptyParam, False,False,
                          xlNoChange, xlLocalSessionChanges,true,0,0)
      except
      end;
      xlWorkbook.Saved := True;
      xlWorkbook.Close;
    finally
        xlApplication.Quit;
    end;
    result:= True;
  end
end;
//========================================================================
// TURepQuery
//========================================================================

constructor TURepQuery.Create(aBDEDataSet: TBDEDataSet);
begin
  FBDEDataSet := aBDEDataSet;
  inherited Create(aBDEDataSet);
end;

function TURepQuery.InternalExportTo(const FileName: WideString; TableType: URepExportType; AsciiDelimited: WordBool) : WordBool;
var
  DestTbl: TTable;
begin
  result := Inherited InternalExportTo(FileName,TableType,AsciiDelimited);
  if TableType<>exptExcel then
  begin
    DestTbl := TTable.Create(nil);
    DestTbl.TableName := FileName;
    try
      ExportDataSet(FBDEDataSet, DestTbl, ConvTableType(TableType), 'ascii', AsciiDelimited, 0); //'db866ru0'
    finally
      DestTbl.Free;
    end;
    result:= True;
  end;
end;

//========================================================================
// TURepSingleQuery
//========================================================================
destructor TURepSingleQuery.Destroy;
begin
  FDataSet.Free;
  inherited Destroy;
end;

//========================================================================
// TURepSingleDataSet
//========================================================================
destructor TURepSingleDataSet.Destroy;
begin
  FDataSet.Free;
  inherited Destroy;
end;


end.
