unit Impl_URepQuerys;

interface

uses
  Windows, Dialogs, Classes, SysUtils, Forms, Db, Dbtables,
  ComObj, ActiveX, URepGen_TLB, AskMe, BdeUtils;

type
  TURepQuerys = class(TAutoIntfObject, URepQuerys)
  private
    FIntList: TInterfaceList;    // list of objects in collection
    function ConvertIndex(V: OleVariant): Integer;
  protected
    function Get__NewEnum: IUnknown; safecall;
    function Get_Item(Index: OleVariant): URepQuery; safecall;
    function Get_Count: Integer; safecall;
    function ExportToExcel(const FileName: WideString): WordBool; safecall;
  public
    constructor Create(QueryList: TQueryList; HiddenForm: TForm);
    destructor Destroy; override;
  end;


implementation

uses Impl_URepObjectFactory, Impl_URepCollection, Impl_URepQuery, URepConstants;

//========================================================================
// TURepQuerys - collection of URepQuery
//========================================================================
constructor TURepQuerys.Create(QueryList: TQueryList; HiddenForm: TForm);
var
  i : integer;
  queryListItem : TQueryListItem;
  aDataSet: TDataSet;
  aURepQuery: TURepSimpleDataSet;
begin
  inherited Create(GetURepTypeLib, URepQuerysDisp);
  FIntList := TInterfaceList.Create;

  for i:=0 to QueryList.Count-1 do
    begin
    queryListItem := TQueryListItem(QueryList.Items[i]);
    aDataSet := queryListItem.FindDataSet(HiddenForm);
    if aDataSet is TBDEDataSet Then
      aURepQuery := TURepQuery.Create(aDataSet as TBDEDataSet)
    else
      aURepQuery := TURepSimpleDataSet.Create(aDataSet);
    aURepQuery.FNumm := i + 1;
    aURepQuery.FFullName := queryListItem.name;
    FIntList.Add(aURepQuery as IUnknown);  //!!!vv
    end;
end;

destructor TURepQuerys.Destroy;
begin
  FIntList.Free;
  inherited Destroy;
end;

function TURepQuerys.ConvertIndex(V: OleVariant): Integer;
var
  i : integer;
  bFind : Boolean;
begin
  Result := 0;
  case VarType(V) of
    varSmallInt, varInteger, varSingle, varDouble, varCurrency:
      begin
      Result := V - 1;
      if (Result<0) or (Result>FIntList.Count-1) then
        raise Exception.Create(SInvalidIndexVal);
      end;
    varOleStr:
      begin
        bFind := False;
        for i:=0 to FIntList.Count-1 do
          if ((FIntList.Items[i] as URepQuery).ShortName)=V then
          begin
            bFind := True;
            Result := i;
            Break;
          end;
        if not bFind then
         raise Exception.Create(SInvalidIndexVal);
      end;
    varUnknown:
      Result := FIntList.IndexOf(IUnknown(V));
    varDispatch:
      Result := FIntList.IndexOf(IDispatch(V) as IUnknown);
    else
      raise Exception.Create(SInvalidIndexTyp);
  end;
end;

function TURepQuerys.Get__NewEnum: IUnknown;
begin
  result := TIEnumVariantEnumerator.Create(Self, FIntList, 0) as IUnknown;
end;

function TURepQuerys.Get_Item(Index: OleVariant): URepQuery;
begin
  Result := FIntList.Items[ConvertIndex(Index)] as URepQuery;
end;

function TURepQuerys.Get_Count: Integer;
begin
  result :=  FIntList.Count;
end;

function TURepQuerys.ExportToExcel(const FileName: WideString): WordBool;
var
  i: integer;
  oURepQuery: URepQuery;
  CellFrom, CellTo : string;
  xlApplication, xlWorkbooks, xlWorkbook, xlSheet : OleVariant;
begin
  result := False;
  xlApplication := CreateOLEObject('Excel.Application');
  if VarType(xlApplication) <> VarDispatch then
    begin
    MessageDlg('Excel  !',mtError,[mbAbort],0);
    Exit;
    end;

  try
    xlWorkbooks := xlApplication.WorkBooks;
    xlWorkbook := xlWorkbooks.Add;

    for i:=1 to FIntList.Count do
      begin
      oURepQuery := Get_Item(i);
      if i=1 then
        xlSheet := xlWorkbook.WorkSheets.Add
      else
        xlSheet := xlWorkbook.WorkSheets.Add(After := xlApplication.ActiveSheet);
      xlSheet.Name := oURepQuery.ShortName;

      CellFrom := 'A1';
      CellTo   := Char(Word('A')+oURepQuery.FieldCount-1)+
                  IntToStr(oURepQuery.RecordCount+1);
      xlSheet.Range[CellFrom, CellTo].FormulaArray  := oURepQuery.LabeledValues;
      oURepQuery := nil;
      xlSheet.Cells[1,1].Select;
      xlSheet.Rows[1].Font.Bold:=true;
      xlSheet.Columns.AutoFit;
      end;
    try
      xlWorkbook.SaveAs(FileName,xlWorkbookNormal, EmptyParam, EmptyParam, False,False,
                        xlNoChange, xlLocalSessionChanges,true,0,0)
    except
    end;
    xlWorkbook.Saved := True;
    xlWorkbook.Close;
  finally
    xlApplication.Quit;
  end;
  result := True;
end;

end.
