unit ImportActScrSite;

interface
uses
  Windows, Classes, SysUtils, Forms, Dialogs,
  ComObj, ActiveX, ActivScp, URepGen_TLB, DBmwEdit;

type

  IDataImport = interface
    ['{1D411824-5099-11D3-9931-00C0DFE97D5A}']
    procedure DataImport(FileName: WideString); stdcall;
  end;

  TImportActiveScriptSite = class(TInterfacedObject,
                                IActiveScriptSite, IActiveScriptSiteWindow, IDataImport)
  private
    FGLURepDataImport : URepDataImport;
    FmwEdit: TDBmwEdit;
  protected
    // IActiveScriptSite
    function GetLCID(out plcid: UINT): HResult; stdcall;
    function GetItemInfo(pstrName: PWideChar; dwReturnMask: UINT; out ppiunkItem: IUnknown;
                         out ppti: ITypeInfo): HResult; stdcall;
    function GetDocVersionString(out pbstrVersion: WideString): HResult; stdcall;
    function OnScriptTerminate(var pvarResult: OleVariant; var pexcepinfo: EXCEPINFO): HResult; stdcall;
    function OnStateChange(ssScriptState: tagSCRIPTSTATE): HResult; stdcall;
    function OnScriptError(const pscripterror: IActiveScriptError): HResult; stdcall;
    function OnEnterScript: HResult; stdcall;
    function OnLeaveScript: HResult; stdcall;
    // IActiveScriptSiteWindow
    function GetWindow(out wnd: HWnd): HResult; stdcall;
    function EnableModeless(fEnable: Integer): HResult; stdcall;
    //IDataImport
    procedure DataImport(FileName: WideString); stdcall;
  public
    constructor Create(IdImport: integer; mwEdit: TDBmwEdit);
    destructor Destroy; override;
  end;

const
  GlObjURepDataImportName = 'GlobalURepDataImport';

procedure RunImportScript(IdImport: integer; FileName: WideString; mwEdit: TDBmwEdit=nil);

implementation

uses Impl_URepImports, fURepFunc0, URepTypeLibs;


procedure RunImportScript(IdImport: integer; FileName: WideString; mwEdit: TDBmwEdit=nil);
var
  i_IDataImport: IDataImport;
begin
  i_IDataImport := TImportActiveScriptSite.Create(IdImport, mwEdit);
  i_IDataImport.DataImport(FileName);
  i_IDataImport := nil;
end;

procedure TImportActiveScriptSite.DataImport(FileName: WideString);
var
  i_IActiveScriptSite: IActiveScriptSite;
  i_IActiveScript: IActiveScript;
  i_IActiveScriptParse: IActiveScriptParse;
  sScrEngName, sScriptText : WideString;
  varResult: OleVariant;
  ExcepInfo: TExcepInfo;
begin
//  
  sScrEngName := FGLURepDataImport.ScrEngName;
  sScriptText := FGLURepDataImport.ScriptText;
  FGLURepDataImport.FileName := FileName;
  i_IActiveScriptSite := Self;
  OleCheck(GetIActiveScript( Trim(sScrEngName), i_IActiveScript));
  i_IActiveScriptParse := i_IActiveScript as IActiveScriptParse;
  OleCheck(i_IActiveScript.SetScriptSite(i_IActiveScriptSite));
  OleCheck(i_IActiveScriptParse.InitNew);
  OleCheck(i_IActiveScript.AddNamedItem(PWideChar(WideString(GlObjURepDataImportName)),
                               SCRIPTITEM_ISVISIBLE or SCRIPTITEM_ISSOURCE));

  //OleCheck(i_IActiveScript.AddTypeLib(LIBID_URepGen, URepGenMajorVersion, URepGenMinorVersion, 0));
  //OleCheck(i_IActiveScript.AddTypeLib(LIBID_Scripting, ScriptingMajorVersion, ScriptingMinorVersion, 0));
  ActiveScript_AddTypeLib(i_IActiveScript, sScriptText, sScrEngName);
  //
  i_IActiveScriptParse.ParseScriptText(PWideChar(sScriptText),
                                  PWideChar(WideString(GlObjURepDataImportName)),
                                  nil, nil, 0, 0, 0, varResult, ExcepInfo);
  URepLogAction(4, FGLURepDataImport.Id, FGLURepDataImport.Name); // log
  OleCheck(i_IActiveScript.SetScriptState(SCRIPTSTATE_CONNECTED));
  OleCheck(i_IActiveScript.Close);
  i_IActiveScript := nil;
  i_IActiveScriptParse := nil;
  i_IActiveScriptSite := nil;
end;


// IActiveScriptSite
constructor TImportActiveScriptSite.Create(IdImport: integer; mwEdit: TDBmwEdit);
begin
  //ShowMessage('TImportActiveScriptSite.Create');
  inherited Create;
  FGLURepDataImport := TURepDataImport.Create(IdImport);
  FmwEdit := mwEdit;
end;

destructor TImportActiveScriptSite.Destroy;
begin
  //ShowMessage('TImportActiveScriptSite.Destroy');
  FGLURepDataImport := nil;
  inherited Destroy;
end;

function TImportActiveScriptSite.GetLCID(out plcid: UINT): HResult;
begin
  result := E_NOTIMPL;
end;

function TImportActiveScriptSite.GetItemInfo(pstrName: PWideChar; dwReturnMask: UINT; out ppiunkItem: IUnknown;
                         out ppti: ITypeInfo): HResult;
var
  wstrName: WideString;
begin
  result := TYPE_E_ELEMENTNOTFOUND;

  if (dwReturnMask and SCRIPTINFO_ITYPEINFO)>0 then
    begin
    ppti := nil;
    result := TYPE_E_ELEMENTNOTFOUND;
    Exit;
    end;

  if (dwReturnMask and SCRIPTINFO_IUNKNOWN)>0  then
    begin
    ppiunkItem := nil;
    wstrName := pstrName;
    if wstrName=GlObjURepDataImportName then
      begin
      ppiunkItem := FGLURepDataImport;
      result := S_OK;
      end
    else
      result := TYPE_E_ELEMENTNOTFOUND;
    end;
end;

function TImportActiveScriptSite.GetDocVersionString(out pbstrVersion: WideString): HResult;
begin
  result := E_NOTIMPL;
end;

function TImportActiveScriptSite.OnScriptTerminate(var pvarResult: OleVariant; var pexcepinfo: EXCEPINFO): HResult;
begin
  result := S_OK;
end;

function TImportActiveScriptSite.OnStateChange(ssScriptState: tagSCRIPTSTATE): HResult;
begin
  result := S_OK;
end;

function TImportActiveScriptSite.OnScriptError(const pscripterror: IActiveScriptError): HResult;
var
  ExcepInfo: TExcepInfo;
  SourceContext: UINT;
  LineNumber: UINT;
  CharacterPosition: Integer;
  ErrMsg: String;
  hhResult: HResult;
  SourceLineTxt: WideString;
  pointXY: TPoint;
begin
  ErrMsg:= '';
  hhResult := pscripterror.GetSourceLineText(SourceLineTxt);
  If Succeeded(hhResult) Then
  begin
    ErrMsg:= SourceLineTxt + Chr(13)+Chr(10);
  end;
  pscripterror.GetExceptionInfo(ExcepInfo);
  pscripterror.GetSourcePosition(SourceContext, LineNumber, CharacterPosition);

  ErrMsg:= ErrMsg + ': '+ExcepInfo.bstrSource + Chr(13)+Chr(10)+
          ': '+ IntToStr(LineNumber+1) + ' : '+IntToStr(CharacterPosition+1)+ Chr(13)+Chr(10)+
          ExcepInfo.bstrDescription;
  ShowMessage(ErrMsg);
  If assigned(FmwEdit) then
  begin
    If FmwEdit.CanFocus Then FmwEdit.SetFocus;
    pointXY.x := CharacterPosition+1;
    pointXY.Y := LineNumber+1;
    FmwEdit.CaretXY := pointXY;
  end;
  result := S_OK;
end;

function TImportActiveScriptSite.OnEnterScript: HResult;
begin
  result := S_OK;
end;

function TImportActiveScriptSite.OnLeaveScript: HResult;
begin
  result := S_OK;
end;

// IActiveScriptSiteWindow
function TImportActiveScriptSite.GetWindow(out wnd: HWnd): HResult;
begin
  wnd := Forms.Application.Handle;
  result := S_OK;
end;

function TImportActiveScriptSite.EnableModeless(fEnable: Integer): HResult;
begin
 result := S_OK;
end;

end.
