{ :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  :: QuickReport 3.0 for Delphi and C++Builder               ::
  ::                                                         ::
  :: QRPREV.PAS - QUICKREPORT STANDARD PREVIEW FORM          ::
  ::                                                         ::
  :: Copyright (c) 1998 QuSoft AS                            ::
  :: All Rights Reserved                                     ::
  ::                                                         ::
  :: web: http://www.qusoft.no                               ::
  ::                                                         ::
  ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: }


unit Qrprevr;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, StdCtrls, Buttons, QRPrntr, QR3const, ComCtrls, ToolWin, Menus, ImgList,
  RXSpin, RXCtrls, Spin;

type
  TQRRussianPreview = class(TForm)
    StatusBar: TStatusBar;
    Images: TImageList;
    QRPreview: TQRPreview;
    ToolBar1: TToolBar;
    ToolButton4: TToolButton;
    ZoomFit: TToolButton;
    Zoom100: TToolButton;
    ZoomToWidth: TToolButton;
    ToolButton3: TToolButton;
    ToolButton5: TToolButton;
    FirstPage: TToolButton;
    PreviousPage: TToolButton;
    ToolButton2: TToolButton;
    LastPage: TToolButton;
    Separator2: TToolButton;
    PrintSetup: TToolButton;
    Print: TToolButton;
    Separator3: TToolButton;
    SaveReport: TToolButton;
    LoadReport: TToolButton;
    Separator4: TToolButton;
    ExitButton: TSpeedButton;
    RxSpinButton1: TRxSpinButton;
    spbtnScale: TRxSpeedButton;
    PopupMenu1: TPopupMenu;
    mnu500: TMenuItem;
    mnu200: TMenuItem;
    mnu100: TMenuItem;
    mnu75: TMenuItem;
    mnu50: TMenuItem;
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure ZoomToFitClick(Sender: TObject);
    procedure ZoomTo100Click(Sender: TObject);
    procedure ZoomToWidthClick(Sender: TObject);
    procedure FirstPageClick(Sender: TObject);
    procedure PrevPageClick(Sender: TObject);
    procedure NextPageClick(Sender: TObject);
    procedure LastPageClick(Sender: TObject);
    procedure PrintClick(Sender: TObject);
    procedure ExitClick(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure SaveClick(Sender: TObject);
    procedure PrintSetupClick(Sender: TObject);
    procedure LoadClick(Sender: TObject);
    procedure QRPreviewPageAvailable(Sender: TObject; PageNum: Integer);
    procedure QRPreviewProgressUpdate(Sender: TObject; Progress: Integer);
    procedure FormCreate(Sender: TObject);
    procedure FormDeactivate(Sender: TObject);
    procedure RxSpinButton1TopClick(Sender: TObject);
    procedure RxSpinButton1BottomClick(Sender: TObject);
    procedure mnuClick(Sender: TObject);
  private
    FZoomUpdating: Boolean;
    FQRPrinter : TQRPrinter;
    Gauge : TProgressBar;
    LastProgress : integer;
    FPreviewClosed: Boolean;
    procedure UpdateZoomState;
  public
    constructor CreatePreview(AOwner : TComponent; aQRPrinter : TQRPrinter); virtual;
    procedure Show;
    procedure UpdateInfo;
    property QRPrinter : TQRPrinter read FQRPrinter write FQRPrinter;
  end;

implementation

{$R *.DFM}

constructor TQRRussianPreview.CreatePreview(AOwner : TComponent; aQRPrinter : TQRPrinter);
begin
  inherited Create(AOwner);
  FZoomUpdating := False;
  QRPrinter := aQRPrinter;
  WindowState := wsMaximized;
  QRPreview.QRPrinter := aQRPrinter;
  if (QRPrinter <> nil) and (QRPrinter.Title <> '') then Caption := QRPrinter.Title;
  Gauge := TProgressBar.Create(Self);
//  Gauge.Parent := self;//
  Gauge.Top := 2;
  Gauge.Left := 2;
  Gauge.Height := 10;//
  Gauge.Width := 100;
  LastProgress := 0;
  ZoomToWidth.Down := true;
end;

procedure TQRRussianPreview.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  if not FPreviewClosed then
  begin
    QRPrinter.ClosePreview(Self);
    FPreviewClosed := True;
  end;
  Action := caFree;
end;

procedure TQRRussianPreview.UpdateInfo;
begin
  StatusBar.Panels[1].Text := SqrPage + ' ' + IntToStr(QRPreview.PageNumber) + ' ' +
    SqrOf + ' ' + IntToStr(QRPreview.QRPrinter.PageCount);
end;

procedure TQRRussianPreview.UpdateZoomState;
begin
  if not FZoomUpdating then
    begin
    FZoomUpdating := True;
    if assigned(QRPreview) then
      spbtnScale.Caption := IntToStr(QRPreview.Zoom)+'%';
    FZoomUpdating := False;
    end;
end;

procedure TQRRussianPreview.ZoomToFitClick(Sender: TObject);
begin
  Application.ProcessMessages;
  QRPreview.ZoomToFit;
  UpdateZoomState;
end;

procedure TQRRussianPreview.ZoomTo100Click(Sender: TObject);
begin
  Application.ProcessMessages;
  QRPreview.Zoom := 100;
  UpdateZoomState;
end;

procedure TQRRussianPreview.ZoomToWidthClick(Sender: TObject);
begin
  Application.ProcessMessages;
  QRPreview.ZoomToWidth;
  UpdateZoomState;
end;

procedure TQRRussianPreview.FirstPageClick(Sender: TObject);
begin
  QRPreview.PageNumber := 1;
  UpdateInfo;
end;

procedure TQRRussianPreview.PrevPageClick(Sender: TObject);
begin
  QRPreview.PageNumber := QRPreview.PageNumber - 1;
  UpdateInfo;
end;

procedure TQRRussianPreview.NextPageClick(Sender: TObject);
begin
  QRPreview.PageNumber := QRPreview.PageNumber + 1;
  UpdateInfo;
end;

procedure TQRRussianPreview.LastPageClick(Sender: TObject);
begin
  QRPreview.PageNumber := QRPrinter.PageCount;
  UpdateInfo;
end;

procedure TQRRussianPreview.PrintClick(Sender: TObject);
begin
  QRPrinter.Print;
end;

procedure TQRRussianPreview.ExitClick(Sender: TObject);
begin
  Close;
end;

procedure TQRRussianPreview.FormResize(Sender: TObject);
begin
  QRPreview.UpdateZoom;
  UpdateZoomState;
end;

procedure TQRRussianPreview.Show;
begin
{  QRPreview.QRPrinter.ShowingPreview:=true;}
  inherited Show;
  UpdateInfo;
  UpdateZoomState;
end;

procedure TQRRussianPreview.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  case Key of
    VK_Next : if Shift = [ssCtrl] then
                LastPageClick(Self)
              else
                NextPageClick(Self);
    VK_Prior : if Shift = [ssCtrl] then
                 FirstPageClick(Self)
               else
                 PrevPageClick(Self);
    VK_Home : FirstPageClick(Self);
    VK_End : LastPageClick(Self);
  end;
end;

procedure TQRRussianPreview.SaveClick(Sender: TObject);
var
  aExportFilter : TQRExportFilter;
{$ifndef win32}
  FileExt : string;
  I : integer;
{$endif}
begin
  aExportFilter := nil;
  with TSaveDialog.Create(Application) do
  try
    Title := SqrSaveReport;
    Filter := QRExportFilterLibrary.SaveDialogFilterString;
    DefaultExt := cQRPDefaultExt;
    if Execute then
    begin
    {$ifdef win32}
      if FilterIndex = 1 then
        QRPrinter.Save(Filename)
      else
      begin
        try
          aExportFilter := TQRExportFilterLibraryEntry(
            QRExportFilterLibrary.Filters[FilterIndex - 2]).ExportFilterClass.Create(Filename);
          QRPrinter.ExportToFilter(aExportFilter);
        finally
          aExportFilter.Free
        end
      end
    {$else}
      FileExt := ExtractFileExt(Filename);
      if copy(FileExt, 1, 1) = '.' then delete(FileExt, 1, 1);
      if (FileExt = '') or (FileExt = cQRPDefaultExt) then
        QRPrinter.Save(Filename)
      else
      begin
        for I := 0 to QRExportFilterLibrary.Filters.Count - 1 do
        begin
          if TQRExportFilterLibraryEntry(QRExportFilterLibrary.Filters[I]).Extension = FileExt then
          try
            aExportFilter := TQRExportFilterLibraryEntry(
            QRExportFilterLibrary.Filters[I]).ExportFilterClass.Create(Filename);
            QRPrinter.ExportToFilter(aExportFilter);
          finally
            aExportFilter.Free;
          end;
        end;
      end;
    {$endif}
    end;
  finally
    Free;
  end;
end;

procedure TQRRussianPreview.PrintSetupClick(Sender: TObject);
begin
  QRPrinter.PrintSetup;
end;

procedure TQRRussianPreview.LoadClick(Sender: TObject);
begin
  with TOpenDialog.Create(Application) do
  try
    Title := SqrLoadReport;
    Filter := SqrQRFile + ' (*.' +cQRPDefaultExt + ')|*.' + cqrpDefaultExt;
    if Execute then
      if FileExists(FileName) then
      begin
        QRPrinter.Load(Filename);
        QRPreview.PageNumber := 1;
        QRPreview.PreviewImage.PageNumber := 1;
        UpdateInfo;
        PrintSetup.Enabled := False;
      end
      else
        ShowMessage(SqrFileNotExist);
  finally
    free;
  end;
end;

procedure TQRRussianPreview.QRPreviewPageAvailable(Sender: TObject;
  PageNum: Integer);
begin
  UpdateInfo;
  UpdateZoomState;
end;

procedure TQRRussianPreview.QRPreviewProgressUpdate(Sender: TObject;
  Progress: Integer);
begin
  if Progress >= LastProgress + 5 then
  begin
    StatusBar.Panels[0].Text := IntToStr(Progress)+'%';
    LastProgress := Progress;
  end;
//  Gauge.Position := Progress;
  if (Progress = 0) or (Progress = 100) then StatusBar.Panels[0].Text := '';
end;

procedure TQRRussianPreview.FormCreate(Sender: TObject);
begin
{$ifdef win32}
   HorzScrollbar.Tracking := true;
   VertScrollbar.Tracking := true;
{$endif}
end;

procedure TQRRussianPreview.FormDeactivate(Sender: TObject);
var
  i: integer;
  Find: Boolean;
begin
  Find := False;
  if Screen.ActiveForm=Application.MainForm then
    Find := True;;
  if  not Find then
    for i:=0 to Application.MainForm.MDIChildCount-1 do
     if  Screen.ActiveForm=Application.MainForm.MDIChildren[i] then
       begin
       Find := True;
       Break;
       end;
  if Find then
    Close;
end;

procedure TQRRussianPreview.RxSpinButton1TopClick(Sender: TObject);
begin
  Application.ProcessMessages;
  QRPreview.Zoom := QRPreview.Zoom + 10;
  ZoomFit.Down := False;
  Zoom100.Down := False;
  ZoomToWidth.Down := False;
  UpdateZoomState;
end;

procedure TQRRussianPreview.RxSpinButton1BottomClick(Sender: TObject);
begin
  Application.ProcessMessages;
  QRPreview.Zoom := QRPreview.Zoom - 10;
  ZoomFit.Down := False;
  Zoom100.Down := False;
  ZoomToWidth.Down := False;
  UpdateZoomState;
end;

procedure TQRRussianPreview.mnuClick(Sender: TObject);
begin
  QRPreview.Zoom := (Sender as TMenuItem).Tag;
  ZoomFit.Down := False;
  Zoom100.Down := False;
  ZoomToWidth.Down := False;
  UpdateZoomState;
end;

end.
