unit URepTypeLibs;

interface

uses Windows, Messages, SysUtils, classes, ComObj, ActiveX, ActivScp;

Type
  TReferTypeLibUseType = (tlbNotUse, tlbRegstrUse, tlbScriptUse, tlbInternalUse);

  PReferTypeLibItem = ^TReferTypeLibItem;
  TReferTypeLibItem = record
    Name: String;
    GUID: TGUID;
    VerMajor : integer;
    VerMinor : integer;
    Flags: integer;
    lcid: integer;
    Path: string;
    UseType: TReferTypeLibUseType;
    NewUseType: TReferTypeLibUseType;
  end;

const
  cnstScriptTypeLibs = 'ScriptTypeLibs';

Procedure ClearAndNilReferTypeLibItemList(var refTLList: TList);
function LoadFullReferTypeLibItemList(refTLList: TList; ScriptText,CommentStr: string): integer;

Function AddNewTypeLibItemData(refTLList: TList; Name: string;
                               GUID: TGUID; VerMajor, VerMinor: integer;
                               UseType:TReferTypeLibUseType): PReferTypeLibItem;
function FindRefferTypeLib(findTLItem: PReferTypeLibItem) : PReferTypeLibItem;

function LoadReferedTypeLibs(ScriptText, CommentStr: string): integer;
procedure ActiveScript_AddTypeLib(i_IActiveScript: IActiveScript; ScriptText, ScrEngName: string);
function IdScrEngToCommentStr(IdScrEng: integer): string;

function CreateScrReferTypeLibStr(Comment:string; GUID:TGUID; verMajor, verMinor: integer): string;
function CrackScrReferTypeLibStr(ScrReferTypeLibStr: string; var GUID:TGUID; var verMajor, verMinor: integer): Boolean;

var
  ReferedTypeLibItemList: TList; // "{GUID}=major.minor" in Reg

implementation

uses AppUtils, URepGen_TLB, URepConstants, dmdURep;

procedure ActiveScript_AddTypeLib(i_IActiveScript: IActiveScript; ScriptText, ScrEngName: string);
var
  i: integer;
  pReferTLItem : PReferTypeLibItem;
  CommentStr: string;
begin
  if not dmURep.qScrEng.Active then dmURep.qScrEng.Open;
  dmURep.qScrEng.Locate('Item', ScrEngName, []);
  CommentStr := dmURep.qScrEngComment.Value;

  LoadReferedTypeLibs(ScriptText, CommentStr);

  for i:=0 to ReferedTypeLibItemList.Count-1 do
  begin
    pReferTLItem := PReferTypeLibItem(ReferedTypeLibItemList.Items[i]);
    If pReferTLItem.UseType in [tlbScriptUse, tlbInternalUse] then
      OleCheck(i_IActiveScript.AddTypeLib(pReferTLItem.GUID,
                                          pReferTLItem.VerMajor,
                                          pReferTLItem.VerMinor, 0));
  end;
end;


function IdScrEngToCommentStr(IdScrEng: integer): string;
begin
  if not dmURep.qScrEng.Active then dmURep.qScrEng.Open;
  dmURep.qScrEng.Locate('Ival', IdScrEng, []);
  result := dmURep.qScrEngComment.Value;
end;


function GetStrVersPart(str: string; num:integer): integer;
var
  ps, i: integer;
begin
  for i:=0 to num-1 do
  begin
    ps:=pos('.', str);
    if ps>0 then str := copy(str, ps+1, length(str)-ps);
  end;
  ps:=pos('.',str);
  if ps>0 then str := copy(str, 1, ps-1);
  result := StrToIntDef(str, 0);
end;


function Hex2DecDef(const S: string; Default: Longint): Longint;
var
  HexStr: string;
begin
  if Pos('$', S) = 0 then HexStr := '$' + S
  else HexStr := S;
  Result := StrToIntDef(HexStr, Default);
end;

function RegQuerySubkeyDefValue(hkKey: HKEY; subKey: PChar; Value: PByte; lenValue: PDWORD): integer;
var
  hkSubKey: HKEY;
begin
  result := RegOpenKeyEx(hkKey, subKey, 0, KEY_READ, hkSubKey);
  If result<>ERROR_SUCCESS then Exit;
  result := RegQueryValueEx( hkSubKey, nil , nil, nil, Value, lenValue);
  If result<>ERROR_SUCCESS then Exit;
  RegCloseKey(hkSubKey);
end;

Function AddNewTypeLibItemData(refTLList: TList; Name: string; GUID: TGUID;
    VerMajor, VerMinor: integer; UseType:TReferTypeLibUseType): PReferTypeLibItem;
var
  pReferTLItem: PReferTypeLibItem;
begin
  New(pReferTLItem);
  pReferTLItem.Name := Name;
  pReferTLItem.GUID := GUID;
  pReferTLItem.VerMajor := VerMajor;
  pReferTLItem.VerMinor := VerMinor;
  pReferTLItem.UseType := UseType;
  pReferTLItem.NewUseType := UseType;
  pReferTLItem.lcid := 0;
  pReferTLItem.Flags := 0;
  result := pReferTLItem;

  refTLList.Add(pReferTLItem);
end;

function  LoadReferedTypeLibs(ScriptText, CommentStr: string): integer;
var
  sKey: string;
  hkScrRefTypeLib: HKEY;
  ValueName : array[0..64] of Char;
  Data  : array[0..MAX_PATH] of Char;
  LibName  : array[0..MAX_PATH] of Char;
  dwIndex, lnValueName, lnData, lnLibName: DWORD;
  i, res: integer;
  lines: TStrings;
  GUID:TGUID;
  verMajor, verMinor: integer;

  function FindTypeLibName : Longint;
  var
    hkName: HKEY;
  begin
    result := RegOpenKeyEx(HKEY_CLASSES_ROOT, PChar(sKey), 0, KEY_READ, hkName);
    If result=ERROR_SUCCESS then
    begin
      lnLibName := sizeof(LibName);
      result:=RegQueryValueEx(hkName, nil , nil, nil, @LibName, @lnLibName);
      RegCloseKey(hkName);
    end;
  end;
begin
  result := ERROR_SUCCESS;
  If Assigned(ReferedTypeLibItemList) Then Exit;
  ReferedTypeLibItemList := TList.Create;

  // Constant (internal) TypeLibs
  AddNewTypeLibItemData(ReferedTypeLibItemList, LIB_URepGen_Name,
                        LIBID_URepGen, URepGenMajorVersion,
                        URepGenMinorVersion, tlbInternalUse);
  AddNewTypeLibItemData(ReferedTypeLibItemList, LIB_Scripting_Name,
                        LIBID_Scripting, ScriptingMajorVersion,
                        ScriptingMinorVersion, tlbInternalUse);

  // Registry (Computer saved) TypeLibs
  sKey := GetDefaultIniRegKey+ '\'+cnstScriptTypeLibs;
  res := RegOpenKeyEx(HKEY_CURRENT_USER, PChar(sKey), 0,
                         KEY_READ, hkScrRefTypeLib);
  If res=ERROR_SUCCESS then  // m.b. ERROR_FILE_NOT_FOUND
  begin
    dwIndex:=0; lnValueName:=sizeof(ValueName); lnData:=sizeof(Data);
    while RegEnumValue(hkScrRefTypeLib, dwIndex,
                       ValueName, lnValueName, nil, nil,
                       @Data, @lnData )=ERROR_SUCCESS do
    begin
      sKey := 'TypeLib\'+ ValueName +'\' + Data; // TypeLib\{GUID}\major.minor
      If FindTypeLibName=ERROR_SUCCESS then
        AddNewTypeLibItemData(ReferedTypeLibItemList,
                              LibName,
                              StringToGUID(ValueName),
                              GetStrVersPart(Data, 0),
                              GetStrVersPart(Data, 1),
                               tlbRegstrUse);

      Inc(dwIndex); lnValueName:=sizeof(ValueName); lnData:=sizeof(Data);
    end;
    RegCloseKey(hkScrRefTypeLib);
  end;

  // Script level type Libs
  If Length(Trim(ScriptText))>0 then
  begin
    lines := TStringList.Create;
    lines.Text := ScriptText;
    for i:=0 to lines.Count-1 do
    begin
      if Pos(CommentStr, lines.Strings[i])<>1 then Break;
      if not CrackScrReferTypeLibStr(lines.Strings[i], GUID, verMajor, verMinor) then Break;
      sKey := 'TypeLib\'+ GUIDToString(GUID) +'\' + IntToStr(verMajor)+'.'+IntToStr(verMinor); // TypeLib\{GUID}\major.minor
      If FindTypeLibName=ERROR_SUCCESS then
        AddNewTypeLibItemData(ReferedTypeLibItemList,
                              LibName,
                              GUID,
                              verMajor, verMinor,
                              tlbScriptUse);
    end;
    lines.Free;
  end;
end;


function FindRefferTypeLib(findTLItem: PReferTypeLibItem) : PReferTypeLibItem;
var
  i: integer;
  pReferTLItem: PReferTypeLibItem;
begin
  result := nil;
  If not assigned(ReferedTypeLibItemList) then Exit;
  for i := 0 to ReferedTypeLibItemList.Count-1 do
  begin
    pReferTLItem := PReferTypeLibItem(ReferedTypeLibItemList.Items[i]);
    If IsEqualGUID(pReferTLItem.GUID,findTLItem.GUID) and
       (pReferTLItem.VerMajor=findTLItem.VerMajor) and
       (pReferTLItem.VerMinor=findTLItem.VerMinor) then
    begin
      result := pReferTLItem;
      Break;
    end;
  end;
end;

//---------------------------------------------------------------------------
//  Base Functions
//---------------------------------------------------------------------------
Procedure ClearAndNilReferTypeLibItemList(var refTLList: TList);
var
  i: integer;
  pReferTLItem: PReferTypeLibItem;
begin
  If refTLList=nil then Exit;
  for i:=0 to refTLList.Count-1 do
  begin
    pReferTLItem := PReferTypeLibItem(refTLList.Items[i]);
    Dispose(pReferTLItem);
  end;
  refTLList.Free;
  refTLList := nil;
end;


function LoadFullReferTypeLibItemList(refTLList: TList; ScriptText,CommentStr: string): integer;
var
  hkTypeLib, hkGUID, hkGUIDver: HKEY;
  sVer : array[0..64] of Char;
  sGUID: array[0..64] of Char;
  sParam: array[0..64] of Char;
  sParamSubKey: array[0..64] of Char;
  sName: array[0..MAX_PATH] of Char;
  sParamVal: array[0..MAX_PATH] of Char;
  dwIndex, dwIndex2, dwIndex3, lnGUID, dwVer, dwParam: DWORD;
  lName, lParamVal, iFlags, iLcid, iTmp: Longint;
  pFullTLItem, pReferTLItem: PReferTypeLibItem;
  Path: string;
  GUID: TGUID;
begin
  ClearAndNilReferTypeLibItemList(ReferedTypeLibItemList);
  LoadReferedTypeLibs(ScriptText,CommentStr);

  result := RegOpenKeyEx(HKEY_CLASSES_ROOT, 'TypeLib', 0, KEY_READ, hkTypeLib);
  If result=ERROR_SUCCESS then
  begin
    // Enum \TypeInfo
    dwIndex:=0; lnGUID := sizeof(sGUID);
    while RegEnumKeyEx(hkTypeLib, dwIndex, sGUID, lnGUID,
                       nil, nil, nil, nil) = ERROR_SUCCESS do
    begin
      result := RegOpenKeyEx(hkTypeLib, sGUID, 0, KEY_READ, hkGUID);
      If result=ERROR_SUCCESS then
      begin
        GUID := StringToGUID(sGUID);

        // Enum all at this level (major.minor = name)
        dwIndex2 := 0 ; dwVer := sizeof(sVer);
        while RegEnumKeyEx(hkGUID, dwIndex2, sVer, dwVer,
                           nil, nil, nil, nil)=ERROR_SUCCESS do
        begin
          result := RegOpenKeyEx(hkGUID, sVer, 0, KEY_READ, hkGUIDver);
          If result=ERROR_SUCCESS then
          begin
            lName := sizeof(sName);
            result := RegQueryValueEx( hkGUIDver, nil , nil, nil, @sName, @lName);
            If result=ERROR_SUCCESS then
            begin
              iFlags := 0; iLcid := -1; Path := '';
              // Enum GUID\ver\ subkeys {FLAGS , HelpDir or lcid/win32 }
              dwIndex3 := 0 ; dwParam := sizeof(sParam);
              while RegEnumKeyEx(hkGUIDver, dwIndex3, sParam, dwParam,
                                 nil, nil, nil, nil)=ERROR_SUCCESS do
              begin
                lParamVal := sizeof(sParamVal);
                If UpperCase(sParam)='FLAGS' then
                begin
                  result := RegQuerySubkeyDefValue(hkGUIDver, sParam, @sParamVal, @lParamVal);
                  If result=ERROR_SUCCESS then
                    iFlags := Hex2DecDef(sParamVal, 0)
                  else
                    iFlags := 0;
                end;
                iTmp := Hex2DecDef(sParam, -1);
                If iTmp>=0 then
                begin
                  iLcid := iTmp;
                  sParamSubKey[0] := #0;
                  StrCopy(sParamSubKey, sParam);
                  StrCat(sParamSubKey ,'\win32');
                  result := RegQuerySubkeyDefValue(hkGUIDver, sParamSubKey, @sParamVal, @lParamVal);
                  If result=ERROR_SUCCESS then Path := sParamVal;
                end;
                Inc(dwIndex3); dwParam := sizeof(sParam);
              end;

              If ((iFlags and LIBFLAG_FRESTRICTED)=0) and
                 ((iFlags and LIBFLAG_FHIDDEN)=0) and
                 ((iFlags and LIBFLAG_FCONTROL)=0) and
                 (iLcid>=0) and (Path <> '') then
              begin
                pFullTLItem := AddNewTypeLibItemData(refTLList, sName, GUID,
                                                   GetStrVersPart(sVer, 0),
                                                   GetStrVersPart(sVer, 1),
                                                   tlbNotUse);
                pFullTLItem.Path := Path;
                pFullTLItem.lcid := iLcid;
                pFullTLItem.Flags := iFlags;
                pReferTLItem := FindRefferTypeLib(pFullTLItem);
                If pReferTLItem<>nil then
                begin
                  pFullTLItem.UseType := pReferTLItem.UseType;
                  pFullTLItem.NewUseType := pReferTLItem.UseType;
                end;
              end;
            end;
            RegCloseKey( hkGUIDver );
          end;
          Inc(dwIndex2); dwVer := sizeof(sVer);
        end;
        RegCloseKey( hkGUID ) ;
      end;

      Inc(dwIndex); lnGUID := sizeof(sGUID);
    end;
    RegCloseKey( hkTypeLib );
  end;
end;

const
  cnst1Space = ' ';
  cnstReference ='reference';
  cnstGuid = 'guid';
  cnstEqv = '=';
  cnstVersion = 'version';
  cnstPoint = '.';

function CreateScrReferTypeLibStr(Comment:string; GUID:TGUID; verMajor, verMinor: integer): string;
begin
  result := Comment + cnst1Space + cnstReference + cnst1Space +  cnstGuid + cnstEqv +
            GUIDToString(GUID) + cnst1Space + cnstVersion + cnstEqv +
            IntToStr(verMajor)+ cnstPoint + IntToStr(verMinor);
end;


function CrackScrReferTypeLibStr(ScrReferTypeLibStr: string; var GUID:TGUID; var verMajor, verMinor: integer): Boolean;
var
  ssR, ssL: String;
  function mySubstr(constR: string): Boolean;
  var n1: integer;
  begin
    result := False;
    n1 := Pos(UpperCase(constR), UpperCase(ssR));
    If n1 <= 0 then Exit;
    ssL := Trim(Copy(ssR, 1, n1-1 ));
    ssR := Copy(ssR, n1+length(constR), length(ssR));
    result := True;
  end;
begin
  result := False;
  ssR := ScrReferTypeLibStr;

  If not mySubstr(cnstReference) then Exit;
  If not mySubstr(cnstGuid) then Exit;
  If not mySubstr(cnstEqv) then Exit;
  If not mySubstr(cnstVersion) then Exit;
  GUID := StringToGUID(ssL);

  If not mySubstr(cnstEqv) then Exit;
  verMajor := GetStrVersPart(ssR , 0);
  verMinor := GetStrVersPart(ssR , 1);
  result := True;
end;


initialization

finalization
  ClearAndNilReferTypeLibItemList(ReferedTypeLibItemList);

end.
