unit dlgFilt;

interface

uses Windows, SysUtils, Classes, Graphics, Forms, Controls, StdCtrls,
  Buttons, ExtCtrls, ToolEdit, Mask, CurrEdit, Tabs, DB, DBTables, dbgrids,
  dlgOrd,  Nfindlg2, RXSpin;

type
  TdlgFilter = class(TForm)
    OKBtn: TButton;
    CancelBtn: TButton;
    Label1: TLabel;
    Label2: TLabel;
    lblField: TLabel;
    fltSearch: TCurrencyEdit;
    dateSearch: TDateEdit;
    txtSearch: TEdit;
    cbFieldChange: TComboEdit;
    cmbOper: TComboBox;
    Label3: TLabel;
    btnSet: TBitBtn;
    cmbLogic: TComboBox;
    lblLogic: TLabel;
    BitBtnClear: TBitBtn;
    ResetBtn: TButton;
    memField: TMemo;
    RxSpinDate: TRxSpinButton;
    procedure cbFieldChangeButtonClick(Sender: TObject);
    procedure btnSetClick(Sender: TObject);
    procedure BitBtnClearClick(Sender: TObject);
    procedure ResetBtnClick(Sender: TObject);
    procedure OKBtnClick(Sender: TObject);
    procedure memFieldChange(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure cmbLogicChange(Sender: TObject);
    procedure cmbOperChange(Sender: TObject);
    procedure RxSpinDateTopClick(Sender: TObject);
    procedure RxSpinDateBottomClick(Sender: TObject);
  private
    { Private declarations }
    dsToFilter: TDataSet;
    aFieldList : TMyFieldList;
    FfltDBGrid: TDBGrid;
    SelField : TMyFieldRec;
    procedure OnFieldChange;
  public
    { Public declarations }
    constructor MyCreate(AOwner: TComponent; dsFilter: TDataSet; fltDBGrid: TDBGrid);
  end;

var
  dlgFilter: TdlgFilter;

implementation

{$R *.DFM}

constructor TdlgFilter.MyCreate(AOwner: TComponent; dsFilter: TDataSet; fltDBGrid: TDBGrid);
begin
  inherited Create(AOwner);
  memField.Lines.Clear;
  dsToFilter := dsFilter;
  FfltDBGrid := fltDBGrid;
  aFieldList := TMyFieldList.Create;
  aFieldList.readFields(dsToFilter);
  SelField := nil;
  if dsToFilter.Filtered=True then
    memField.Lines.Add(Trim(dsToFilter.Filter));
  if FfltDBGrid <> nil then
   begin
   SelField := aFieldList.getFieldByName(FfltDBGrid.SelectedField.FullName);
   OnFieldChange;
   end;
  cmbLogicChange(nil);
  memFieldChange(nil);
end;

procedure TdlgFilter.FormDestroy(Sender: TObject);
begin
  aFieldList.Free;
end;


procedure TdlgFilter.OnFieldChange;
var
  ft : TFieldType;
begin
  if (SelField<>nil) then
    begin
    lblField.Caption:= SelField.DisplayLabel;
    cbFieldChange.Text:=SelField.FieldName;
    end;

  if (SelField=nil) or
     (cmbOper.ItemIndex>=6) then
    begin
    txtSearch.Visible  := False;
    dateSearch.Visible := False;
    RxSpinDate.Visible := False;
    fltSearch.Visible  := False;
    Exit;
    end;

  // 
  ft := SelField.DataType;
  case ft of
    ftSmallint, ftInteger, ftAutoInc, ftBoolean,
    ftLargeint, ftFloat, ftCurrency : begin
      fltSearch.Value := 0;
      txtSearch.Visible  := False;
      dateSearch.Visible := False;
      RxSpinDate.Visible := False;
      fltSearch.Visible  := True;
      if ft=ftFloat then
        begin
        fltSearch.DisplayFormat:=',0.#######';
        fltSearch.DecimalPlaces:=7;
        end;
      if ft=ftCurrency then
        begin
        fltSearch.DisplayFormat:=',0.00;-,0.00';
        fltSearch.DecimalPlaces:=2;
        end;
      if ft in [ftSmallint, ftInteger, ftAutoInc, ftBoolean, ftLargeint] then
        begin
        fltSearch.DisplayFormat:='#0';
        fltSearch.DecimalPlaces:=0;
        end;
      end;
    ftDate, ftDateTime : begin
      dateSearch.Date := Now;
      txtSearch.Visible  := False;
      dateSearch.Visible := True;
      RxSpinDate.Visible := True;
      fltSearch.Visible  := False;
      end;
    else begin  //ftString
      txtSearch.Text:='';
      txtSearch.Visible  := True;
      dateSearch.Visible := False;
      RxSpinDate.Visible := False;
      fltSearch.Visible  := False;
      end;
    end;
end;

procedure TdlgFilter.cbFieldChangeButtonClick(Sender: TObject);
var
  dlgNFields : TdlgNFields;
begin
  dlgNFields := TdlgNFields.MyCreate(Self, aFieldList);
  dlgNFields.Left:= Self.Left+cbFieldChange.Left+10;
  dlgNFields.Top:= Self.Top+cbFieldChange.Top+cbFieldChange.Height+10;
  if dlgNFields.ShowModal=mrOK then
    begin
    SelField:= aFieldList.Field[dlgNFields.grid.Row -1];
    OnFieldChange;
    end;
end;


procedure TdlgFilter.btnSetClick(Sender: TObject);
var
  s : string;
begin
  if (SelField<>nil) and
     (cmbOper.ItemIndex>=0) then
    begin
    s := '[' + SelField.FieldName + ']';

    if cmbOper.ItemIndex>=6 then
      s := s + ' ';
    s := s + cmbOper.Text;
    if cmbOper.ItemIndex>=6 then
      s := s + ' ';

    if cmbOper.ItemIndex<6 then
      begin
      case SelField.DataType of
        ftString : begin
          s := s + '''' + txtSearch.Text + '''';
          txtSearch.Text := '';
          end;
        ftSmallint, ftInteger, ftAutoInc, ftBoolean, ftLargeint  : begin
          s := s + FloatToStr(fltSearch.Value);
          fltSearch.Value := 0;
          end;
        ftFloat, ftCurrency : begin
          s := s + FloatToStr(fltSearch.Value);
          fltSearch.Value := 0;
          end;
        ftDate, ftDateTime : begin
          s := s + ''''+ DateToStr(dateSearch.Date)+ '''';
          end;
        end;
      end;

    if cmbLogic.ItemIndex>=0 then
      s := cmbLogic.Text + ' ' + s ;

    memField.Lines.Add(Trim(s));
    cbFieldChange.Text := '';
    lblField.Caption := '';
    SelField := nil;
    cmbOper.ItemIndex := -1;
    cmbLogic.ItemIndex := -1;
    end;
end;


procedure TdlgFilter.BitBtnClearClick(Sender: TObject);
begin
  memField.Lines.Clear;
  memFieldChange(nil);
end;

procedure TdlgFilter.ResetBtnClick(Sender: TObject);
begin
  dsToFilter.Filter := '';
  dsToFilter.Filtered := False;
  if FfltDBGrid<>nil then
    FfltDBGrid.FixedColor := clBtnFace; // clBtnFace
end;

procedure TdlgFilter.OKBtnClick(Sender: TObject);
var
 s: string;
begin
  s := memField.Lines.Text;
  s := Trim(s);
  if length(s)>0 then
    begin
    dsToFilter.Filter := s;
    dsToFilter.Filtered := True;
    if FfltDBGrid<>nil then
      FfltDBGrid.FixedColor := clRed; // clBtnFace
    end;
end;

procedure TdlgFilter.memFieldChange(Sender: TObject);
begin
 if Length(memField.Text)>1 then
   begin
   OKBtn.Enabled :=True;
   cmbLogic.Enabled :=True;
   end
 else
   begin
   OKBtn.Enabled :=False;
   cmbLogic.Enabled :=False;
   end;
end;

procedure TdlgFilter.cmbLogicChange(Sender: TObject);
begin
  if ((cmbLogic.Enabled and (cmbLogic.ItemIndex>=0)) or
      (not cmbLogic.Enabled)) and
     (SelField<>nil) and
     (cmbOper.ItemIndex>=0) then
    btnSet.Enabled := True
  else
    btnSet.Enabled := False;
end;

procedure TdlgFilter.cmbOperChange(Sender: TObject);
begin
  cmbLogicChange(nil);

  if cmbOper.ItemIndex>=6 then
    begin
    txtSearch.Visible  := False;
    dateSearch.Visible := False;
    RxSpinDate.Visible := False;
    fltSearch.Visible  := False;
    end;
end;

procedure TdlgFilter.RxSpinDateTopClick(Sender: TObject);
begin
  dateSearch.Date := dateSearch.Date + 1;
end;

procedure TdlgFilter.RxSpinDateBottomClick(Sender: TObject);
begin
  dateSearch.Date := dateSearch.Date - 1;
end;

end.
