unit fURepBrowser;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  RXSplit, ComCtrls, vvTreeView, ExtCtrls, StdCtrls, ImgList, unAxBowser,
  Placemnt, ToolWin, Menus, fURepReferences, ActnList, RXCtrls, DB;

type
  TfrmURepBrowser = class(TForm)
    Panel1: TPanel;
    tvAxBrowser: TTreeView;
    Panel2: TPanel;
    RxSplitter1: TRxSplitter;
    ImageList1: TImageList;
    RichEdit1: TRichEdit;
    FormStorage1: TFormStorage;
    PopupMenuStyle: TPopupMenu;
    mnVBstile: TMenuItem;
    mnFullInfo: TMenuItem;
    N1: TMenuItem;
    mnSortByName: TMenuItem;
    mnSortByFuncID: TMenuItem;
    N2: TMenuItem;
    mnReferences: TMenuItem;
    ComboBoxLib: TComboBox;
    spbtnScale: TRxSpeedButton;
    ActionList1: TActionList;
    ActionConfig: TAction;
    mnPrint: TMenuItem;
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure ComboBoxLibChange(Sender: TObject);
    procedure tvAxBrowserChange(Sender: TObject; Node: TTreeNode);
    procedure FormDestroy(Sender: TObject);
    procedure tvAxBrowserMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure MenuItemClick(Sender: TObject);
    procedure mnReferencesClick(Sender: TObject);
    procedure ActionConfigExecute(Sender: TObject);
    procedure mnPrintClick(Sender: TObject);
    procedure tvAxBrowserMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure tvAxBrowserMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
  private
    FTLItemList: TTLItemList;
    FDefObjName: string;
    FDefObjGUID: TGUID;
    FTreeViewEnabled: Boolean;
    FDataSet: TDataSet;

    FOldGUID: TGUID;

    fMouseDownX, fMouseDownY: integer;
    fWaitForDragging: Boolean;

    procedure PopulateComboBoxLib;
    procedure RefreshObjectsTree;
    procedure ClearAndFreeFTLItemList;
  public
    constructor Create(Aowner: TComponent; objName: string; GUID: TGUID; DataSet: TDataSet); reintroduce;
    //procedure ReShowTree(objName: string; GUID: TGUID);
  end;


implementation

{$R *.DFM}

uses ComObj, ActiveX, Impl_URepObjectFactory, URepTypeLibs, TreeReport,
     dmdURep, URepConstants;


{type
  TvvTVDragObject = class(TDragObject)
    FDragImageList : TDragImageList;
  protected
    function GetDragImages: TDragImageList; override;
    constructor Create(DragImageList : TDragImageList);
  end;

function TvvTVDragObject.GetDragImages: TDragImageList;
begin
  result := FDragImageList;
end;

constructor TvvTVDragObject.Create(DragImageList: TDragImageList);
begin
  FDragImageList := DragImageList;
  inherited Create;
end;
}


constructor TfrmURepBrowser.Create(Aowner: TComponent; objName: string; GUID: TGUID; DataSet: TDataSet);
var
  sScript, sComment: string;
begin
  fWaitForDragging := False;
  FDefObjName := objName;
  FDefObjGUID := GUID;
  FDataSet := DataSet;
  inherited Create(Aowner);

  FTreeViewEnabled := False;
  SetTypeLibListStyle([stlVB, stlSortByName]);
  FTLItemList := nil;

  sScript := DataSet.FieldByName(fldScriptText).Value;
  sComment := IdScrEngToCommentStr(DataSet.FieldByName(fldIdScrEng).asInteger);

  ClearAndNilReferTypeLibItemList(ReferedTypeLibItemList);
  LoadReferedTypeLibs(sScript, sComment);

  FOldGUID := GUID_NULL;
  PopulateComboBoxLib;
end;


procedure TfrmURepBrowser.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  Action := caFree;
end;

type TReferTypeLib = class
 public
 pReferTLItem: PReferTypeLibItem;
 end;

procedure TfrmURepBrowser.PopulateComboBoxLib;
var
  i, oldItemIndex, selItemIndex: integer;
  pReferTLItem: PReferTypeLibItem;
  oReferTypeLib: TReferTypeLib;
  bLibChanged: Boolean;
begin
  selItemIndex := 0; bLibChanged:= True;
  oldItemIndex := ComboBoxLib.ItemIndex;
  for i:=0 to ComboBoxLib.Items.Count-1 do // free ComboBoxLib.Items.Objects
  begin
    oReferTypeLib := ComboBoxLib.Items.Objects[i] as TReferTypeLib;
    oReferTypeLib.Free;
  end;

  ComboBoxLib.Items.Clear;

  If ReferedTypeLibItemList=nil then Exit;
  for i:=0 to ReferedTypeLibItemList.Count-1 do
  begin
    pReferTLItem := PReferTypeLibItem(ReferedTypeLibItemList.Items[i]);
    if (oldItemIndex>=0) and
        IsEqualGUID(FOldGUID, pReferTLItem.GUID) then
      begin
      bLibChanged := False;
      selItemIndex := i;
      end;
    oReferTypeLib:= TReferTypeLib.Create;
    oReferTypeLib.pReferTLItem := pReferTLItem;
    ComboBoxLib.Items.AddObject(pReferTLItem.Name, oReferTypeLib);
  end;
  ComboBoxLib.ItemIndex := selItemIndex;

  if bLibChanged then
    ComboBoxLibChange(nil);
end;

procedure TfrmURepBrowser.ComboBoxLibChange(Sender: TObject);
var
  libGUID: TGUID;
begin
  libGUID := (ComboBoxLib.Items.Objects[ComboBoxLib.ItemIndex] as TReferTypeLib).pReferTLItem.GUID;
  If IsEqualGUID(libGUID, FOldGUID) Then Exit;
  RefreshObjectsTree;
  FOldGUID := (ComboBoxLib.Items.Objects[ComboBoxLib.ItemIndex] as TReferTypeLib).pReferTLItem.GUID;
end;

procedure TfrmURepBrowser.RefreshObjectsTree;
var
  i_ITypeLib : ITypeLib;
  pReferTLItem: PReferTypeLibItem;
  oReferTypeLib: TReferTypeLib;
begin
  FTreeViewEnabled := False;

  // Get Tipe Lib
  oReferTypeLib := ComboBoxLib.Items.Objects[ComboBoxLib.ItemIndex] as TReferTypeLib;
  pReferTLItem := oReferTypeLib.pReferTLItem;
  try
    OleCheck(LoadRegTypeLib(pReferTLItem.GUID, pReferTLItem.VerMajor, pReferTLItem.VerMinor, 0, i_ITypeLib));
  except
    ClearAndFreeFTLItemList; // Clear Tree Items data
    tvAxBrowser.Items.Clear;
  end;

  // Load Tree
  ClearAndFreeFTLItemList; // Clear Tree Items data
  LoadTypeLibItemList(i_ITypeLib, FTLItemList, FDefObjName, FDefObjGUID);

  tvAxBrowser.Items.BeginUpdate;
  try
    tvAxBrowser.Items.Clear;
    TypeLibItemList2TreeView(FTLItemList, tvAxBrowser);
  finally
    tvAxBrowser.Items.EndUpdate;
  end;
  FTreeViewEnabled := True;
  tvAxBrowser.Selected := tvAxBrowser.Items[0];
end;

procedure TfrmURepBrowser.tvAxBrowserChange(Sender: TObject;
  Node: TTreeNode);
var
  pItemData : PTTLItemData;
begin
  If FTreeViewEnabled Then
  begin
    pItemData := Node.Data;
    OnTLItemChange(pItemData, RichEdit1);
  end;
end;

procedure TfrmURepBrowser.ClearAndFreeFTLItemList;
begin
  If FTLItemList<>nil Then
  begin
    FreeTTLItemList(FTLItemList);
    FTLItemList := Nil;
  end;
end;

procedure TfrmURepBrowser.FormDestroy(Sender: TObject);
var
  i: integer;
  oReferTypeLib: TReferTypeLib;
begin
  ClearAndNilReferTypeLibItemList(ReferedTypeLibItemList);
  ClearAndFreeFTLItemList;
  for i:=0 to ComboBoxLib.Items.Count-1 do // free ComboBoxLib.Items.Objects
  begin
    oReferTypeLib := ComboBoxLib.Items.Objects[i] as TReferTypeLib;
    oReferTypeLib.Free;
  end;
end;

procedure TfrmURepBrowser.MenuItemClick(Sender: TObject);
begin
 if Sender=mnVBstile then
 begin
   mnVBstile.Checked := True;
   If mnSortByFuncID.Checked Then SetTypeLibListStyle([stlVB, stlSortByID])
   else SetTypeLibListStyle([stlVB, stlSortByName]);
 end
 else if Sender=mnFullInfo then
 begin
   mnFullInfo.Checked := True;
   If mnSortByFuncID.Checked Then SetTypeLibListStyle([stlFull, stlSortByID])
   else SetTypeLibListStyle([stlFull, stlSortByName]);
 end
 else if Sender=mnSortByFuncID then
 begin
   mnSortByFuncID.Checked := True;
   If mnVBstile.Checked Then SetTypeLibListStyle([stlVB, stlSortByID])
   else SetTypeLibListStyle([stlFull, stlSortByID]);
 end
 else if Sender=mnSortByName then
 begin
   mnSortByName.Checked := True;
   If mnVBstile.Checked Then SetTypeLibListStyle([stlVB, stlSortByName])
   else SetTypeLibListStyle([stlFull, stlSortByName]);
 end;
 RefreshObjectsTree;
end;

procedure TfrmURepBrowser.mnReferencesClick(Sender: TObject);
var
  frmURepReferences: TfrmURepReferences;
begin
  frmURepReferences := TfrmURepReferences.Create(Forms.Application, FDataSet);
  frmURepReferences.ShowModal;
  frmURepReferences.Release;
  PopulateComboBoxLib;
end;

procedure TfrmURepBrowser.ActionConfigExecute(Sender: TObject);
begin
// don't erase comment!
end;

procedure TfrmURepBrowser.mnPrintClick(Sender: TObject);
begin
  TreeReportPreview(tvAxBrowser, ComboBoxLib.Items.Strings[ComboBoxLib.ItemIndex]);
end;

procedure TfrmURepBrowser.tvAxBrowserMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
  pItemData : PTTLItemData;
begin
  if Button<>mbLeft then Exit;
  If tvAxBrowser.Selected=nil Then Exit;
  pItemData := tvAxBrowser.Selected.Data;
  If IsItemCanDragAndDropped(pItemData) Then
  begin
    fWaitForDragging := True;
    fMouseDownX := X;
    fMouseDownY := Y;
  end;
end;


procedure TfrmURepBrowser.tvAxBrowserMouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: Integer);
begin
  if fWaitForDragging then begin
    if (Abs(fMouseDownX - X) > GetSystemMetrics(SM_CXDRAG)) or
       (Abs(fMouseDownY - Y) > GetSystemMetrics(SM_CYDRAG)) then begin
      fWaitForDragging := false;
      tvAxBrowser.BeginDrag(True);
    end;
  end
end;

procedure TfrmURepBrowser.tvAxBrowserMouseUp(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  fWaitForDragging := false;
end;

{
procedure TfrmURepBrowser.tvAxBrowserStartDrag(Sender: TObject;
  var DragObject: TDragObject);
begin
  DragObject := TvvTVDragObject.Create(ImageList1);
end;
}

{procedure TfrmURepBrowser.ReShowTree(objName: string; GUID: TGUID);
begin
  FDefObjName := objName;
  FDefObjGUID := GUID;
  //FOldItemIndex := -1;  //   

  LoadReferedTypeLibs('', '');
  PopulateComboBoxLib;
  ComboBoxLibChange(nil);
end; }



end.
