unit fURepDelDep;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Buttons, ExtCtrls, Grids, RXGrids, GridEx, DB, DBTables,
  Placemnt;


type
  TURepObjectType = (otNone, otGroup, otForm, otSQL, otParam,
                     otScript, otImportData, otRemoteData);

const
  TURepObjectTableNames : array [TURepObjectType] of string =
    ('', 'URepGroup', 'URepForm', 'URepQuery', 'URepParam',
     'URepScript', 'URepImports', 'URepRemoteData');

type
  TURepDelType = (delNone, delParent, delEdit);

  TURepDependList = class;

  TURepDependItem = class
  private
    FDelete: Boolean;
    FURepDependList: TURepDependList;

    procedure Set_Delete(bDelete: Boolean);
    procedure _SetChildDelete(bDelete: Boolean);
  public
    TypObj: TURepObjectType;// | Uniq!
    IdObj: integer;         // |

    ObjName: string;
    DelType: TURepDelType;  //delNone, delParent, delEdit

    Sequence: integer;
    ParentId: integer;
    ParentTypObj: TURepObjectType;

    property Delete: Boolean read FDelete write Set_Delete;
    procedure SetDefDelete;
    function GetImage :TBitmap;
  end;


  TURepDependList = class
  private
    FMainTypObj: TURepObjectType;
    FMainIdObj: integer;

    FDependList : TList;
    function FGetCount: integer;
    function FGetItems(iIndex: integer) : TURepDependItem;
  public
    property Count: Integer read FGetCount;
    property Items[iIndex: integer] : TURepDependItem read FGetItems; default;

    function Add1Item(IdObj: Integer; ObjName: string;
                     aTypObj: TURepObjectType; aDelType: TURepDelType;
                     iSequence: integer;
                     iIdParent: integer; aParentTypObj: TURepObjectType): TURepDependItem;

    procedure AddItems(sSQL: string;
                     aTypObj: TURepObjectType; aDelType: TURepDelType;
                     iSequence: integer;
                     iIdParent: integer; aParentTypObj: TURepObjectType);
    procedure DoDelete;

    constructor Create(MainIdObj: integer; MainTypObj: TURepObjectType);
    destructor Destroy; override;
  end;


  TfrmURepDelDepends = class(TForm)
    Panel1: TPanel;
    Label1: TLabel;
    Panel2: TPanel;
    Panel3: TPanel;
    Label2: TLabel;
    imgObjTyp: TImage;
    BitBtnOk: TBitBtn;
    BitBtnCancel: TBitBtn;
    lblObjName: TLabel;
    RxDrawGrid1: TRxDrawGrid;
    Label3: TLabel;
    FormPlacement1: TFormPlacement;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure RxDrawGrid1DrawCell(Sender: TObject; ACol, ARow: Integer;
      Rect: TRect; State: TGridDrawState);
    procedure RxDrawGrid1KeyPress(Sender: TObject; var Key: Char);
    procedure BitBtnOkClick(Sender: TObject);
    procedure RxDrawGrid1DblClick(Sender: TObject);
  private
    { Private declarations }
    FIdObj: integer;
    FTypObj: TURepObjectType;
    FObjName: string;
    FDependList: TURepDependList;
    FTopObjList: TList;
    constructor CreateFrm(AOwner: TComponent;
                IdObj: integer; TypObj: TURepObjectType; ObjName: string;
                DependList: TURepDependList; TopObjList: TList);
  public
    { Public declarations }
  end;

function DelObjWithDependList(IdObj: integer; TypObj: TURepObjectType; sObjName: String): Boolean;

implementation

{$R *.DFM}

uses dmdURep, fURepFunc1, Askme, URepConstants, Common, QueryEx;

const
  cnstApostrof = '"';
  cnstApstrfQuery = '"?';

//=======================================================================
//     TURepDependItem
//=======================================================================
procedure TURepDependItem._SetChildDelete(bDelete: Boolean);
var
  i: integer;
begin
  for i:=0 to FURepDependList.Count-1 do
  if (FURepDependList.Items[i].DelType=delParent) and
    (FURepDependList.Items[i].ParentId=IdObj) and
    (FURepDependList.Items[i].ParentTypObj=TypObj) then
   FURepDependList.Items[i].FDelete := bDelete;
end;


procedure TURepDependItem.Set_Delete(bDelete: Boolean);
begin
 case DelType of
   delNone:
     if bDelete=True then
       ErrorMsg('Cant be deleted (delNone)');
   delEdit:
     begin
     FDelete := bDelete;
     _SetChildDelete(bDelete);
     end;
   delParent:
     ErrorMsg('Cant be set directly (delParent)');
 end;
end;

procedure TURepDependItem.SetDefDelete;
begin
 case DelType of
   delNone:
     begin
     FDelete := False;
     _SetChildDelete(False);
     end;
   delEdit:
     begin
     FDelete := True;
     _SetChildDelete(True);
     end;
   delParent:
     FDelete := True;
 end;
end;


var
  GalochkaOnly: TBitmap;

function TURepDependItem.GetImage :TBitmap;
begin
 result := Nil;
 case DelType of
   delEdit:
     result := GetBoolImage(FDelete);
   delParent:
     if FDelete then
     begin
       if GalochkaOnly=nil then
       begin
         GalochkaOnly:=TBitmap.Create;
         GalochkaOnly.LoadFromResourceName(HInstance, 'CHECKONLY');
       end;
       result := GalochkaOnly;
     end
 end;
end;


//=======================================================================
//   TURepDependList
//=======================================================================
constructor TURepDependList.Create(MainIdObj: integer; MainTypObj: TURepObjectType);
begin
  FMainIdObj := MainIdObj;
  FMainTypObj := MainTypObj;
  FDependList := TList.Create;
end;

destructor TURepDependList.Destroy;
var i: integer;
begin
  for i:=0 to FDependList.Count-1 do
    TURepDependItem(FDependList.Items[i]).Free;
  FDependList.Free;
  inherited Destroy;
end;

function TURepDependList.FGetCount: integer;
begin
  result := FDependList.Count;
end;

function TURepDependList.FGetItems(iIndex: integer) : TURepDependItem;
begin
  result := TURepDependItem(FDependList.Items[iIndex]);
end;


function TURepDependList.Add1Item(IdObj: Integer; ObjName: string;
                     aTypObj: TURepObjectType; aDelType: TURepDelType;
                     iSequence: integer;
                     iIdParent: integer; aParentTypObj: TURepObjectType): TURepDependItem;
begin
  result := TURepDependItem.Create();
  result.TypObj := aTypObj;
  result.DelType := aDelType;
  result.IdObj := IdObj;
  result.ObjName := ObjName;
  result.Sequence := iSequence;
  result.ParentId := iIdParent;
  result.ParentTypObj := aParentTypObj;

  FDependList.Add(result);
  result.FURepDependList := Self;

  result.SetDefDelete;
end;

procedure TURepDependList.AddItems(sSQL: string;
                     aTypObj: TURepObjectType; aDelType: TURepDelType;
                     iSequence: integer;
                     iIdParent: integer; aParentTypObj: TURepObjectType);
var
  aQuery: TQuery;
begin
  aQuery := TQuery.Create(nil);
  try
    aQuery.DatabaseName := glDatabaseName;
    aQuery.SQL.Text := sSQL;
    aQuery.Open;
    aQuery.First;
    while not aQuery.EOF do
    begin
      Add1Item(aQuery.FieldByName(fldId).asInteger,
               aQuery.FieldByName(fldName).asString,
               aTypObj, aDelType,
               iSequence,
               iIdParent, aParentTypObj);
      aQuery.Next;
    end;
  finally
    aQuery.Free;
  end;
end;

procedure TURepDependList.DoDelete;
var
  i: integer;
  oURepDependItem: TURepDependItem;
begin
  // delete childs
  for i:=Count-1 downto 0 do
  begin
    oURepDependItem := Items[i];
    if oURepDependItem.Delete then
    begin
      case oURepDependItem.TypObj of
        otGroup:      URepGroupDelete(oURepDependItem.IdObj);
        otForm:       URepFormDelete(oURepDependItem.IdObj);
        otScript:     URepScriptDelete(oURepDependItem.IdObj);
        otSQL:        begin
                      if oURepDependItem.ParentId>0 then
                      DoSQL('delete from URepFrm2Qry where IdForm='+IntToStr(oURepDependItem.ParentId)+
                            ' and IdQuery='+IntToStr(oURepDependItem.IdObj));
                      URepQueryDelete(oURepDependItem.IdObj);
                      end;
        otRemoteData: begin
                      if oURepDependItem.ParentId>0 then
                      DoSQL('delete from URepFrm2Qry where IdForm='+IntToStr(oURepDependItem.ParentId)+
                            ' and IdRemoteData='+IntToStr(oURepDependItem.IdObj));
                      URepRemoteDataDelete(oURepDependItem.IdObj);
                      end;
        otParam:      URepParamDelete(oURepDependItem.IdObj);
      end;
    end;
  end;

  // delete main obj
  Case FMainTypObj of
    otGroup:      URepGroupDelete(FMainIdObj);
    otForm:       URepFormDelete(FMainIdObj);
    otSQL:        URepQueryDelete(FMainIdObj);
    otRemoteData: URepRemoteDataDelete(FMainIdObj);
  end;

end;

//=======================================================================
procedure _AddTopObjList(TopObjList: TList; sSQL: string);
begin
end;


procedure URepQueryDependList(DependList: TURepDependList; IdQuery: integer;
                              iSequence: integer);
var
  sSQL: string;
begin
  sSQL:= //URepParams
    'select Id, ParLabel Name from URepParam '+
    'where IdQuery='+IntToStr(IdQuery);
  SQLStrOrderBy(sSQL, fldNumm);
  DependList.AddItems(sSQL, otParam, delParent, iSequence, IdQuery, otSQL);
end;

procedure URepRemoteDataDependList(DependList: TURepDependList; IdRemoteData: integer;
                              iSequence: integer);
var
  sSQL: string;
begin
  sSQL:= //URepParams
    'select Id, ParLabel Name from URepParam '+
    'where IdRemoteData='+IntToStr(IdRemoteData);
  SQLStrOrderBy(sSQL, fldNumm);
  DependList.AddItems(sSQL, otParam, delParent, iSequence, IdRemoteData, otRemoteData);
end;


procedure URepFormDependList(DependList: TURepDependList; IdForm: integer;
                              iSequence: integer);
var
  sSQL, sObjName, sIdFldNm: string;
  iIdQuery, iIdRemoteData, iObjId: integer;
  aQuery: TQuery;
  oDependItem: TURepDependItem;
  TypObj: TURepObjectType;
  DelType: TURepDelType;
begin
  sSQL:=  //URepFrm2Qry
   'select Id, IdQuery, IdRemoteData from URepFrm2Qry '+
   'where IdForm='+IntToStr(IdForm);
  SQLStrOrderBy(sSQL, fldNumm);

  aQuery := TQuery.Create(nil);
  try
    aQuery.DatabaseName := glDatabaseName;
    aQuery.SQL.Text := sSQL;
    aQuery.Open;
    aQuery.First;
    while not aQuery.EOF do
    begin
      iIdQuery := aQuery.FieldByName(fldIdQuery).asInteger;
      iIdRemoteData := aQuery.FieldByName(fldIdRemoteData).asInteger;
      DelType := delEdit;
      if iIdQuery>0 then
      begin
        iObjId := iIdQuery;
        TypObj := otSQL;
        sIdFldNm := fldIdQuery;
      end
      else
      begin
        iObjId := iIdRemoteData;
        TypObj := otRemoteData;
        sIdFldNm := fldIdRemoteData;
      end;
      sObjName :=  DLookup('select Name from '+TURepObjectTableNames[TypObj]+
                           ' where Id='+IntToStr(iObjId));
      if DLookup('select count(*) from URepFrm2Qry where '+sIdFldNm+'='+IntToStr(iObjId)+
                 ' and IdForm<>'+IntToStr(IdForm))>0 then
        DelType := delNone;

      oDependItem:=DependList.Add1Item(iObjId, sObjName,
                            TypObj, DelType,
                            0, IdForm, otForm);
      if iIdQuery>0 then
        URepQueryDependList(DependList, iIdQuery, 1)
      else
        URepRemoteDataDependList(DependList, iIdRemoteData, 1);
      oDependItem.SetDefDelete;

      aQuery.Next;
    end;
  finally
    aQuery.Free;
  end;
end;

procedure FillDependList(DependList: TURepDependList; TopObjList: TList;
                         IdObj: integer; TypObj: TURepObjectType);
begin
  if TypObj=otSQL then //URepQuery
  begin
    //sSQL:= // URepForms
    //  'select 2 IdTypObj, f.Id, f.Name from URepFrm2Qry fq, URepForm f ' +
    //  'where fq.IdQuery='+IntToStr(IdObj)+' and fq.IdForm=f.Id order by f.Numm';
    //_AddTopObjList(TopObjList, sSQL);

    URepQueryDependList(DependList, IdObj, 0);
  end;

  if TypObj=otRemoteData then //URepRemoteData
  begin
    URepRemoteDataDependList(DependList, IdObj, 0);
  end;

  if TypObj=otForm then  //URepForm
  begin
    //sSQL:=  //URepGroup
    // 'select 0, 1, gb.IdGroup, g.GrpName from URepGrpBody gb, URepGroup g '+
    // 'where gb.IdForm='+IntToStr(IdObj)+' and gb.IdGroup=g.Id order by g.Numm';
    //_AddTopObjList(TopObjList, sSQL);

    URepFormDependList(DependList, IdObj, 0);
  end;


end;

//----------------------------------------------------------------------------
//           DelObjWithDependList
//----------------------------------------------------------------------------
function DelObjWithDependList(IdObj: integer; TypObj: TURepObjectType; sObjName: String): Boolean;
var
  DependList: TURepDependList;
  TopObjList: TList;
  frmURepDelDepends: TfrmURepDelDepends;
  sCaption: String;
begin
  result:= False;
  DependList := TURepDependList.Create(IdObj, TypObj);
  TopObjList := TList.Create;
  FillDependList(DependList, TopObjList, IdObj, TypObj);
  if DependList.Count>0 then
  begin
    frmURepDelDepends:=
      TfrmURepDelDepends.CreateFrm(Forms.Application,
                                   IdObj, TypObj, sObjName,
                                   DependList, TopObjList);
    result:= (frmURepDelDepends.ShowModal=mrOK);
    frmURepDelDepends.Release;
  end
    else
  begin
    case TypObj of
      otGroup:  sCaption := cnstDelGroup;
      otScript: sCaption := cnstDelScript;
      otForm:   sCaption := cnstDelForm;
      otSQL:    sCaption := cnstDelSQL;
      otRemoteData: sCaption := cnstDelRemoteData;
    end;
    sCaption := sCaption + cnstApostrof + sObjName + cnstApstrfQuery;
    if not Confirmation(sCaption) then Exit;
    result:= True;
    case TypObj of
      otGroup:   begin URepGroupDelete(IdObj); ReQueryOne(dmURep.qGroup); end;
      otScript:  begin URepScriptDelete(IdObj); end;
      otForm:    begin URepFormDelete(IdObj); ReQueryOne(dmURep.qForm); end;
      otSQL:     begin URepQueryDelete(IdObj); end;
      otRemoteData: begin URepRemoteDataDelete(IdObj); end;
    end;
  end;
end;
//----------------------------------------------------------------------------
//           TfrmURepDelDepends
//----------------------------------------------------------------------------

constructor TfrmURepDelDepends.CreateFrm(AOwner: TComponent;
                IdObj: integer; TypObj: TURepObjectType; ObjName: string;
                DependList: TURepDependList; TopObjList: TList);
begin
  FIdObj := IdObj;
  FTypObj := TypObj;
  FObjName := ObjName;

  FDependList := DependList;
  FTopObjList := TopObjList;
  inherited Create(AOwner);
end;


procedure TfrmURepDelDepends.FormCreate(Sender: TObject);
begin
  lblObjName.Caption := FObjName;
  dmURep.imgLstURep.GetBitmap(URepImageIndexFromIdTypObj(Ord(FTypObj)),
                              imgObjTyp.Picture.Bitmap);
  RxDrawGrid1.RowCount := FDependList.Count+1;
  case FTypObj of
    otGroup: Caption := cnstDelGroup;
    otForm:  Caption := cnstDelForm;
    otSQL:   Caption := cnstDelSQL;
    otRemoteData: Caption := cnstDelRemoteData;
  end;
  Caption := Caption + cnstApostrof + FObjName + cnstApstrfQuery;
end;


procedure TfrmURepDelDepends.FormDestroy(Sender: TObject);
begin
  FDependList.Free;
end;

procedure TfrmURepDelDepends.RxDrawGrid1DrawCell(Sender: TObject; ACol,
  ARow: Integer; Rect: TRect; State: TGridDrawState);
var
  ss : string;
  oDependItem : TURepDependItem;
  aBitmap : TBitmap;
  TxtRect: TRect;
begin
 ss := '';
 if ARow = 0 then
 begin
   case ACol of
    0 : ss := '';
    1 : ss := '';
   end;
   RxDrawGrid1.DrawStr( Rect, ss, taLeftJustify );
 end
 else
 begin
   if FDependList.Count > 0 then
   begin
     oDependItem := FDependList.Items[ARow - 1];
     case ACol of
      0 : begin
          aBitmap := oDependItem.GetImage;
          if aBitmap=nil then
            RxDrawGrid1.DrawStr( Rect, '', taLeftJustify )
          else
            RxDrawGrid1.DrawPicture(Rect, aBitmap);
          end;
      1 : begin
          aBitmap := TBitmap.Create;
          dmURep.imgLstURep.GetBitmap(URepImageIndexFromIdTypObj(Ord(oDependItem.TypObj)),
                                      aBitmap);
          RxDrawGrid1.Canvas.FillRect(Rect);

          RxDrawGrid1.Canvas.Draw(Rect.Left+(10*oDependItem.Sequence)+5,
                              (Rect.Top+Rect.Bottom-aBitmap.Height) div 2,
                              aBitmap);
          TxtRect := Rect;
          TxtRect.Left := Rect.Left+(10*oDependItem.Sequence)+aBitmap.Width+5;
          RxDrawGrid1.Canvas.TextRect(TxtRect, TxtRect.Left+10,
                                   Rect.Top, oDependItem.ObjName);
          aBitmap.Free;

          end;
     end;
   end;
 end;
end;

procedure TfrmURepDelDepends.RxDrawGrid1KeyPress(Sender: TObject;
  var Key: Char);
begin
  if Key=#32 then
    RxDrawGrid1DblClick(nil);
end;

procedure TfrmURepDelDepends.RxDrawGrid1DblClick(Sender: TObject);
var
  oDependItem : TURepDependItem;
begin
  if RxDrawGrid1.Col=0 then
    begin
    oDependItem := FDependList.Items[RxDrawGrid1.Row - 1];
    if oDependItem.DelType=delEdit then
      begin
      oDependItem.Delete := not oDependItem.Delete;
      RxDrawGrid1.Repaint;
      end;
    end;
end;

procedure TfrmURepDelDepends.BitBtnOkClick(Sender: TObject);
begin
 (GetDBObject as TDatabase).StartTransaction;
  try
    FDependList.DoDelete;
    (GetDBObject as TDatabase).Commit;
  except
    (GetDBObject as TDatabase).Rollback;
    raise;
  end;
end;

{

  for i:=0 to FDependList.Count-1 do
    begin
    oDependItem := FDependList.Items[i];

    if (oDependItem.TypObj=otGroup) then
      begin
      if FTypObj=otForm then
        begin
        CheckBeginTran;
        DoSQL('delete from URepGrpBody where IdForm='+ IntToStr(FIdObj)+
              ' and IdGroup='+ IntToStr(oDependItem.IdObj));
        end;
      end;

    if (oDependItem.TypObj=otForm) then
      begin
      if FTypObj=otGroup then
        begin
        CheckBeginTran;
        DoSQL('delete from URepGrpBody where IdForm='+ IntToStr(oDependItem.IdObj)+
              ' and IdGroup='+ IntToStr(FIdObj));
        //if (oDependItem.IsDeleted>0) and
        //   (oDependItem.bDelete) then
          URepFormDelete(oDependItem.IdObj);
        end;
      if FTypObj=otSQL then //3-
        begin
        CheckBeginTran;
        DoSQL('delete from URepFrm2Qry where IdForm='+ IntToStr(oDependItem.IdObj)+
              ' and IdQuery='+ IntToStr(FIdObj));
        end;
      end;

    if (oDependItem.TypObj=otSQL) then //3-
      begin
      CheckBeginTran;
      if FTypObj=otForm then //2-
        DoSQL('delete from URepFrm2Qry where IdForm='+ IntToStr(FIdObj)+
              ' and IdQuery='+ IntToStr(oDependItem.IdObj));
      //if (oDependItem.IsDeleted>0) and
      //   (oDependItem.bDelete) then
        URepQueryDelete(oDependItem.IdObj);
      end;
    end;

}

initialization
finalization
  GalochkaOnly.Free

end.
