unit fURepReferences;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  RXSplit, StdCtrls, Buttons, ExtCtrls, ComObj, registry, ActiveX,
  RXCtrls, DB, URepTypeLibs, Placemnt;

type
  TfrmURepReferences = class(TForm)
    Panel1: TPanel;
    Panel2: TPanel;
    Memo1: TMemo;
    RxSplitter1: TRxSplitter;
    Panel3: TPanel;
    BitBtnOK: TBitBtn;
    BitBtnCancel: TBitBtn;
    Label1: TLabel;
    RxCheckListBoxLevel: TRxCheckListBox;
    Panel4: TPanel;
    RxCheckListBoxLib: TRxCheckListBox;
    RxSplitter3: TRxSplitter;
    FormStorage1: TFormStorage;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure BitBtnOKClick(Sender: TObject);
    procedure RxCheckListBoxLibClick(Sender: TObject);
    procedure RxCheckListBoxLibClickCheck(Sender: TObject);
  private
    { Private declarations }
    FrefTLList: TList;
    FListViewEnabled: Boolean;

    FDataSet: TDataSet;

    procedure LoadReferTypeLibList;
  public
    { Public declarations }
    constructor Create(Aowner: TComponent; DataSet: TDataSet); reintroduce;
  end;

implementation

{$R *.DFM}

uses AppUtils, URepConstants;

function CompareReferTypeLibItem_ByName(p1, p2: Pointer): Integer;
var
  Item1, Item2: PReferTypeLibItem;
begin
  Item1 := PReferTypeLibItem(p1);
  Item2 := PReferTypeLibItem(p2);
  result := Ord(Item2.UseType) - Ord(Item1.UseType);
  If result=0 then
  begin
    result := CompareStr(Item1.Name, Item2.Name);
  end;
end;

//-----------------------------------------------------------------------
//  TfrmURepReferences
//-----------------------------------------------------------------------

constructor TfrmURepReferences.Create(AOwner: TComponent; DataSet: TDataSet);
begin
  FDataSet := DataSet;
  inherited Create(AOwner);
end;

procedure TfrmURepReferences.FormCreate(Sender: TObject);
begin
  FrefTLList := TList.Create;
  LoadReferTypeLibList;
end;

procedure TfrmURepReferences.LoadReferTypeLibList;
var
  i, res: integer;
  pReferTLItem: PReferTypeLibItem;
  scrText, sComment: string;
begin
  scrText := FDataSet.FieldByName(fldScriptText).AsString;
  sComment := IdScrEngToCommentStr(FDataSet.FieldByName(fldIdScrEng).asInteger);

  res := LoadFullReferTypeLibItemList(FrefTLList, scrText, sComment);
  If res<>ERROR_SUCCESS then
  begin
    ShowMessage('Error: '+IntToStr(res));
    Exit;
  end;
  FrefTLList.Sort(CompareReferTypeLibItem_ByName);

  FListViewEnabled := False;
  RxCheckListBoxLib.Items.BeginUpdate;
  try
    RxCheckListBoxLib.Items.Clear;

    for i:=0 to FrefTLList.Count-1 do
    begin
      pReferTLItem := PReferTypeLibItem(FrefTLList.Items[i]);

      RxCheckListBoxLib.Items.Add(pReferTLItem.Name+ ' (ver.'+
                          IntToStr(pReferTLItem.VerMajor)+'.'+
                          IntToStr(pReferTLItem.VerMinor)+')');
      If pReferTLItem.UseType in [tlbInternalUse, tlbRegstrUse, tlbScriptUse] then
        RxCheckListBoxLib.Checked[i] := True;
      If pReferTLItem.UseType =tlbInternalUse then
        RxCheckListBoxLib.EnabledItem[i] := False;
    end;
  finally
    RxCheckListBoxLib.Items.EndUpdate;
    FListViewEnabled := True;
  end;
  RxCheckListBoxLib.ItemIndex := 0;
  RxCheckListBoxLibClick(nil);
end;

procedure TfrmURepReferences.FormDestroy(Sender: TObject);
begin
  ClearAndNilReferTypeLibItemList(FrefTLList);
end;

procedure TfrmURepReferences.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
begin
  FListViewEnabled := False;
end;

procedure TfrmURepReferences.BitBtnOKClick(Sender: TObject);
var
  i: integer;
  RegIniFile : TRegIniFile;
  pFullTLItem: PReferTypeLibItem;
  sIdentName, sValue: string;

  procedure AddTypeLibToRegistry;
  begin
    RegIniFile.WriteString(cnstScriptTypeLibs, sIdentName, sValue);
    AddNewTypeLibItemData(ReferedTypeLibItemList, pFullTLItem.Name,
                          pFullTLItem.GUID, pFullTLItem.VerMajor,
                          pFullTLItem.VerMinor, tlbRegstrUse);
  end;

  procedure AddTypeLibToScript;
  var
    isEditMode: Boolean;
    sRef, sScript, sComment: string;
  begin
    // write in script
    If FDataSet=nil Then Exit;
    isEditMode:=False;
    if not(FDataSet.State in [dsEdit, dsInsert]) and
      (FDataSet.RecordCount>=1) then isEditMode:=True;
    if isEditMode then FDataSet.Edit;

    sComment := IdScrEngToCommentStr(FDataSet.FieldByName(fldIdScrEng).Value);
    sRef := CreateScrReferTypeLibStr(sComment, pFullTLItem.GUID,
                        pFullTLItem.VerMajor, pFullTLItem.VerMinor);
    sScript := FDataSet.FieldByName(fldScriptText).AsString;
    FDataSet.FieldByName(fldScriptText).AsString := sRef + Chr(13)+Chr(10) + sScript;
    if isEditMode then FDataSet.Post;
    AddNewTypeLibItemData(ReferedTypeLibItemList, pFullTLItem.Name,
                          pFullTLItem.GUID, pFullTLItem.VerMajor,
                          pFullTLItem.VerMinor, tlbScriptUse);
  end;

  procedure DelTypeLibFromRegistry;
  var
    pRefferTLItem: PReferTypeLibItem;
  begin
    RegIniFile.DeleteKey(cnstScriptTypeLibs, sIdentName);
    // delete item from List
    pRefferTLItem := FindRefferTypeLib(pFullTLItem);
    If pRefferTLItem<>nil then
    begin
      ReferedTypeLibItemList.Remove(pRefferTLItem);
      Dispose(pRefferTLItem);
    end;
  end;

  procedure DelTypeLibFromScript;
  var
    pRefferTLItem: PReferTypeLibItem;
    i, verMajor, verMinor: integer;
    isEditMode: Boolean;
    lines : TStringList;
    GUID: TGUID;
    sComment: String;
  begin
    // write in script
    If FDataSet=nil Then Exit;
    isEditMode:=False;
    if not(FDataSet.State in [dsEdit, dsInsert]) and
      (FDataSet.RecordCount>=1) then isEditMode:=True;
    if isEditMode then FDataSet.Edit;

    sComment := IdScrEngToCommentStr(FDataSet.FieldByName(fldIdScrEng).Value);
    lines := TStringList.Create;
    lines.Text := FDataSet.FieldByName(fldScriptText).AsString;
    for i:=0 to lines.Count-1 do
    begin
      if Pos(sComment, lines.Strings[i])<>1 then Break;
      if not CrackScrReferTypeLibStr(lines.Strings[i], GUID, verMajor, verMinor) then Break;
      If IsEqualGUID(pFullTLItem.GUID, GUID) and
         (verMajor=pFullTLItem.VerMajor) and
         (verMinor=pFullTLItem.VerMinor) then
        lines.Delete(i);
    end;
    FDataSet.FieldByName(fldScriptText).AsString := lines.Text;
    lines.Free;
    if isEditMode then FDataSet.Post;
    // delete item from List
    pRefferTLItem := FindRefferTypeLib(pFullTLItem);
    If pRefferTLItem<>nil then
    begin
      ReferedTypeLibItemList.Remove(pRefferTLItem);
      Dispose(pRefferTLItem);
    end;
  end;

begin
  RegIniFile := TRegIniFile.Create(GetDefaultIniRegKey);
  try
    for i:=0 to FrefTLList.Count-1 do
    begin
      pFullTLItem := PReferTypeLibItem(FrefTLList.Items[i]);
      sIdentName := GUIDToString(pFullTLItem.GUID);
      sValue := IntToStr(pFullTLItem.VerMajor)+'.'+IntToStr(pFullTLItem.VerMinor);
      If pFullTLItem.UseType<>tlbInternalUse then
      begin
        If (pFullTLItem.UseType=tlbNotUse) Then
        begin
          If (pFullTLItem.NewUseType=tlbRegstrUse) then
          begin
            AddTypeLibToRegistry;
          end
          Else If(pFullTLItem.NewUseType=tlbScriptUse) then
          begin
            AddTypeLibToScript;
          end;
        end
        Else If (pFullTLItem.UseType=tlbRegstrUse) then
        begin
          If (pFullTLItem.NewUseType=tlbNotUse) then
          begin
            DelTypeLibFromRegistry;
          end
          else If (pFullTLItem.NewUseType=tlbScriptUse) then
          begin
            DelTypeLibFromRegistry;
            AddTypeLibToScript;
          end;
        end
        Else If (pFullTLItem.UseType=tlbScriptUse) then
        begin
          If (pFullTLItem.NewUseType=tlbNotUse) then
          begin
            DelTypeLibFromScript;
          end
          else If (pFullTLItem.NewUseType=tlbRegstrUse) then
          begin
            DelTypeLibFromScript;
            AddTypeLibToRegistry;
          end;
        end;
      end;
    end;
  finally
    RegIniFile.Free;
  end;
end;


procedure TfrmURepReferences.RxCheckListBoxLibClick(Sender: TObject);
var
  pReferTLItem : PReferTypeLibItem;
  iCheckedIndex : integer;
  bEnabled : Boolean;
begin
  If not FListViewEnabled then Exit;
  pReferTLItem := PReferTypeLibItem(FrefTLList.Items[RxCheckListBoxLib.ItemIndex]);
  with Memo1.Lines do
  begin
   BeginUpdate;
   Clear;
   Add('Path : '+pReferTLItem.Path);
   Add('GUID : '+GUIDToString(pReferTLItem.GUID));
   Add('lcid : '+IntToStr(pReferTLItem.lcid));
   EndUpdate;
  end;

  bEnabled := False; iCheckedIndex := 0;
  case pReferTLItem.NewUseType of
   tlbNotUse:      begin iCheckedIndex := 0; bEnabled := False; end;
   tlbRegstrUse:   begin iCheckedIndex := 3; bEnabled := True; end;
   tlbScriptUse:   begin iCheckedIndex := 2; bEnabled := True; end;
   tlbInternalUse: begin iCheckedIndex := 1; bEnabled := False; end;
  end;
  RxCheckListBoxLevel.ItemIndex := -1;
  RxCheckListBoxLevel.CheckedIndex := iCheckedIndex;
  RxCheckListBoxLevel.EnabledItem[2] := bEnabled;
  RxCheckListBoxLevel.EnabledItem[3] := bEnabled;
end;

procedure TfrmURepReferences.RxCheckListBoxLibClickCheck(Sender: TObject);
var
  pReferTLItem: PReferTypeLibItem;
begin
  If not FListViewEnabled then Exit;
  pReferTLItem := PReferTypeLibItem(FrefTLList.Items[RxCheckListBoxLib.ItemIndex]);

  If RxCheckListBoxLib.Checked[RxCheckListBoxLib.ItemIndex] Then
  begin
    if RxCheckListBoxLevel.CheckedIndex=2 then
      pReferTLItem.NewUseType := tlbScriptUse
    else
      pReferTLItem.NewUseType := tlbRegstrUse;
  end    
  else
    pReferTLItem.NewUseType := tlbNotUse;
end;

end.
