unit fUrepActScpSite;

interface

uses
  Windows, Classes, SysUtils, Forms, Dialogs,
  ComObj, ActiveX, ActivScp, URepGen_TLB, DBmwEdit;

type
  TURepActiveScriptSite = class(TInterfacedObject,
                                IActiveScriptSite, IActiveScriptSiteWindow)
  private
    FGLURepMain : URepMain;
    FmwEdit: TDBmwEdit;
  protected
    // IActiveScriptSite
    function GetLCID(out plcid: UINT): HResult; stdcall;
    function GetItemInfo(pstrName: PWideChar; dwReturnMask: UINT; out ppiunkItem: IUnknown;
                         out ppti: ITypeInfo): HResult; stdcall;
    function GetDocVersionString(out pbstrVersion: WideString): HResult; stdcall;
    function OnScriptTerminate(var pvarResult: OleVariant; var pexcepinfo: EXCEPINFO): HResult; stdcall;
    function OnStateChange(ssScriptState: tagSCRIPTSTATE): HResult; stdcall;
    function OnScriptError(const pscripterror: IActiveScriptError): HResult; stdcall;
    function OnEnterScript: HResult; stdcall;
    function OnLeaveScript: HResult; stdcall;
    // IActiveScriptSiteWindow
    function GetWindow(out wnd: HWnd): HResult; stdcall;
    function EnableModeless(fEnable: Integer): HResult; stdcall;
  public
    constructor Create(mwEdit: TDBmwEdit);
    destructor Destroy; override;
  end;


procedure RunScript(ScrEngName: string; ScriptText: WideString;
                    IdScript: integer; ScriptName: string;
                    mwEdit: TDBmwEdit = nil);
const
  GlObjURepMainName: WideString = 'GlobalURepMain';

implementation

uses Impl_URepObjectFactory, Impl_URepMain, URepTypeLibs, fURepFunc0;


//=========================================================================
//  RunScript - function
//=========================================================================
procedure RunScript(ScrEngName: string; ScriptText: WideString;
                    IdScript: integer; ScriptName: string;
                    mwEdit: TDBmwEdit = nil);
var
  i_IActiveScriptSite: IActiveScriptSite;
  i_IActiveScript: IActiveScript;
  i_IActiveScriptParse: IActiveScriptParse;
  varResult: OleVariant;
  ExcepInfo: TExcepInfo;
begin
  //  
  i_IActiveScriptSite := TURepActiveScriptSite.Create(mwEdit);
  OleCheck(GetIActiveScript( Trim(ScrEngName),
                             i_IActiveScript));
  i_IActiveScriptParse := i_IActiveScript as IActiveScriptParse;
  OleCheck(i_IActiveScript.SetScriptSite(i_IActiveScriptSite));
  OleCheck(i_IActiveScriptParse.InitNew);
  OleCheck(i_IActiveScript.AddNamedItem(PWideChar(GlObjURepMainName),
                               SCRIPTITEM_ISVISIBLE or SCRIPTITEM_ISSOURCE));
  ActiveScript_AddTypeLib(i_IActiveScript, ScriptText, ScrEngName);
  i_IActiveScriptParse.ParseScriptText(PWideChar(ScriptText),
                                  PWideChar(GlObjURepMainName),
                                  nil, nil, 0, 0, 0, varResult, ExcepInfo);
  URepLogAction(3, IdScript, ScriptName); // log
  OleCheck(i_IActiveScript.SetScriptState(SCRIPTSTATE_CONNECTED));
  OleCheck(i_IActiveScript.Close);
  i_IActiveScript := nil;
  i_IActiveScriptParse := nil;
  i_IActiveScriptSite := nil;
end;


//=====================================================================
// TURepActiveScriptSite  - implement IActiveScript for URep Scripting
//=====================================================================
// IActiveScriptSite
constructor TURepActiveScriptSite.Create(mwEdit: TDBmwEdit);
begin
  inherited Create;
  //ShowMessage('TURepActiveScriptSite.Create');
  FGLURepMain := TURepMain.Create(GetURepTypeLib, URepMainDisp);
  FmwEdit := mwEdit;
end;

destructor TURepActiveScriptSite.Destroy;
begin
  //ShowMessage('TURepActiveScriptSite.Destroy');
  FGLURepMain := nil;
  inherited Destroy;
end;

function TURepActiveScriptSite.GetLCID(out plcid: UINT): HResult;
begin
  result := E_NOTIMPL;
end;

function TURepActiveScriptSite.GetItemInfo(pstrName: PWideChar; dwReturnMask: UINT; out ppiunkItem: IUnknown;
                         out ppti: ITypeInfo): HResult;
var
  wstrName: WideString;
begin
  result := TYPE_E_ELEMENTNOTFOUND;

  if (dwReturnMask and SCRIPTINFO_ITYPEINFO)>0 then
    begin
    ppti := nil;
    result := TYPE_E_ELEMENTNOTFOUND;
    Exit;
    end;

  if (dwReturnMask and SCRIPTINFO_IUNKNOWN)>0  then
    begin
    ppiunkItem := nil;
    wstrName := pstrName;
    if wstrName=GlObjURepMainName then
      begin
      ppiunkItem := FGLURepMain;
      result := S_OK;
      end
    else
      result := TYPE_E_ELEMENTNOTFOUND;
    end;
end;

function TURepActiveScriptSite.GetDocVersionString(out pbstrVersion: WideString): HResult;
begin
  result := E_NOTIMPL;
end;

function TURepActiveScriptSite.OnScriptTerminate(var pvarResult: OleVariant; var pexcepinfo: EXCEPINFO): HResult;
begin
  result := S_OK;
end;

function TURepActiveScriptSite.OnStateChange(ssScriptState: tagSCRIPTSTATE): HResult;
begin
  result := S_OK;
end;


function TURepActiveScriptSite.OnScriptError(const pscripterror: IActiveScriptError): HResult;
var
  ExcepInfo: TExcepInfo;
  SourceContext: UINT;
  LineNumber: UINT;
  CharacterPosition: Integer;
  ErrMsg: String;
  hhResult: HResult;
  SourceLineTxt: WideString;
  pointXY: TPoint;
begin
  ErrMsg:= '';
  hhResult := pscripterror.GetSourceLineText(SourceLineTxt);
  If Succeeded(hhResult) Then
  begin
    ErrMsg:= SourceLineTxt + Chr(13)+Chr(10);
  end;
  pscripterror.GetExceptionInfo(ExcepInfo);
  pscripterror.GetSourcePosition(SourceContext, LineNumber, CharacterPosition);

  ErrMsg:= ErrMsg + ': '+ExcepInfo.bstrSource + Chr(13)+Chr(10)+
          ': '+ IntToStr(LineNumber+1) + ' : '+IntToStr(CharacterPosition+1)+ Chr(13)+Chr(10)+
          ExcepInfo.bstrDescription;
  ShowMessage(ErrMsg);
  If assigned(FmwEdit) then
  begin
    pointXY.X := CharacterPosition+1;
    pointXY.Y := LineNumber+1;
    FmwEdit.CaretXY := pointXY;
    If FmwEdit.CanFocus Then FmwEdit.SetFocus;
  end;
  result := S_OK;
end;

function TURepActiveScriptSite.OnEnterScript: HResult;
begin
  result := S_OK;
end;

function TURepActiveScriptSite.OnLeaveScript: HResult;
begin
  result := S_OK;
end;

// IActiveScriptSiteWindow
function TURepActiveScriptSite.GetWindow(out wnd: HWnd): HResult;
begin
  //wnd := Forms.Application.DialogHandle;
  wnd := Forms.Application.Handle;
  result := S_OK;
end;

function TURepActiveScriptSite.EnableModeless(fEnable: Integer): HResult;
begin
 result := S_OK;
end;


end.
