unit unAddQRFnc;

interface

uses Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
     qrexpr, QRPrntr, Qrprevr, WordNumber;

{ TQREvNumToRusFunction }
type
  TQREvNumToRusFunction = class(TQREvElementFunction)
  public
    function Calculate : TQREvResult; override;
  end;


{ TQREvUserFIOFunction }
type
  TQREvUserFIOFunction = class(TQREvElementFunction)
  public
    function Calculate : TQREvResult; override;
  end;

{TQRRussianPreviewInterface}
type
  TQRRussianPreviewInterface = class(TQRPreviewInterface)
  public
    function Show(AQRPrinter : TQRPrinter) : TWinControl; override;
    function ShowModal(AQRPrinter : TQRPrinter): TWinControl; override;
  end;

implementation

uses UserInfo;

const
  msgArgCountError = ' - ';

{ TQREvNumToRusFunction }
// NumToRus( fVal [,sIntName0 [, bRod,
//            [, sFracName0 [, iFracAcc
//           [, sIntName1, sIntName2, sFracName1, sFracName2]]]]] )
function TQREvNumToRusFunction.Calculate : TQREvResult;
var
  fValue: Extended;
  bMoney: Boolean;
  iFracAccuracy: integer;
  IntNames : TStringList;
  FracNames : TStringList;
  procedure GetValue0fVal;
    begin
    case Argument(0).Kind of
      resInt :  fValue := Argument(0).intResult;
      resDouble :  fValue := Argument(0).dblResult;
      end;
    end;
  procedure GetValue1sIntName0;
    begin
    IntNames.Add(Argument(1).strResult);
    end;
  procedure GetValue2bRod;
    begin
    bMoney :=Argument(2).booResult;
    end;
  procedure GetValue3sFracName0;
    begin
    FracNames.Add(Argument(3).strResult);
    end;
  procedure GetValue4iFracAcc;
    begin
    iFracAccuracy := Argument(4).intResult;
    end;
begin
  if (ArgList.Count in [1, 2, 3, 4, 5, 9]) then
    begin
    Result.Kind := resString;
    fValue := 0;
    bMoney := True;
    iFracAccuracy := 2;
    IntNames := nil; FracNames := nil;

    if (ArgList.Count=1) and  // NumToRus( fVal )
       ( (Argument(0).Kind = resDouble) or
         (Argument(0).Kind = resInt) ) then
      begin
      GetValue0fVal;
      end;

    if (ArgList.Count=2) and  // NumToRus( fVal, sIntName0 )
       ( (Argument(0).Kind = resDouble) or
         (Argument(0).Kind = resInt) ) and
       (Argument(1).Kind = resString) then
      begin
      GetValue0fVal;
      IntNames := TStringList.Create;
      GetValue1sIntName0;
      end;

    if (ArgList.Count=3) and  // NumToRus( fVal, sIntName0, bRod )
       ( (Argument(0).Kind = resDouble) or
         (Argument(0).Kind = resInt) ) and
       (Argument(1).Kind = resString) and
       (Argument(2).Kind = resBool) then
      begin
      GetValue0fVal;
      IntNames := TStringList.Create;
      GetValue1sIntName0;
      GetValue2bRod;
      end;

    if (ArgList.Count=4) and  // NumToRus( fVal, sIntName0, bRod, sFracName0 )
       ( (Argument(0).Kind = resDouble) or
         (Argument(0).Kind = resInt) ) and
       (Argument(1).Kind = resString) and
       (Argument(2).Kind = resBool) and
       (Argument(3).Kind = resString) then
      begin
      GetValue0fVal;
      IntNames := TStringList.Create;
      GetValue1sIntName0;
      GetValue2bRod;
      FracNames := TStringList.Create;
      GetValue3sFracName0;
      end;

    if (ArgList.Count=5) and // NumToRus( fVal, sIntName0, bRod, sFracName0, iFracAcc)
       ( (Argument(0).Kind = resDouble) or
         (Argument(0).Kind = resInt) ) and
       (Argument(1).Kind = resString) and
       (Argument(2).Kind = resBool) and
       (Argument(3).Kind = resString) and
       (Argument(4).Kind = resInt) then
      begin
      GetValue0fVal;
      IntNames := TStringList.Create;
      GetValue1sIntName0;
      GetValue2bRod;
      FracNames := TStringList.Create;
      GetValue3sFracName0;
      GetValue4iFracAcc;
      end;

    if (ArgList.Count=9) and // NumToRus( fVal, sIntName0, bRod, sFracName0, iFracAcc,
                             //           sIntName1, sIntName2, sFracName1, sFracName2)
       ( (Argument(0).Kind = resDouble) or
         (Argument(0).Kind = resInt) ) and
       (Argument(1).Kind = resString) and
       (Argument(2).Kind = resBool) and
       (Argument(3).Kind = resString) and
       (Argument(4).Kind = resInt) and
       (Argument(5).Kind = resString) and
       (Argument(6).Kind = resString) and
       (Argument(7).Kind = resString) and
       (Argument(8).Kind = resString) then
      begin
      GetValue0fVal;
      IntNames := TStringList.Create;
      GetValue1sIntName0;
      GetValue2bRod;
      FracNames := TStringList.Create;
      GetValue3sFracName0;
      GetValue4iFracAcc;
      IntNames.Add(Argument(5).strResult);
      IntNames.Add(Argument(6).strResult);
      FracNames.Add(Argument(7).strResult);
      FracNames.Add(Argument(8).strResult);
      end;

    try
      Result.StrResult := FullWord_Number(fValue, bMoney, iFracAccuracy,
                                          IntNames, FracNames);
    finally
      if IntNames<>nil then IntNames.Free;
      if FracNames<>nil then FracNames.Free;
      end;
    end
  else
    begin
    Result.Kind := resError;
    Result.StrResult := msgArgCountError;
    end
end;


{ TQREvUserFIOFunction }
function TQREvUserFIOFunction.Calculate : TQREvResult;
begin
  if ArgList.Count = 0 then
    begin
    Result.Kind := resString;
    try
      Result.StrResult := GetFullLoginName;
    except
      Result.StrResult := '????';
      end;
    end
  else
    begin
    Result.Kind := resError;
    Result.StrResult := msgArgCountError;
    end
end;

{TQRRussianPreviewInterface}
function TQRRussianPreviewInterface.Show(AQRPrinter : TQRPrinter) : TWinControl;
begin
  Result := TQRRussianPreview.CreatePreview(Application, AQRPrinter);
  TQRRussianPreview(Result).Show;
  Result.BringToFront;
end;

function TQRRussianPreviewInterface.ShowModal(AQRPrinter : TQRPrinter) : TWinControl;
begin
  Result := TQRRussianPreview.CreatePreview(Application, AQRPrinter);
  TQRRussianPreview(Result).ShowModal;
end;

initialization
 RegisterQRFunction(TQREvNumToRusFunction,'NUMTORUS',
    'NUMTORUS(<X>)|   NumToRus(fVal[,sIntName0[,bRod,[sFracName0[,iFracAcc  [,sIntName1, sIntName2, sFracName1, sFracName2]]]]]', 'vv', '7N');
 RegisterQRFunction(TQREvUserFIOFunction,'USERFIO',
    'USERFIO| ,  ,  ', 'vv', '7');

 RegisterPreviewClass(TQRRussianPreviewInterface);
end.
