unit unAxBowser;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls, ExtCtrls, ComObj, ActiveX;

Type
  TTLItemType =(itTypeLib,
                  itEnums,
                    itEnum,
                      itConst,
                  itObjects,
                    itCoClass,
                    itDispIntf,
                      itPropFunc);

  PTTLItemData = ^TTLItemData;

  TTLTypeLibData = record
    HelpFile: PString;
    GUID: TGUID;
  end;

  TTLCoClassData = record
    GUID: TGUID;
    bCreatable: Boolean;
    ProgID: PString;
  end;

  TTLPropFuncData = class
  private
    FuncId : Integer;
    InvokeKind: TInvokeKind;
    VT_DataType: Word; // VARENUM
    DataTypeName: String; // for Interface
    paramList: TList;
  public
    constructor Create(aFuncId: Integer; aInvokeKind: TInvokeKind;
                       aDataType: word; sDataTypeName: String);
    destructor Destroy; override;
    procedure AddParam(Name: String; wParamFlags: Word;
                       DataType: word; sDataTypeName: String;
                       pPParamDescEx: PParamDescEx);
    procedure SetFuncPrototype(pItemData: PTTLItemData; RichEdit: TRichEdit);
    function GetEnterStr(pItemData: PTTLItemData; IdScrLang: Integer;
                         Lines: TStrings; CaretXY: TPoint) : String;
  end;

  TTLItemList = class;

  TTLItemData = record
    Name : string;
    Comment : string;
    parentItemData: PTTLItemData;
    ChildItemList: TTLItemList; // ChildItemList->Item : PTTLItemData
    case ItemType: TTLItemType of
      itTypeLib : (TypeLibData: TTLTypeLibData );
      itConst   : (iValue: Integer);
      itCoClass : (CoClassData: TTLCoClassData );
      itDispIntf: (intfGUID: TGUID);
      itPropFunc: (PropFuncData: TTLPropFuncData);
  end;

  TTLItemList = class
  private
    FTLItemList : TList;
    function FGetCount: integer;
    function FGetItems(iIndex: integer) : PTTLItemData;
    function FGetItemByName(sName: string) : PTTLItemData;
  public
    property Count: Integer read FGetCount;
    procedure Add(pItemData: PTTLItemData);
    property Items[iIndex: integer] : PTTLItemData read FGetItems; default;
    property ItemByName[sName: string] : PTTLItemData read FGetItemByName;
    procedure Sort(Compare: TListSortCompare);
    procedure Remove(pItemData: PTTLItemData);

    constructor Create;
    destructor Destroy; override;
  end;

procedure LoadTypeLibItemList(i_ITypeLib : ITypeLib; var TLItemList: TTLItemList;
                              DefObjName: string; DefObjGUID: TGUID);
procedure TypeLibItemList2TreeView(TLItemList: TTLItemList; TreeView: TTreeView);
procedure FreeTTLItemList(ItemList: TTLItemList);

procedure OnTLItemChange(pItemData: PTTLItemData; RichEdit: TRichEdit);
Function IsItemCanDragAndDropped(pItemData: PTTLItemData): Boolean;

Const ScrLangVBScript = 1;
      ScrLangJScript  = 2;

function GetEnterStr(pItemData: PTTLItemData; IdScrLang: Integer;
                     Lines: TStrings; CaretXY: TPoint) : String;

Type TTLIListStyles = set of (stlVB, stlFull, stlSortByID, stlSortByName);

procedure SetTypeLibListStyle(style: TTLIListStyles);

implementation

uses unRTFfunc;

Var
    FDefObjName: String; // for Internal Lib ONLY
    FDefObjGUID: TGUID;
    FDefObjPresent: Boolean;
    FTLListStyle: TTLIListStyles;

procedure SetTypeLibListStyle(style: TTLIListStyles);
begin
  FTLListStyle := style;
end;


//    TList
function Compare_TTLItemData(p1, p2: Pointer): Integer;
var
  Item1, Item2: TTLItemData;
  function IsDefaultObject(ItemData: TTLItemData): Boolean;
  begin
    result := (ItemData.ItemType = itDispIntf) and
              (IsEqualGUID(FDefObjGUID, ItemData.intfGUID));
  end;
begin
  result := 0;
  Item1 := (PTTLItemData(p1))^;
  Item2 := (PTTLItemData(p2))^;
  If FDefObjPresent Then
  begin
    If IsDefaultObject(Item1) Then begin result:=-1; Exit; end;
    If IsDefaultObject(Item2) Then begin result:= 1; Exit; end;
  end;
  If (Item1.ItemType=Item2.ItemType) or
     ((Item1.ItemType in [itCoClass,itDispIntf]) and
      (Item2.ItemType in [itCoClass,itDispIntf])) Then
  begin
    result := CompareStr(Item1.Name, Item2.Name);
  end;
end;

function Compare_TTLItemDataMemId(p1, p2: Pointer): Integer;
var
  Item1, Item2: TTLItemData;
  FuncId1, FuncId2: Integer;
begin
  result := 0;
  Item1 := (PTTLItemData(p1))^;
  Item2 := (PTTLItemData(p2))^;
  If (Item1.ItemType= itPropFunc) and
     (Item2.ItemType = itPropFunc) Then
  begin
    FuncId1 := Item1.PropFuncData.FuncId;
    FuncId2 := Item2.PropFuncData.FuncId;
    If FuncId1=FuncId2 Then
      result := Item1.PropFuncData.InvokeKind - Item2.PropFuncData.InvokeKind
    Else
      result := FuncId1 - FuncId2;
  end;
end;

//---------------------------------------------------------------
//           TTLItemList
//---------------------------------------------------------------
constructor TTLItemList.Create;
begin
  FTLItemList := TList.Create;
end;

destructor TTLItemList.Destroy;
begin
  FTLItemList.Free;
  inherited Destroy;
end;

function TTLItemList.FGetCount: integer;
begin
  result := FTLItemList.Count;
end;

function TTLItemList.FGetItems(iIndex: integer) : PTTLItemData;
begin
  result := PTTLItemData(FTLItemList.Items[iIndex]);
end;

function TTLItemList.FGetItemByName(sName: string) : PTTLItemData;
var
  i: integer;
begin
  for i:= 0 to FTLItemList.Count-1 do
  begin
    If FGetItems(i).Name=sName Then
    begin
      result := FGetItems(i);
      Exit;
    end;
  end;
  result := nil;
end;

procedure TTLItemList.Add(pItemData: PTTLItemData);
begin
  FTLItemList.Add(pItemData);
end;

procedure TTLItemList.Remove(pItemData: PTTLItemData);
begin
  FTLItemList.Remove(pItemData);
end;

procedure TTLItemList.Sort(Compare: TListSortCompare);
begin
  FTLItemList.Sort(Compare);
end;

//---------------------------------------------------------------
// TTLPropFuncData
//---------------------------------------------------------------
Type
  TTLPropFuncParam = record
    Name: String;
    wParamFlags: Word;
    VT_DataType: Word;  //VARENUM
    DataTypeName: String;
    varDefaultValue: Variant;
  end;
  PTTLPropFuncParam = ^TTLPropFuncParam;

constructor TTLPropFuncData.Create(aFuncId: Integer; aInvokeKind: TInvokeKind;
                                   aDataType: word; sDataTypeName: String);
begin
  inherited Create;
  Self.FuncId := aFuncId;
  Self.InvokeKind := aInvokeKind;
  Self.VT_DataType := aDataType;
  Self.DataTypeName := sDataTypeName;
  Self.paramList := nil;
end;

destructor TTLPropFuncData.Destroy;
var
  i: integer;
  p: PTTLPropFuncParam;
begin
  If Self.paramList<>nil Then
  begin
    for i:=0 to Self.paramList.Count-1 do
    begin
      p := PTTLPropFuncParam(paramList.Items[i]);
      If p<>nil Then
      begin
        Dispose(p);
        paramList.Items[i] := nil;
      end;
    end;
    FreeAndNil(Self.paramList);
  end;
  inherited Destroy;
end;


procedure TTLPropFuncData.AddParam(Name: String; wParamFlags: Word;
                        DataType: word; sDataTypeName: String;
                        pPParamDescEx: PParamDescEx);
var
  p: PTTLPropFuncParam;
  varDefaultValue: TVariantArg;
  v: OleVariant;
  ws: WideString;
begin
  If Self.paramList = nil then
    Self.paramList := TList.Create;
  New(p);
  p.Name := Name;
  p.wParamFlags := wParamFlags;
  p.VT_DataType := DataType;
  p.DataTypeName := sDataTypeName;
  If ((wParamFlags and PARAMFLAG_FOPT) >0) and
     ((wParamFlags and PARAMFLAG_FHASDEFAULT) >0) Then
  begin
    varDefaultValue := pPParamDescEx.varDefaultValue;
    case varDefaultValue.vt of
      VT_UI1:   v := varDefaultValue.bVal;
      VT_I2:    v := varDefaultValue.iVal;
      VT_I4:    v := varDefaultValue.lVal;
      VT_R4:    v := varDefaultValue.fltVal;
      VT_R8:    v := varDefaultValue.dblVal;
      VT_BOOL:  v := varDefaultValue.vbool;
      VT_CY:    v := varDefaultValue.cyVal;
      VT_DATE:  v := varDefaultValue.date;
      VT_BSTR:  begin
                ws := varDefaultValue.bstrVal;
                v := ws;
                end;
      else v := '????'
    end;
    p.varDefaultValue := v;
  end;
  Self.paramList.Add(p);
end;

function OLEDataTypeToStr(DataType: word; Name: String) : String;
begin
  case DataType of
    vt_i2       : result := 'Smallint';   //varSmallint = $0002;
    vt_i4       : result := 'Integer'; //varInteger  = $0003;
    vt_r4       : result := 'Single';//varSingle   = $0004;
    vt_r8       : result := 'Double';//varDouble   = $0005;
    vt_cy       : result := 'Currency';//varCurrency = $0006;
    vt_date     : result := 'Date';//varDate     = $0007;
    vt_bstr     : result := 'String';//varOleStr   = $0008;
    vt_dispatch : result := 'Dispatch';//varDispatch = $0009;
    vt_error    : result := 'Error';//varError    = $000A;
    vt_bool     : result := 'Boolean';//varBoolean  = $000B;
    vt_variant  : result := 'Variant';//varVariant  = $000C;
    VT_UI1      : result := 'Byte';
    VT_INT      : result := 'Long';
    else
      If Name<>'' Then
        result := Name
      Else
        result := '????'+IntToStr(DataType);
  end;
end;


procedure TTLPropFuncData.SetFuncPrototype(pItemData: PTTLItemData; RichEdit: TRichEdit);
var
  i, k : integer;
  pFuncParam : PTTLPropFuncParam;
Const
  InvokeValueStr : array [1..3] of string = ('Get', 'Put', 'Set');
  function varDefaultValueToStr(v: OleVariant): string;
  begin
    case VarType(v) of
      varOleStr  : result := ''''+VarToStr(v)+'''';
      varBoolean : result := VarToStr(v);
      else result := VarToStr(v);
    end;
  end;
begin
  If Self.InvokeKind = INVOKE_FUNC Then
  begin
    If Self.VT_DataType in [VT_VOID, VT_HRESULT] then
      RichEdit.SelText := 'Sub '
    Else
      RichEdit.SelText := 'Function ';
  end
  Else begin
    RichEdit.SelText := 'Property ';
    If stlVB in FTLListStyle then
    begin
      If Self.InvokeKind=INVOKE_PROPERTYGET then
        RichEdit.SelText := 'Read-Only '
      Else If (Self.InvokeKind and INVOKE_PROPERTYGET)=0 then
        RichEdit.SelText := 'Write-Only ';
    end
    else begin
      For i:= 1 to 3 do
      begin
        k := 1 shl i;
        If (k and Self.InvokeKind)>0 then
          RichEdit.SelText := InvokeValueStr[i]+ ' ';
       end;
    end;
  End;

  SelTexBold(RichEdit, pItemData.Name);
  If Self.paramList<>nil Then
  begin
    For i:= 0 to Self.paramList.Count-1 do
    begin
      pFuncParam := Self.paramList.Items[i];
      If i = 0 then
        RichEdit.SelText :=  '(';

      If (pFuncParam.wParamFlags and PARAMFLAG_FOPT) >0 Then
        RichEdit.SelText := '[';

      RichEdit.SelText := pFuncParam.Name;

      If pFuncParam.VT_DataType<>VT_VARIANT Then
        RichEdit.SelText :=' as ' +
                           OLEDataTypeToStr(pFuncParam.VT_DataType, pFuncParam.DataTypeName);

      If ((pFuncParam.wParamFlags and PARAMFLAG_FOPT) >0) and
         ((pFuncParam.wParamFlags and PARAMFLAG_FHASDEFAULT) >0) Then
        RichEdit.SelText := ' ='+ varDefaultValueToStr(pFuncParam.varDefaultValue);

      If (pFuncParam.wParamFlags and PARAMFLAG_FOPT) >0 Then
        RichEdit.SelText := ']';

      If i = Self.paramList.Count-1 Then
        RichEdit.SelText := ')'
      else
        RichEdit.SelText := ', ';
    end;
  end;
  If not(Self.VT_DataType in [VT_VOID, VT_HRESULT]) then
    RichEdit.SelText := ' as ' + OLEDataTypeToStr(Self.VT_DataType, Self.DataTypeName);
end;

function TTLPropFuncData.GetEnterStr(pItemData: PTTLItemData; IdScrLang: Integer;
                         Lines: TStrings; CaretXY: TPoint) : String;
var
  ss: string;
  i : integer;
  bSubCall: Boolean;
  pFuncParam : PTTLPropFuncParam;
  Procedure AddSs(s2: string);
  begin
    ss := ss + s2;
  end;
begin
  bSubCall := False;
  case IdScrLang of
    ScrLangVBScript:
    begin
      If (Self.InvokeKind = INVOKE_FUNC) and
         (Self.VT_DataType=VT_VOID) then
        bSubCall := True;

      ss := pItemData.Name;
      If Self.paramList<>nil Then
      begin
        For i:= 0 to Self.paramList.Count-1 do
        begin
          pFuncParam := Self.paramList.Items[i];
          If (i = 0) then
            If bSubCall then AddSs(' ') else AddSs('(');

          If (pFuncParam.wParamFlags and PARAMFLAG_FOPT) >0 Then
            AddSs('[');

          AddSs(pFuncParam.Name);

          If (pFuncParam.wParamFlags and PARAMFLAG_FOPT) >0 Then
            AddSs(']');

          If i = Self.paramList.Count-1 Then
            If bSubCall then AddSs(' ') else AddSs(')')
          else
            AddSs(', ');
        end;
      end;
    end;
  end;
  result := ss;
end;


//---------------------------------------------------------------
//  
//---------------------------------------------------------------
Function NewItemData(ItemType: TTLItemType; Name, Comment: String): PTTLItemData;
var
  PItemData: PTTLItemData;
begin
  New(PItemData);
  PItemData.ItemType := ItemType;
  PItemData.Name := Trim(Name);
  PItemData.Comment := Trim(Comment);
  PItemData.ChildItemList := nil;
  case ItemType of
  itTypeLib : PItemData.TypeLibData.HelpFile := nil;
  itPropFunc: PItemData.PropFuncData := nil;
  itCoClass:  PItemData.CoClassData.ProgID := nil;
  end;
  result := PItemData;
end;

procedure FreeItemDataTypeRecs(PItemData: PTTLItemData);
begin
  case PItemData.ItemType of
  itTypeLib :
  begin
    DisposeStr(PItemData.TypeLibData.HelpFile);
    PItemData.TypeLibData.HelpFile := nil;
  end;
  itCoClass:
  begin
    DisposeStr(PItemData.CoClassData.ProgID);
    PItemData.CoClassData.ProgID := nil;
  end;
  itPropFunc:
    If PItemData.PropFuncData <> nil Then
      FreeAndNil(PItemData.PropFuncData);
  end;
end;


procedure FreeTTLItemList(ItemList: TTLItemList);
var
  i : integer;
  PItemData: PTTLItemData;
  ChildList : TTLItemList;
begin
  for i:=0 to ItemList.Count - 1 do
  begin
    PItemData := ItemList.Items[i];
    FreeItemDataTypeRecs(PItemData);
    ChildList := PItemData.ChildItemList;
    If ChildList<>nil then
    begin
      FreeTTLItemList(ChildList);
    end;
    Dispose(PItemData);
  end;
  ItemList.Free;
end;


procedure AddChild(parentItemData, childItemData: PTTLItemData);
begin
  If parentItemData.ChildItemList=nil Then
    parentItemData.ChildItemList := TTLItemList.Create;
  parentItemData.ChildItemList.add(childItemData);
  childItemData.parentItemData := parentItemData;
end;



procedure SetNodeProperty(node: TTreeNode; PItemData: PTTLItemData);
var
  sText: string;
  iImageIndex, iSelectedIndex: Integer;
begin
  Node.Data := pItemData;

  with pItemData^ do
  begin
    sText := Name;
    If Length(Comment)>0 Then
      sText := sText + '  {' + Comment+'}';
    Node.Text := sText;

    iSelectedIndex := -1;
    case ItemType of
    itTypeLib :
      If (stlVB in FTLListStyle) then iImageIndex := 0
      Else iImageIndex := 12;
    itEnums, itObjects:
      begin iImageIndex := 1; iSelectedIndex := 2; end;
    itEnum:
      If (stlVB in FTLListStyle) then iImageIndex := 3
      Else iImageIndex := 13;
    itConst:
      If (stlVB in FTLListStyle) then iImageIndex := 4
      Else iImageIndex := 14;
    itCoClass:
      If (stlVB in FTLListStyle) then iImageIndex := 6
      Else iImageIndex := 15;
    itDispIntf:
      If FDefObjPresent and
         IsEqualGUID(intfGUID, FDefObjGUID) Then
        If (stlVB in FTLListStyle) then iImageIndex := 5
        Else iImageIndex := 16
      Else
        If (stlVB in FTLListStyle) then iImageIndex := 6
        Else iImageIndex := 17;
    itPropFunc:
      begin
      If (PropFuncData.InvokeKind and INVOKE_FUNC)>0 then
        If (stlVB in FTLListStyle) then iImageIndex := 7
        Else iImageIndex := 18
      Else
        If (stlVB in FTLListStyle) then
          If PropFuncData.FuncId=0 Then  iImageIndex := 8
          Else iImageIndex := 9
        Else
          If (PropFuncData.InvokeKind=INVOKE_PROPERTYGET) then iImageIndex := 19
          Else iImageIndex := 20;
      end
    else
      iImageIndex := -1;
    end;
    Node.ImageIndex := iImageIndex;
    If iSelectedIndex<0 Then  Node.SelectedIndex := iImageIndex
    Else Node.SelectedIndex := iSelectedIndex;
  end;
end;


procedure FillTreeBranchFromList(TreeView: TTreeView; ParentNode: TTreeNode; ItemList: TTLItemList);
var
  i : integer;
  node1 : TTreeNode;
  ChildList : TTLItemList;
begin
  for i:=0 to ItemList.Count - 1 do
  begin
    node1 := TreeView.Items.AddChild(ParentNode, '');
    SetNodeProperty(node1, ItemList.Items[i]);
    ChildList := ItemList.Items[i].ChildItemList;
    If ChildList<>nil then
    begin
      FillTreeBranchFromList(TreeView, node1, ChildList);
    end;
  end;
end;


Function TypeDescToString(i_ITypeInfo:ITypeInfo; tdesc: TTypeDesc): String;
var
  i_RefTypeInfo: ITypeInfo;
  Name, DocString : WideString;
  n: integer;
begin
  result := '';
  If (tdesc.vt=VT_PTR) Then
    result := TypeDescToString(i_ITypeInfo, tdesc.ptdesc^);
  If ((tdesc.vt and $0FFF) = VT_CARRAY) Then
  begin
    result := TypeDescToString(i_ITypeInfo, tdesc.padesc.tdescElem);
    for n := 0 to tdesc.padesc.cDims-1 do
    begin
      result := result + '[' + IntToStr(tdesc.padesc.rgbounds[n].cElements)+ ']';
    end;
  end;
  If ((tdesc.vt and $0FFF) = VT_SAFEARRAY) Then
    result := 'SAFEARRAY(' + TypeDescToString(i_ITypeInfo, tdesc.ptdesc^) + ')' ;
  If (tdesc.vt=VT_USERDEFINED) Then
  begin
    // Use ptdesc->hreftype and pti->GetRefTypeInfo
    OleCheck(i_ITypeInfo.GetRefTypeInfo( tdesc.hreftype, i_RefTypeInfo));
    OleCheck(i_RefTypeInfo.GetDocumentation(MEMBERID_NIL, @Name, @DocString, nil, nil));
    result := Name;
  end;
  If tdesc.vt<14 Then  //  
    result := OLEDataTypeToStr(tdesc.vt, '');
end;


//---------------------------------------------------------------------
//    LoadTypeLibItemList
//---------------------------------------------------------------------
procedure WrapDispIntfPropertys(PropMethodList: TTLItemList);
var
  l, l2, FuncId, FuncId2: integer;
  PPropFuncItemData, PPropFuncItemData2: PTTLItemData;
begin
  If PropMethodList=nil then Exit;
  l:=0;
  while l<PropMethodList.Count do
  begin
    PPropFuncItemData := PropMethodList.Items[l];
    FuncId := PPropFuncItemData.PropFuncData.FuncId;
    l2:=l + 1;  // PropMethodList sorted by FuncId + InvokeKind
    If l2<PropMethodList.Count Then
    begin
      PPropFuncItemData2 := PropMethodList.Items[l2];
      FuncId2 := PPropFuncItemData2.PropFuncData.FuncId;
      If FuncId=FuncId2 Then
      begin
        PPropFuncItemData.PropFuncData.InvokeKind :=
          PPropFuncItemData.PropFuncData.InvokeKind +
            PPropFuncItemData2.PropFuncData.InvokeKind;
        PropMethodList.Remove(PPropFuncItemData2);
        FreeItemDataTypeRecs(PPropFuncItemData2);
        Dispose(PPropFuncItemData2);
      end else
        Inc(l);
    end else
      Inc(l);
  end;
end;


procedure LoadDispIntfList(Name, DocString: WideString;
                          parentItemData: PTTLItemData; DispIntfList: TTLItemList;
                          TypeAttr: PTypeAttr; i_TypeInfo: ITypeInfo);
var
  j, k: integer;
  FuncDesc: PFuncDesc;
  PDispItemData, PPropFuncItemData: PTTLItemData;
  ElemDesc: TElemDesc;
  ParamDesc: TParamDesc;
  rgbstrNames: array[0..255] of TBStr;
  cNames: Integer;
  paramName: string;
  HelpContext: Longint;
begin
  PDispItemData := NewItemData(itDispIntf, Name, DocString);
  PDispItemData.parentItemData := parentItemData;
  PDispItemData.intfGUID := TypeAttr.guid;

  DispIntfList.Add(PDispItemData);
  for j:=0 to TypeAttr.cFuncs - 1 do //  
  begin
    OleCheck(i_TypeInfo.GetFuncDesc(j, FuncDesc));
    try
      If (FuncDesc.memid>=0) and
         ((FuncDesc.wFuncFlags and FUNCFLAG_FRESTRICTED)=0) and
         ((FuncDesc.wFuncFlags and FUNCFLAG_FHIDDEN)=0) Then  // user only
      begin
        OleCheck(i_TypeInfo.GetDocumentation(FuncDesc.memid, @Name, @DocString, @HelpContext, nil));
        PPropFuncItemData := NewItemData(itPropFunc, Name, DocString);
        PPropFuncItemData.PropFuncData :=
          TTLPropFuncData.Create(FuncDesc.memid, FuncDesc.invkind,
                                 FuncDesc.elemdescFunc.tdesc.vt,
                                 TypeDescToString(i_TypeInfo, FuncDesc.elemdescFunc.tdesc));
        OleCheck(i_TypeInfo.GetNames(FuncDesc.memid, @rgbstrNames, 255, cNames ));
        AddChild(PDispItemData, PPropFuncItemData);
        for k:= 0 to (FuncDesc.cParams-1) do //  
        begin
          ElemDesc := FuncDesc.lprgelemdescParam[k];
          ParamDesc := ElemDesc.paramdesc; // ?
          // fix for 'rhs' problem
          if cNames > (k+1) Then
            paramName := rgbstrNames[k+1]
          else
            paramName := 'rhs';
          PPropFuncItemData.PropFuncData.AddParam(paramName, ParamDesc.wParamFlags,
            ElemDesc.tdesc.vt, TypeDescToString(i_TypeInfo, ElemDesc.tdesc),
            ParamDesc.pparamdescex);
          // ...
        end;
      end;
    finally
      i_TypeInfo.ReleaseFuncDesc(FuncDesc);
    end;
  end;
  If PDispItemData.ChildItemList<> nil Then
  begin
    PDispItemData.ChildItemList.Sort(Compare_TTLItemDataMemId); // sort for wrap
    if (stlVB in FTLListStyle) then
      WrapDispIntfPropertys(PDispItemData.ChildItemList);
    If stlSortByName in FTLListStyle then
      PDispItemData.ChildItemList.Sort(Compare_TTLItemData); // sort for wrap
  end;
end;


function IsDispIntfPresentInCoClass(dispGUID: TGUID; VBClassList: TTLItemList): Boolean;
var
  i: integer;
  pIItemData: PTTLItemData;
begin
  result := False;
  If VBClassList=nil then Exit;
  for i:=0 to VBClassList.Count-1 do
  begin
    pIItemData := VBClassList.Items[i];
    If pIItemData.ItemType=itDispIntf then
      If IsEqualGUID(dispGUID,pIItemData.intfGUID) then
      begin
        result := True;
        Break;
      end;
    If pIItemData.ItemType=itCoClass then
      If  IsDispIntfPresentInCoClass(dispGUID, pIItemData.ChildItemList) Then
      begin
        result := True;
        Break;
      end;
  end;
end;


procedure WrapCoClassDispIntf(VBClassList: TTLItemList; darEventsGUID: array of TGUID);
var
  i, j: integer;
  pCoClassItemData, pDispIntfItemData: PTTLItemData;
  nameDispIntf, nameCoClass: TStrings;
  CoClassChildList, DispIntChildList: TTLItemList;
  procedure RenameParametrs(ItemList: TTLItemList);
  var
    pItemData : PTTLItemData;
    iIndex, i: integer;
  begin
    If ItemList=nil then Exit;
    for i:=0 to ItemList.Count-1 do
    begin
      pItemData := ItemList.Items[i];
      case pItemData.ItemType of
        itCoClass ,
        itDispIntf: RenameParametrs(pItemData.ChildItemList);
        itPropFunc:
           begin
           iIndex := nameDispIntf.IndexOf(pItemData.PropFuncData.DataTypeName);
           If iIndex>=0 then
             pItemData.PropFuncData.DataTypeName := nameCoClass.Strings[iIndex];
           end;
      end;
    end;
  end;
begin
  If VBClassList=nil then Exit;
  nameDispIntf := TStringList.Create;
  nameCoClass := TStringList.Create;

  for i:=0 to VBClassList.Count-1 do
  begin
    pCoClassItemData := VBClassList.Items[i];
    if pCoClassItemData.ItemType=itCoClass then
    begin
      CoClassChildList := pCoClassItemData.ChildItemList;
      If (CoClassChildList<>nil) and
         (CoClassChildList.Count=1) then
      begin
        pDispIntfItemData := CoClassChildList.Items[0];
        If pDispIntfItemData.ItemType=itDispIntf then
        begin
          nameCoClass.Add(pCoClassItemData.Name);
          nameDispIntf.Add(pDispIntfItemData.Name);
          DispIntChildList := pDispIntfItemData.ChildItemList;
          pCoClassItemData.ChildItemList := DispIntChildList;
          CoClassChildList.Free;
          pDispIntfItemData.ChildItemList := nil;
          Dispose(pDispIntfItemData);
          If DispIntChildList<>nil then
            for j:=0 to DispIntChildList.Count-1 do
              DispIntChildList.Items[j].parentItemData := pCoClassItemData;
        end;
      end;
    end;
  end;

  RenameParametrs(VBClassList);
  nameCoClass.Free;
  nameDispIntf.Free;
end;

procedure LoadTypeLibItemList(i_ITypeLib : ITypeLib; var TLItemList: TTLItemList;
                              DefObjName: string; DefObjGUID: TGUID);
var
  Name, DocString, HelpFile: WideString;
  tlibattr: PTLibAttr;
  HelpContext: Longint;

  n, i, TypeInfoCount: Integer;
  i_TypeInfo: ITypeInfo;
  TypeAttr: PTypeAttr;

  VBClassList: TTLItemList;
  pTypeLibItemData, pEnumsItemData, pObjectsItemData: PTTLItemData;
  darEventsGUID: array of TGUID;

  procedure LoadEnumList;
  var
    j: integer;
    VarDesc: PVarDesc;
    vValue: OleVariant;
    PEnumData, PConstData: PTTLItemData;
  begin
    PEnumData := NewItemData(itEnum, Name, DocString);
    AddChild(pEnumsItemData, PEnumData);

    for j:=0 to TypeAttr.cVars - 1 do  //  
    begin
      OleCheck(i_TypeInfo.GetVarDesc(j, VarDesc));
      try
        OleCheck(i_TypeInfo.GetDocumentation(VarDesc.memid, @Name, @DocString, @HelpContext, nil));
        vValue := (VarDesc.lpvarValue)^;
        PConstData := NewItemData(itConst, Name, DocString);
        PConstData.iValue := vValue;
        AddChild(PEnumData, PConstData);
      finally
        i_TypeInfo.ReleaseVarDesc(VarDesc);
      end;
    end;
  end;

  procedure LoadCoClassList;
  var
    PCoClassData: PTTLItemData;
    j, impltypeflags, iGUID: integer;
    reftype: Cardinal;
    implmntTI: ITypeInfo;
    implmntTypeAttr: PTypeAttr;
  begin
    PCoClassData := NewItemData(itCoClass, Name, DocString);
    PCoClassData.parentItemData := pObjectsItemData;
    PCoClassData.CoClassData.GUID := TypeAttr.guid;
    PCoClassData.CoClassData.bCreatable :=
          (TypeAttr.wTypeFlags and TYPEFLAG_FCANCREATE)>0;
    if PCoClassData.CoClassData.bCreatable then
      PCoClassData.CoClassData.ProgID := NewStr(ClassIDToProgID(TypeAttr.guid));

    VBClassList.Add(PCoClassData);

    for j:=0 to TypeAttr.cImplTypes - 1 do  //  
    begin
      If j=0 Then
      begin
        PCoClassData.ChildItemList := TTLItemList.Create;
      end;
      OleCheck(i_TypeInfo.GetImplTypeFlags(j, impltypeflags));
      OleCheck(i_TypeInfo.GetRefTypeOfImplType(j, reftype));
      OleCheck(i_TypeInfo.GetRefTypeInfo(reftype, implmntTI));
      OleCheck(implmntTI.GetDocumentation(-1, @Name, @DocString, @HelpContext, nil));
      OleCheck(implmntTI.GetTypeAttr(implmntTypeAttr));
      try
        If (stlFull in FTLListStyle) or
           ((stlVB in FTLListStyle) and (j=0)) then  //   !
          LoadDispIntfList(Name, DocString, PCoClassData,
                           PCoClassData.ChildItemList, implmntTypeAttr, implmntTI);
        if (stlVB in FTLListStyle) and (j>0) and
           ((implmntTypeAttr.wTypeFlags and TYPEFLAG_FHIDDEN)=0) Then // Hide HIDDEN
        begin
          iGUID := Length(darEventsGUID);
          SetLength(darEventsGUID, iGUID+1);
          darEventsGUID[iGUID] := implmntTypeAttr.guid;
        end;

      finally
        implmntTI.ReleaseTypeAttr(implmntTypeAttr);
      end;
    end;
  end;
  function IsDispIntfPresentInEvents(dispGUID: TGUID): Boolean;
  var
    i: integer;
  begin
    result := False;
    for i:=Low(darEventsGUID) to High(darEventsGUID) do
      If IsEqualGUID(darEventsGUID[i], dispGUID) then
      begin
        result := True;
        Break;
      end;
  end;
begin
  FDefObjName := DefObjName;
  FDefObjGUID := DefObjGUID;
  FDefObjPresent := not IsEqualGUID(FDefObjGUID, GUID_NULL);

  If TLItemList=nil Then
    TLItemList := TTLItemList.Create;
  VBClassList := TTLItemList.Create;

  // Get TypeLib property
  OleCheck(i_ITypeLib.GetDocumentation(-1, @Name, @DocString, @HelpContext, @HelpFile));
  pTypeLibItemData := NewItemData(itTypeLib, Name, DocString);
  pTypeLibItemData.TypeLibData.HelpFile := NewStr(HelpFile) ;
  OleCheck(i_ITypeLib.GetLibAttr(tlibattr));
  try
    pTypeLibItemData.TypeLibData.GUID := tlibattr.guid;
  finally
    i_ITypeLib.ReleaseTLibAttr(tlibattr);
  end;

  // Create Root objects (Enums, Objects)
  pObjectsItemData := NewItemData(itObjects, 'Objects', '');
  AddChild(pTypeLibItemData, pObjectsItemData);
  pObjectsItemData.ChildItemList := VBClassList;

  pEnumsItemData := NewItemData(itEnums, 'Enums', '');
  AddChild(pTypeLibItemData, pEnumsItemData);

  // Fill Lists
  TypeInfoCount := i_ITypeLib.GetTypeInfoCount();
  for n:=1 to 2 do // 2 pass: CoClass, DispIntf
  begin
    for i:=0 to (TypeInfoCount-1) do
    begin
      OleCheck(i_ITypeLib.GetTypeInfo(i, i_TypeInfo));
      OleCheck(i_TypeInfo.GetDocumentation(-1, @Name, @DocString, @HelpContext, nil));
      OleCheck(i_TypeInfo.GetTypeAttr(TypeAttr));
      try
        If ((TypeAttr.wTypeFlags and TYPEFLAG_FHIDDEN)=0) then // hide HIDDEN
          case TypeAttr.typekind of
            TKIND_ENUM:
              If n=1 Then LoadEnumList;
            TKIND_DISPATCH:
              If n=2 Then
                If (not IsDispIntfPresentInCoClass(TypeAttr.guid, VBClassList)) and
                   ( (stlFull in FTLListStyle) or
                     ( (not IsDispIntfPresentInEvents(TypeAttr.guid)) and
                       (copy(Name,1,1)<>'_') ) ) Then
                  LoadDispIntfList(Name, DocString, pObjectsItemData,
                                   VBClassList, TypeAttr, i_TypeInfo);
            TKIND_COCLASS:
              If n=1 Then LoadCoClassList;
          end;
      finally
        i_TypeInfo.ReleaseTypeAttr(TypeAttr);
      end;
    end;
  end;

  // Sort and Proccess Lists
  If pEnumsItemData.ChildItemList<>nil then
    pEnumsItemData.ChildItemList.Sort(Compare_TTLItemData);
  If stlVB in FTLListStyle then
    WrapCoClassDispIntf(VBClassList, darEventsGUID);
  VBClassList.Sort(Compare_TTLItemData);

  // Fill Tree Item List
  TLItemList.Add(pTypeLibItemData);
end;

procedure TypeLibItemList2TreeView(TLItemList: TTLItemList; TreeView: TTreeView);
var
  i: integer;
begin
  FillTreeBranchFromList(TreeView, nil, TLItemList);
  If TreeView.Items.Count>0 Then
  begin
    TreeView.Items[0].Expand(False);
    For i :=0 to TreeView.Items[0].Count-1 do
      TreeView.Items[0].Item[i].Expand(False);
  end;
end;

//---------------------------------------------------------------------
// View Item Details
//---------------------------------------------------------------------
procedure OnTLItemChange(pItemData: PTTLItemData; RichEdit: TRichEdit);
  //Const cnstArial ='Arial';
  procedure SetSecondLine;
  begin
    RichEdit.Paragraph.FirstIndent := 10;
    //RichEdit.DefAttributes.Name := 'Courier New';
  end;
begin
  If pItemData=nil Then Exit;
  with RichEdit do
  begin
    Clear;
    Paragraph.FirstIndent := 0;
    DefAttributes.Style := [];
    //DefAttributes.Name := cnstArial;

    case pItemData.ItemType of
      itTypeLib :
      begin
        SelText := 'Type Library ';
        //SelAttributes.Name := cnstArial;
        SelTexBold(RichEdit, pItemData.Name);

        AddNewLine(RichEdit);
        SetSecondLine;
        SelTexItalic(RichEdit, 'HelpFile: ');
        SelText :=  (pItemData.TypeLibData.HelpFile)^;

        AddNewLine(RichEdit);
        SelTexItalic(RichEdit, 'GUID: ');
        SelText := GUIDToString(pItemData.TypeLibData.GUID);
      end;

      itEnums, itObjects:
      begin
        OnTLItemChange(pItemData.parentItemData, RichEdit);
      end;

      itEnum:
      begin
        SelText := 'Enum ';
        SelTexBold(RichEdit, pItemData.Name);
      end;

      itConst:
      begin
        SelText := 'Const ';
        SelTexBold(RichEdit, pItemData.Name);
        SelText := ' = '+ IntToStr(pItemData.iValue);
      end;

      itDispIntf:
      begin
        If stlVB in FTLListStyle then  SelText := 'Class '
        else SelText := 'DispInterface ';
        SelTexBold(RichEdit, pItemData.Name);

        If FDefObjPresent and
           IsEqualGUID(pItemData.intfGUID, FDefObjGUID) Then
        begin
          AddNewLine(RichEdit);
          SetSecondLine;
          SelTexItalic(RichEdit, 'Default object: ');
          SelText := FDefObjName;
        end;

        AddNewLine(RichEdit);
        SetSecondLine;
        SelTexItalic(RichEdit, 'GUID: ');
        SelText := GUIDToString(pItemData.intfGUID);
      end;

      itCoClass:
      begin
        If stlVB in FTLListStyle then  SelText := 'Class '
        else SelText := 'CoClass ';
        SelTexBold(RichEdit, pItemData.Name);
        If pItemData.CoClassData.bCreatable Then
        begin
          AddNewLine(RichEdit);
          SetSecondLine;
          SelText := 'Can Create';

          AddNewLine(RichEdit);
          SetSecondLine;
          SelTexItalic(RichEdit, 'ProgId: ');
          SelText := (pItemData.CoClassData.ProgId)^;
        end;
        AddNewLine(RichEdit);
        SetSecondLine;
        SelTexItalic(RichEdit, 'GUID: ');
        SelText := GUIDToString(pItemData.CoClassData.GUID);
      end;

      itPropFunc:
      begin
        pItemData.PropFuncData.SetFuncPrototype(pItemData, RichEdit);

        AddNewLine(RichEdit);
        SetSecondLine;
        SelTexItalic(RichEdit, 'MemId = ');
        SelText := IntToStr(pItemData.PropFuncData.FuncId);
      end;
    end;

    If Length(pItemData.Comment)>0 then
    begin
      AddNewLine(RichEdit);
      SetSecondLine;
      SelTexGreen(RichEdit, pItemData.Comment);
    end;
  end;
end;

Function IsItemCanDragAndDropped(pItemData: PTTLItemData): Boolean;
begin
  result := False;
  If (pItemData.ItemType in [itConst, itPropFunc]) or
     ((pItemData.ItemType=itCoClass) and (pItemData.CoClassData.bCreatable)) or
     (FDefObjPresent and (pItemData.ItemType=itDispIntf) and
      (IsEqualGUID(pItemData.intfGUID, FDefObjGUID)) ) Then
    result := True;
end;

function GetEnterStr(pItemData: PTTLItemData; IdScrLang: Integer;
                     Lines: TStrings; CaretXY: TPoint) : String;
begin
 case pItemData.ItemType of
   itConst:
     result := pItemData.parentItemData.Name+'.'+pItemData.Name;
   itCoClass:
     case IdScrLang of
     ScrLangVBScript:
       result := 'Set o'+ pItemData.Name+ ' = CreateObject("'+
               pItemData.CoClassData.ProgID^ +'")';
     end;
   itDispIntf:
       result := FDefObjName + '.';
   itPropFunc:
       result := pItemData.PropFuncData.GetEnterStr(pItemData, IdScrLang, Lines, CaretXY);
 end;
end;


end.
